Package: rcdk
Version: 3.8.2
Date: 2025-11-30
Title: Interface to the 'CDK' Libraries
Authors@R: c(
  person('Rajarshi', 'Guha', ,'rajarshi.guha@gmail.com', role=c('aut',"cph"),
    comment = c(ORCID = "0000-0001-7403-8819")),
  person('Zachary', 'Charlop-Powers', ,'zach.charlop.powers@gmail.com',role=c('cre'),
      comment = c(ORCID = "0000-0001-8816-4680")),
  person('Emma', 'Schymanski', ,'schymane@gmail.com', role=c('ctb'),
      comment = c(ORCID = "0000-0001-6868-8145")),
  person('Egon', 'Willighagen', ,'egon.willighagen@maastrichtuniversity.nl', role=c('ctb'),
      comment = c(ORCID = "0000-0001-7542-0286")))
Depends: rcdklibs (>= 2.9)
Imports: fingerprint, rJava, methods, png, iterators, itertools
Suggests: xtable, RUnit, knitr, rmarkdown, devtools
License: LGPL
URL: https://github.com/CDK-R/cdkr
LazyLoad: yes
LazyData: true
SystemRequirements: Java (>= 8)
BugReports: https://github.com/CDK-R/cdkr/issues
Description: Allows the user to access functionality in the
    'CDK', a Java framework for cheminformatics. This allows the user to load
    molecules, evaluate fingerprints, calculate molecular descriptors and so on.
    In addition, the 'CDK' API allows the user to view structures in 2D.
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-11-30 15:59:31 UTC; zcpowers
Author: Rajarshi Guha [aut, cph] (ORCID:
    <https://orcid.org/0000-0001-7403-8819>),
  Zachary Charlop-Powers [cre] (ORCID:
    <https://orcid.org/0000-0001-8816-4680>),
  Emma Schymanski [ctb] (ORCID: <https://orcid.org/0000-0001-6868-8145>),
  Egon Willighagen [ctb] (ORCID: <https://orcid.org/0000-0001-7542-0286>)
Maintainer: Zachary Charlop-Powers <zach.charlop.powers@gmail.com>
Repository: CRAN
Date/Publication: 2025-11-30 16:40:08 UTC
Built: R 4.6.0; ; 2026-04-26 00:38:40 UTC; unix

Package: rcdk
Version: 3.8.2
Date: 2025-11-30
Title: Interface to the 'CDK' Libraries
Authors@R: c(
  person('Rajarshi', 'Guha', ,'rajarshi.guha@gmail.com', role=c('aut',"cph"),
    comment = c(ORCID = "0000-0001-7403-8819")),
  person('Zachary', 'Charlop-Powers', ,'zach.charlop.powers@gmail.com',role=c('cre'),
      comment = c(ORCID = "0000-0001-8816-4680")),
  person('Emma', 'Schymanski', ,'schymane@gmail.com', role=c('ctb'),
      comment = c(ORCID = "0000-0001-6868-8145")),
  person('Egon', 'Willighagen', ,'egon.willighagen@maastrichtuniversity.nl', role=c('ctb'),
      comment = c(ORCID = "0000-0001-7542-0286")))
Depends: rcdklibs (>= 2.9)
Imports: fingerprint, rJava, methods, png, iterators, itertools
Suggests: xtable, RUnit, knitr, rmarkdown, devtools
License: LGPL
URL: https://github.com/CDK-R/cdkr
LazyLoad: yes
LazyData: true
SystemRequirements: Java (>= 8)
BugReports: https://github.com/CDK-R/cdkr/issues
Description: Allows the user to access functionality in the
    'CDK', a Java framework for cheminformatics. This allows the user to load
    molecules, evaluate fingerprints, calculate molecular descriptors and so on.
    In addition, the 'CDK' API allows the user to view structures in 2D.
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-11-30 15:59:31 UTC; zcpowers
Author: Rajarshi Guha [aut, cph] (ORCID:
    <https://orcid.org/0000-0001-7403-8819>),
  Zachary Charlop-Powers [cre] (ORCID:
    <https://orcid.org/0000-0001-8816-4680>),
  Emma Schymanski [ctb] (ORCID: <https://orcid.org/0000-0001-6868-8145>),
  Egon Willighagen [ctb] (ORCID: <https://orcid.org/0000-0001-7542-0286>)
Maintainer: Zachary Charlop-Powers <zach.charlop.powers@gmail.com>
Repository: CRAN
Date/Publication: 2025-11-30 16:40:08 UTC
Built: R 4.6.0; ; 2026-04-27 09:18:02 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(hasNext,iload.molecules)
export(cdk.version)
export(compare.isotope.pattern)
export(convert.implicit.to.explicit)
export(copy.image.to.clipboard)
export(do.aromaticity)
export(do.isotopes)
export(do.typing)
export(eval.atomic.desc)
export(eval.desc)
export(generate.2d.coordinates)
export(generate.formula)
export(generate.formula.iter)
export(get.adjacency.matrix)
export(get.alogp)
export(get.atom.count)
export(get.atom.index)
export(get.atomic.desc.names)
export(get.atomic.number)
export(get.atoms)
export(get.bond.order)
export(get.bonds)
export(get.charge)
export(get.chem.object.builder)
export(get.connected.atom)
export(get.connected.atoms)
export(get.connection.matrix)
export(get.depictor)
export(get.desc.categories)
export(get.desc.names)
export(get.element.types)
export(get.exact.mass)
export(get.exhaustive.fragments)
export(get.fingerprint)
export(get.formal.charge)
export(get.formula)
export(get.hydrogen.count)
export(get.isotopes.pattern)
export(get.largest.component)
export(get.mcs)
export(get.mol2formula)
export(get.murcko.fragments)
export(get.natural.mass)
export(get.point2d)
export(get.point3d)
export(get.properties)
export(get.property)
export(get.smiles)
export(get.smiles.parser)
export(get.stereo.types)
export(get.stereocenters)
export(get.symbol)
export(get.title)
export(get.total.charge)
export(get.total.formal.charge)
export(get.total.hydrogen.count)
export(get.tpsa)
export(get.volume)
export(get.xlogp)
export(iload.molecules)
export(is.aliphatic)
export(is.aromatic)
export(is.connected)
export(is.in.ring)
export(is.neutral)
export(isvalid.formula)
export(load.molecules)
export(matches)
export(parse.smiles)
export(remove.hydrogens)
export(remove.property)
export(set.atom.types)
export(set.charge.formula)
export(set.property)
export(set.title)
export(smiles.flavors)
export(view.image.2d)
export(view.molecule.2d)
export(view.table)
export(write.molecules)
import(fingerprint)
import(iterators)
import(itertools)
import(methods)
import(png)
import(rJava)
import(rcdklibs)
importFrom(itertools,hasNext)
importFrom(utils,download.file)
importFrom(utils,write.table)

Sources for the utility JAR file included with this package can be found at https://github.com/rajarshi/cdkr/tree/master/rcdkjar. They are not included with the package since they require a few large dependency JAR files, which would bloat the package

#library(devtools)
library(RUnit)
library(rcdk)
#load_all(".")
# .jinit(classpath=c("/Users/guha/src/cdkr/rcdk/inst/cont/rcdk.jar"))

test.atom.descriptors.alanine <- function() {
  alanine_file <- system.file("molfiles/alanine.sdf", package="rcdk")
  mols <- load.molecules(molfiles=alanine_file,  typing=TRUE, aromaticity=TRUE, verbose=TRUE)
  adn <- get.atomic.desc.names()
  checkTrue(length(adn) > 0)
  
  (lapply(mols, convert.implicit.to.explicit))
  dvals <- eval.atomic.desc(mols[[1]], adn[c(1,6)], verbose=TRUE)
  checkTrue(ncol(dvals) > 2)
  checkTrue(all(is.na(dvals[,2])))
}

test.atom.descriptors.rdf.glutamine <- function() {
  glutamine_file <- system.file("molfiles/glutamine.sdf", package="rcdk")
  mol <- load.molecules(molfiles=glutamine_file,
                         typing=TRUE, aromaticity = TRUE,
                         verbose=as.logical(TRUE))[[1]]
  glutamine_file <- mol
  adn <- get.atomic.desc.names()
  checkTrue(length(adn) > 0)
  
  convert.implicit.to.explicit(mol)
  dvals <- eval.atomic.desc(mol, adn[4:8], verbose=TRUE)
  checkTrue(ncol(dvals) > 2)
  checkTrue(all(dvals$gDr_1[11:20]==0))
  checkTrue(all(dvals$gDr_2[11:20]==0))
  checkTrue(all(dvals$gDr_3[11:20]==0))
  checkTrue(all(dvals$gDr_4[11:20]==0))
  checkTrue(all(dvals$gDr_5[11:20]==0))
  checkTrue(all(dvals$gDr_6[11:20]==0))
  checkTrue(all(dvals$gDr_7[11:20]==0))
  checkTrue(all(!is.na(dvals$gSr_1[18:19])))
  checkTrue(all(!is.na(dvals$gSr_2[18:19])))
  checkTrue(all(!is.na(dvals$gSr_3[18:19])))
  checkTrue(all(!is.na(dvals$gSr_4[18:19])))
  checkTrue(all(!is.na(dvals$gSr_5[18:19])))
  checkTrue(all(!is.na(dvals$gSr_6[18:19])))
  checkTrue(all(!is.na(dvals$gSr_7[18:19])))
}

test.atom.descriptor.conjugated.pi.system <- function() {
  glutamine_file <- system.file("molfiles/glutamine.sdf", package="rcdk")
  mol <- load.molecules(molfiles=glutamine_file,typing=TRUE, aromaticity = TRUE, verbose=TRUE)[[1]]
  adn <- get.atomic.desc.names()
  checkTrue(length(adn) > 0)
  
  convert.implicit.to.explicit(mol)
  dvals <- eval.atomic.desc(mol, adn[c(17,1)], verbose=TRUE)
  checkTrue(ncol(dvals) == 2)
  checkTrue(all(!dvals[,1]))
  
  alanine_file <- system.file("molfiles/alanine.sdf", package="rcdk")
  mol <- load.molecules(molfiles=alanine_file, typing=TRUE, aromaticity = TRUE, verbose=as.logical(TRUE))[[1]]
  adn <- get.atomic.desc.names()
  checkTrue(length(adn) > 0)
  
  convert.implicit.to.explicit(mol)
  dvals <- eval.atomic.desc(mol, adn[c(17,1)], verbose=TRUE)
  checkTrue(ncol(dvals) == 2)
  checkTrue(all(!dvals[,1]))
  
}

## ----eval=FALSE---------------------------------------------------------------
# 
# # Bioconductor Packages. Use BiocManager::install()
# #   Rdisop MetaboCoreUtils ChemmineR ChemmineOB enviPat
# 
# library(plyr)
# library(CHNOSZ)
# library(enviPat)
# library(MetaboCoreUtils)
# library(rcdk)
# library(ChemmineR)
# library(OrgMassSpecR)
# library(Rdisop)
# #library(ChemmineOB)
# 
# data(isotopes)
# 
# # original
# # https://github.com/CDK-R/cdkr/blob/master/rcdk/R/formula.R
# # get.formula <- function(mf, charge=0) {
# #
# #   manipulator <- get("mfManipulator", envir = .rcdk.GlobalEnv)
# #   if(!is.character(mf)) {
# #     stop("Must supply a Formula string");
# #   }else{
# #     dcob <- .cdkFormula.createChemObject()
# #     molecularformula <- .cdkFormula.createFormulaObject()
# #     molecularFormula <- .jcall(manipulator,
# #                                "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
# #                                "getMolecularFormula",
# #                                mf,
# #                                .jcast(molecularformula,.IMolecularFormula),
# #                                TRUE);
# #   }
# #
# #   D <- new(J("java/lang/Integer"), as.integer(charge))
# #   .jcall(molecularFormula,"V","setCharge",D);
# #   object <- .cdkFormula.createObject(.jcast(molecularFormula,.IMolecularFormula));
# #   return(object);
# # }
# 
# 
# mfManipulator    <- J("org/openscience/cdk/tools/manipulator/MolecularFormulaManipulator")
# silentchemobject <- J("org.openscience.cdk.silent.SilentChemObjectBuilder")
# 
# 
# #' Rewrite the formual object and directly access Java
# #'
# get.formula2 <- function(mf) {
# 
#   formula <- mfManipulator$getMolecularFormula(
#     "C2H3",
#     silentchemobject$getInstance())
# 
#   mfManipulator$getMass(formula)
# 
# }
# 
# #' Add type hints
# #'
# get.formula3 <- function(mf) {
#   builderinstance <- .jcall(
#       silentchemobject,
#      "Lorg/openscience/cdk/interfaces/IChemObjectBuilder;",
#      "getInstance")
# 
#   formula  <- .jcall(
#       mfManipulator,
#      "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
#      "getMolecularFormula",
#       mf,
#       builderinstance);
# 
#   mfManipulator$getMass(formula)
# 
# }
# 
# 
# #' Add type hints
# #'
# get.formula4 <- function(mf) {
#   builderinstance <- .jcall(
#       silentchemobject,
#       "Lorg/openscience/cdk/interfaces/IChemObjectBuilder;",
#       "getInstance")
# 
#   formula  <- .jcall(
#       mfManipulator,
#      "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
#      "getMolecularFormula",
#      mf,
#      builderinstance);
# 
#   .jcall(
#       mfManipulator,
#       "D",
#       "getMass",
#      formula)
# }
# 
# 
# 
# benchmark <- microbenchmark::microbenchmark(
#   MetaboCoreUtils = MetaboCoreUtils::calculateMass("C2H6O"),
#   rcdk = rcdk::get.formula("C2H6O", charge = 0)@mass,
#   rcdk2 = get.formula2("C2H6O"),
#   rcdk3 = get.formula3("C2H6O"),
#   rcdk4 = get.formula4("C2H6O"),
#   Rdisop = Rdisop::getMolecule("C2H6O")$exactmass,
#   ChemmineR = ChemmineR::exactMassOB(ChemmineR::smiles2sdf("CCO")),
#   OrgMassSpecR = OrgMassSpecR::MonoisotopicMass(formula = OrgMassSpecR::ListFormula("C2H6O)"), charge = 0),
# 
#   CHNOSZ = CHNOSZ::mass("C2H6O"),
#   enviPat = enviPat::isopattern(isotopes, "C2H6O", charge=FALSE, verbose=FALSE)[[1]][1,1]
#   , times=1000L)
# 
# 
# masses <- c(
#   MetaboCoreUtils=MetaboCoreUtils::calculateMass("C2H6O"),
#   rcdk=rcdk::get.formula("C2H6O", charge = 0)@mass,
#   Rdisop=Rdisop::getMolecule("C2H6O")$exactmass,
#   #ChemmineR=ChemmineR::exactMassOB(ChemmineR::smiles2sdf("CCO")),
#   OrgMassSpecR=OrgMassSpecR::MonoisotopicMass(formula = OrgMassSpecR::ListFormula("C2H6O)"), charge = 0),
#   CHNOSZ=CHNOSZ::mass("C2H6O"),
#   enviPat=enviPat::isopattern(isotopes, "C2H6O", charge=FALSE, verbose=FALSE)[[1]][1,1]
# )
# 
# options(digits=10)
# t(t(sort(masses)))
# summary(benchmark)[order(summary(benchmark)[,"median"]) , ]
# clipr::write_clip(as.data.frame(summary(benchmark)[order(summary(benchmark)[,"median"]) , ] ))

test.bond.order <- function()
{
  m <- parse.smiles('CCN')[[1]]
  b <- get.bonds(m)[[1]]
  checkTrue(b$getOrder() == get.bond.order('single'))
  b$setOrder(get.bond.order("double"))
  checkTrue(b$getOrder() == get.bond.order('double'))
}

########################################################
##  set a cdkFormula function   
.IMolecularFormula <- "org/openscience/cdk/interfaces/IMolecularFormula"

#' @name cdkFormula-class
#' @title  Class cdkFormula, ac class for handling molecular formula
#' @description This class handles molecular formulae. It provides extra 
#'  information such as the IMolecularFormula Java object, elements contained 
#'  and number of them.
#' @section Objects from the Class:
#'   Objects can be created using new constructor and filled with a specific 
#'   mass and window accuracy
#' @author Miguel Rojas-Cherto (\email{miguelrojasch@@yahoo.es})
#' @references A parallel effort to expand the Chemistry Development 
#'   Kit: \url{http://cdk.sourceforge.net}
#' @seealso  
#' \code{\link{get.formula}}
#' \code{\link{set.charge.formula}}
#' \code{\link{get.isotopes.pattern}}
#' \code{\link{isvalid.formula}}
#' @docType class
#' @aliases cdkFormula-class
#' @aliases show,cdkFormula-method
setClass("cdkFormula", representation(mass = "numeric",
                                      objectJ = "jobjRef",
                                      string = "character",
                                      charge = "numeric",
                                      isotopes = "matrix"),
         prototype(mass = 0,
                   objectJ = NULL,
                   string = character(0),
                   charge = 0,
                   isotopes = matrix(nrow = 0, ncol = 0))
         )

#' get.formula
#' 
#' obtain molecular formula from formula string
#' 
#' @param mf Required. Molecular formula
#' @param charge Optional. Default \code{0}
#' @export
get.formula <- function(mf, charge=0) {
    
    manipulator <- get("mfManipulator", envir = .rcdk.GlobalEnv)
    if(!is.character(mf)) {
        stop("Must supply a Formula string");
    }else{
        dcob <- .cdkFormula.createChemObject()
        molecularformula <- .cdkFormula.createFormulaObject()
        molecularFormula <- .jcall(manipulator,
                                   "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
                                   "getMolecularFormula",
                                   mf,
                                   .jcast(molecularformula,.IMolecularFormula),
                                   TRUE);
    }
    
    D <- new(J("java/lang/Integer"), as.integer(charge))
    .jcall(molecularFormula,"V","setCharge",D);
    object <- .cdkFormula.createObject(.jcast(molecularFormula,.IMolecularFormula));
    return(object);
}

setMethod("show", "cdkFormula",
          function(object) {
              cat('cdkFormula: ',object@string,
                  ', mass = ',object@mass, ', charge = ',
                  object@charge,  '\n')
          })
#' 
#' get.mol2formula
#' 
#' 
#' @param molecule The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @param charge Optional. Default \code{0}
#' @export
get.mol2formula <- function(molecule, charge=0) {
    if(attr(molecule, "jclass") != "org/openscience/cdk/interfaces/IAtomContainer") {
        stop("Must supply an IAtomContainerobject")
    }
    
    formulaJ <- .jcall('org/openscience/cdk/tools/manipulator/MolecularFormulaManipulator',
                       "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
                       "getMolecularFormula",
                       molecule, use.true.class=FALSE);
    formulaJ <- .jcast(formulaJ,"org/openscience/cdk/interfaces/IMolecularFormula")
    
    ## needs that all isotopes contain the properties

    string <- .cdkFormula.getString(formulaJ)
    objectF <- .cdkFormula.createFormulaObject()
    moleculaJT <- .jcall('org/openscience/cdk/tools/manipulator/MolecularFormulaManipulator',
                         "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
                         "getMolecularFormula",string,
                         .jcast(objectF,"org/openscience/cdk/interfaces/IMolecularFormula"),TRUE);
    
    Do <- new(J("java/lang/Integer"), as.integer(charge))
    .jcall(moleculaJT,"V","setCharge",Do);	   

    formula <- .cdkFormula.createObject(.jcast(moleculaJT,.IMolecularFormula))
    return(formula);
}
#' 
#' set.charge.formula
#' 
#' Set the charge to a cdkFormula function.
#' 
#' @param formula Required. Molecular formula
#' @param charge Optional. Default \code{-1}
#' @export
set.charge.formula <- function(formula,charge = -1) {
    if (class(formula) != "cdkFormula")
        stop("Supplied object should be a cdkFormula Class")
    
    molecularFormula <- formula@objectJ;
    
    D <- new(J("java/lang/Integer"), as.integer(charge))
    .jcall(molecularFormula,"V","setCharge",D);
    
    formula@objectJ <- molecularFormula;
    formula@charge <- charge;
    
    return(formula)
}

#' 
#' isvalid.formula
#' 
#' Validate a cdkFormula.
#' 
#' @param formula Required. A CDK Formula
#' @param rule Optional. Default \code{rule=c("nitrogen","RDBE")}
#' @export
isvalid.formula <- function(formula,rule=c("nitrogen","RDBE")){
    
    if (class(formula) != "cdkFormula")
        stop("Supplied object should be a cdkFormula Class")
    
    molecularFormula <- formula@objectJ;

    nRule <- get("nRule", envir = .rcdk.GlobalEnv)
    rdbeRule <- get("rdbeRule", envir = .rcdk.GlobalEnv)

    for(i in 1:length(rule)){
        ##Nitrogen Rule
        if(rule[i] == "nitrogen"){
            valid <- .jcall(nRule,"D","validate",molecularFormula);
            if(valid != 1.0){
                return (FALSE)
            }
        }	  
        ##RDBE Rule
        if(rule[i] == "RDBE"){
            valid <- .jcall(rdbeRule,"D","validate",molecularFormula);
            if(valid != 1.0){
                return (FALSE)
            }
            else return(TRUE);
        }
    }
    return(TRUE);
}
#' 
#' get.isotopes.pattern
#' 
#' Generate the isotope pattern given a formula class
#'
#' @param formula Required. A CDK molecule formula
#' @param minAbund Optional. Default \code{0.1}
#' @export
get.isotopes.pattern <- function(formula,minAbund=0.1){
    
    if (class(formula) != "cdkFormula")
        stop("Supplied object should be a cdkFormula Class")
    
    molecularFormula <- formula@objectJ;
    
    isoGen <- .jnew("org/openscience/cdk/formula/IsotopePatternGenerator",as.double(minAbund));
    isoPattern <- .jcall(isoGen,
                         "Lorg/openscience/cdk/formula/IsotopePattern;",
                         "getIsotopes",molecularFormula);
    numIP <- .jcall(isoPattern,"I","getNumberOfIsotopes");
    
    ## create a matrix adding the mass and abundance of the isotope pattern
    iso.col <- c("mass","abund");
    
    massVSabun <- matrix(ncol=2,nrow=numIP);
    colnames(massVSabun)<-iso.col;
    for (i in 1:numIP) {
        isoContainer <- .jcall(isoPattern,"Lorg/openscience/cdk/formula/IsotopeContainer;","getIsotope",as.integer(i-1));
        massVSabun[i,1] <- .jcall(isoContainer,"D","getMass");
        massVSabun[i,2] <- .jcall(isoContainer,"D","getIntensity");
    }
    return (massVSabun);
}
#' 
#' generate.formula.iter
#' 
#' Generate a list of possible formula objects given a mass and a mass tolerance.
#' 
#' @param mass Required. Mass.
#' @param window Optional. Default \code{0.01}
#' @param elements Optional. Default \code{
#'                  list(c('C', 0,50), c('H', 0,50), 
#'                  c('N', 0,50), c('O', 0,50), c('S', 0,50))}
#' @param validation Optional. Default \code{FALSE}
#' @param charge Optional. Default \code{FALSE}
#' @param as.string Optional. Default \code{FALSE}
#' @export
generate.formula.iter <- function(mass, window = 0.01,
                                  elements = list(
                                      c('C', 0,50),
                                      c('H', 0,50),
                                      c('N', 0,50),
                                      c('O', 0,50),
                                      c('S', 0,50)),
                                  validation = FALSE,
                                  charge = 0.0,
                                  as.string=TRUE) {

    mfRange <- .jnew("org/openscience/cdk/formula/MolecularFormulaRange");
    ifac <- .jcall("org/openscience/cdk/config/Isotopes",
                   "Lorg/openscience/cdk/config/Isotopes;",
                   "getInstance");
    for (i in 1:length(elements)) {
      
      ## If the element list is 3, then we have sym, min, max
      ## otherwise it should be sym, min, max, massNumber
      if (length(elements[[i]]) == 3) {
        isotope <- .jcall(ifac,
                          "Lorg/openscience/cdk/interfaces/IIsotope;",
                          "getMajorIsotope",
                          as.character( elements[[i]][1] ), use.true.class=FALSE)
      } else if(length(elements[[i]]) == 4) {
        isotope <- .jcall(ifac,
                          "Lorg/openscience/cdk/interfaces/IIsotope;",
                          "getIsotope",
                          as.character( elements[[i]][1] ),
                          as.integer( elements[[i]][4] ),
                          use.true.class=FALSE)
        if (is.null(isotope)) stop(sprintf("Invalid mass number specified for element %s",elements[[i]][1]))
      } else stop("Elements must be 3-tuples or 4-tuples")

      .jcall(mfRange,
             returnSig="V",
             method="addIsotope",
             isotope,
             as.integer( elements[[i]][2] ),
             as.integer( elements[[i]][3] )
             )
      
    }

    ## Construct range strings
    ## rstrs <- sapply(names(elements), function(x) paste0(c(x, elements[[x]][1], elements[[x]][2]), sep='', collapse=' '))
    ## if (length(rstrs) == 0)
    ##     warning("The element specification resulted in a 0 length vector. This is worrisome")
    
    ## ## Get MF range object
    ## mfRange <- .jcall("org/guha/rcdk/util/Misc",
    ##                   "Lorg/openscience/cdk/formula/MolecularFormulaRange;",
    ##                   "getMFRange",
    ##                   .jarray(rstrs))
    
    ## construct generator
    mfgen <- .jnew("org/openscience/cdk/formula/RoundRobinFormulaGenerator",
                   get("dcob", envir = .rcdk.GlobalEnv),
                   as.double(mass-window),
                   as.double(mass+window),
                   mfRange)

    hasNext <- NA
    formula <- NA
    
    ## hasNx <- function() {
    ##   hasNext <<- .jcall(mfgen, "Lorg/openscience/cdk/interfaces/IMolecularFormula;", "getNextFormula")
    ##   if (is.jnull(hasNext)) {
    ##     ## nothing to do
    ##     formula <<- NA
    ##   } else formula <<- hasNext
    ##   return(!is.jnull(hasNext))
    ## }
    
    nextEl <- function() {
        hasNext <<- NA
        formula <- .jcall(mfgen, "Lorg/openscience/cdk/interfaces/IMolecularFormula;", "getNextFormula")
        if (is.jnull(formula)) stop("StopIteration")
        if (!as.string) {
            return(formula)
        } else {
            return(.jcall("org/openscience/cdk/tools/manipulator/MolecularFormulaManipulator",
                          "S", "getString", formula, FALSE, TRUE))
        }
    }

    ##obj <- list(nextElem = nextEl, hasNext = hasNx)
    obj <- list(nextElem = nextEl)  
    class(obj) <- c("generate.formula2", "abstractiter", "iter")
    return(obj)
}
#' generate.formula
#' 
#' @param mass Required. Mass.
#' @param window Optional. Default \code{0.01}
#' @param elements Optional. Default \code{
#'                  list(c('C', 0,50), c('H', 0,50), 
#'                  c('N', 0,50), c('O', 0,50), c('S', 0,50))}
#' @param validation Optional. Default \code{FALSE}
#' @param charge Optional. Default \code{FALSE}
#' @export
generate.formula <- function(mass, 
                             window=0.01, 
                             elements=list(c("C",0,50),c("H",0,50),c("N",0,50),c("O",0,50),c("S",0,50)), 
                             validation=FALSE,
                             charge=0.0){
    
    builder <- .cdkFormula.createChemObject();
    mfTool <- .jnew("org/openscience/cdk/formula/MassToFormulaTool",builder);
    ruleList <-.jcast(.jcall("org/guha/rcdk/formula/FormulaTools",
                             "Ljava/util/List;",
                             "createList"), "java/util/List")
    
    ## TOLERANCE RULE
    toleranceRule <- .jnew("org/openscience/cdk/formula/rules/ToleranceRangeRule");
    ruleG <- .jcast(toleranceRule, "org/openscience/cdk/formula/rules/IRule");
    D <- new(J("java/lang/Double"), window)
    paramsA <- .jarray(list(D,D))
    paramsB <- .jcastToArray(paramsA)
    .jcall(ruleG,"V","setParameters",paramsB);
    ruleList <-.jcall("org/guha/rcdk/formula/FormulaTools", "Ljava/util/List;", "addTo",ruleList,ruleG)
    
    
    ## ELEMENTS RULE
    elementRule <- .jnew("org/openscience/cdk/formula/rules/ElementRule");
    ruleG <- .jcast(elementRule, "org/openscience/cdk/formula/rules/IRule");
    
    chemObject <- .cdkFormula.createChemObject();
    range <- .jnew("org/openscience/cdk/formula/MolecularFormulaRange");
    ifac <- .jcall("org/openscience/cdk/config/Isotopes",
                   "Lorg/openscience/cdk/config/Isotopes;",
                   "getInstance");

    for (i in 1:length(elements)) {

        ## If the element list is 3, then we have sym, min, max
        ## otherwise it should be sym, min, max, massNumber
        if (length(elements[[i]]) == 3) {
            isotope <- .jcall(ifac,
                              "Lorg/openscience/cdk/interfaces/IIsotope;",
                              "getMajorIsotope",
                              as.character( elements[[i]][1] ), use.true.class=FALSE)
        } else if(length(elements[[i]]) == 4) {
            isotope <- .jcall(ifac,
                              "Lorg/openscience/cdk/interfaces/IIsotope;",
                              "getIsotope",
                              as.character( elements[[i]][1] ),
                              as.integer( elements[[i]][4] ),
                              use.true.class=FALSE)
            if (is.null(isotope)) stop(sprintf("Invalid mass number specified for element %s",elements[[i]][1]))
        } else stop("Elements must be 3-tuples or 4-tuples")

        .jcall(range,
               returnSig="V",
               method="addIsotope",
               isotope,
               as.integer( elements[[i]][2] ),
               as.integer( elements[[i]][3] )
               )

    }
    
    paramsA <- .jarray(list(range))
    paramsB <- .jcastToArray(paramsA)
    .jcall(ruleG,"V","setParameters",paramsB);
    
    ruleList <-.jcall("org/guha/rcdk/formula/FormulaTools",
                      "Ljava/util/List;", "addTo",
                      .jcast(ruleList,"java/util/List"),
                      ruleG)
    
    ## Setting the rules int FormulaTools
    .jcall(mfTool,"V","setRestrictions",.jcast(ruleList,"java/util/List"));
    
    mfSet <- .jcall(mfTool,"Lorg/openscience/cdk/interfaces/IMolecularFormulaSet;",
                    "generate",mass);
    if (is.null(mfSet))
        return(list())

    sizeSet <- .jcall(mfSet,"I","size");
    ecList <- list();
    count = 1;
    
    for (i in 1:sizeSet) {
        mf <- .jcall(mfSet,
                     "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
                     "getMolecularFormula",
                     as.integer(i-1));

        .jcall(mf,"V","setCharge",new(J("java/lang/Integer"), as.integer(charge)));
        object <- .cdkFormula.createObject(.jcast(mf,
                                                  "org/openscience/cdk/interfaces/IMolecularFormula"));
        
        isValid = TRUE;
        if(validation==TRUE)
            isValid = isvalid.formula(object);
        
        if(isValid==TRUE){ ## if it's true add to the list
            ecList[[count]] = object;
            count = count+1;
        }
    }
    ecList
}


#############################################################
##  Intern functions: Creating object
#############################################################

.cdkFormula.createChemObject <- function(){
    get("dcob", envir = .rcdk.GlobalEnv)
                                        #  dcob <- .jcall("org/openscience/cdk/DefaultChemObjectBuilder",
                                        #                 "Lorg/openscience/cdk/interfaces/IChemObjectBuilder;",
                                        #                 "getInstance")
                                        #  dcob
}

.cdkFormula.createFormulaObject <- function(){
    ##   dcob <- .cdkFormula.createChemObject()
    ##   klass <- J("org.openscience.cdk.interfaces.IMolecularFormula")$class
    ##   cfob <- .jcall(dcob,
    ##                  "Lorg/openscience/cdk/interfaces/ICDKObject;",
    ##                  "newInstance",
    ##                  klass
    ##                  );  
    ##   cfob
    .jcast(.jnew("org/openscience/cdk/formula/MolecularFormula"),
           "org/openscience/cdk/interfaces/IMolecularFormula")
}

#############################################################
## extract the molecular formula string form the java object
#############################################################
.cdkFormula.getString <- function(molecularFormula) {
    
    if (attr(molecularFormula, "jclass") != 'org/openscience/cdk/interfaces/IMolecularFormula') {
        stop("Supplied object should be a Java reference to an IMolecularFormula")
    }
    formula <- .jcall('org/openscience/cdk/tools/manipulator/MolecularFormulaManipulator',
                      'S', 'getString', molecularFormula)
}

#############################################################
## create a formula class from the molecularFormula java object
#############################################################
.cdkFormula.createObject <- function(molecularformula){
    
    object <-new("cdkFormula")
    
    object@objectJ <- molecularformula;
    iterable <- .jcall(molecularformula,"Ljava/lang/Iterable;","isotopes"); 
    isoIter <- .jcall(iterable,"Ljava/util/Iterator;","iterator");
    size <- .jcall(molecularformula,"I","getIsotopeCount");
    
    isotopeList = matrix(ncol=3,nrow=size);
    colnames(isotopeList) <- c("isoto","number","mass");
    
    for(i in 1:size){
        isotope = .jcast(.jcall(isoIter,"Ljava/lang/Object;","next"), "org/openscience/cdk/interfaces/IIsotope");
        isotopeList[i,1] <- .jcall(isotope,"S","getSymbol");
        isotopeList[i,2] <- .jcall(molecularformula,"I","getIsotopeCount",isotope);
        #massNum          <- .jcall(isotope,"Ljava/lang/Double;","getMassNumber");
        massNum          <-  isotope$getMassNumber()
        #exactMass        <- .jcall(isotope,"Ljava/lang/Double;","getExactMass");
        exactMass        <- isotope$getExactMass()
        #print(exactMass)
        isotopes         <- J("org/openscience/cdk/config/Isotopes")
        
        if (is.null(massNum)) {
         
          isos         <- isotopes$getInstance()
          majorIsotope <- isos$getMajorIsotope(isotope$getSymbol())
          
          if (!is.null(majorIsotope)) {
            
            exactMass <- majorIsotope$getExactMass()
          }
        } else {
          
          if (is.null(exactMass)) {
            isos    <- isotopes$getInstance()
            temp    <- isos$getIsotope(isotope$getSymbol(), massNum);
            
            if (!is.null(temp)) {
              exactMass <- temp$getExactMass()
            }
          }
        }
        
        #ch <- .jcall(isotope,"Ljava/lang/Double;","getExactMass");
        isotopeList[i,3] <- exactMass
        
    }
    
    object@string <- .cdkFormula.getString(molecularformula);
    manipulator <- get("mfManipulator", envir = .rcdk.GlobalEnv)
    cMass <- .jcall(manipulator,"D","getTotalExactMass",molecularformula);
    object@mass <- cMass;
    chargeDO <- .jcall(molecularformula,"Ljava/lang/Integer;","getCharge");
    charge <- .jcall(chargeDO,"D","doubleValue");
    object <- set.charge.formula(object,charge)
    object@isotopes <- isotopeList;
    
    return(object);
}

#' Construct an isotope pattern similarity calculator.
#'
#' A method that returns an instance of the CDK \code{IsotopePatternSimilarity}
#' class which can be used to compute similarity scores between pairs of
#' isotope abundance patterns.
#'
#' @param tol The tolerance
#' @return A \code{jobjRef} corresponding to an instance of \code{IsotopePatternSimilarity}
#' @seealso \code{\link{compare.isotope.pattern}}
#' @references \url{http://cdk.github.io/cdk/1.5/docs/api/org/openscience/cdk/formula/IsotopePatternSimilarity.html}
#' @author Miguel Rojas Cherto
get.isotope.pattern.similarity <- function(tol = NULL) {
    ips <- .jnew("org/openscience/cdk/formula/IsotopePatternSimilarity")
    if (!is.null(tol)) ips$seTolerance(tol)
    return(ips)
}

#' Construct an isotope pattern generator.
#'
#' Constructs an instance of the CDK \code{IsotopePatternGenerator}, with an optional
#' minimum abundance specified. This object can be used to generate all combinatorial
#' chemical isotopes given a structure.
#'
#' @param minAbundance The minimum abundance
#' @return A \code{jobjRef} corresponding to an instance of \code{IsotopePatternGenerator}
#' @references \url{http://cdk.github.io/cdk/1.5/docs/api/org/openscience/cdk/formula/IsotopePatternGenerator.html}
#' @author Miguel Rojas Cherto
get.isotope.pattern.generator <- function(minAbundance = NULL) {
    if (is.null(minAbundance))
        .jnew("org/openscience/cdk/formula/IsotopePatternGenerator")
    else
        .jnew("org/openscience/cdk/formula/IsotopePatternGenerator", as.double(minAbundance))
}

#' Compare isotope patterns.
#'
#' Computes a similarity score between two different isotope abundance patterns.
#'
#' @param iso1 The first isotope pattern, which should be a \code{jobjRef} corresponding to the \code{IsotopePattern} class
#' @param iso2 The second isotope pattern, which should be a \code{jobjRef} corresponding to the \code{IsotopePattern} class
#' @param ips An instance of the \code{IsotopePatternSimilarity} class. if \code{NULL} one will be constructed automatically
#'
#' @return A numeric value between 0 and 1 indicating the similarity between the two patterns
#' @seealso \code{\link{get.isotope.pattern.similarity}}
#' @export
#' @references \url{http://cdk.github.io/cdk/2.3/docs/api/org/openscience/cdk/formula/IsotopePatternSimilarity.html}
#' @author Miguel Rojas Cherto
compare.isotope.pattern <- function(iso1, iso2, ips = NULL) {
    cls <- unique(c(class(iso1), class(iso2)))
    if (length(cls) != 1) stop("Must supply Java objects of class IsotopePattern")
    if (cls != 'jobjRef') stop("Must supply Java objects of class IsotopePattern")
    if(attr(iso1, "jclass") != "org/openscience/cdk/formula/IsotopePattern" ||
       attr(iso2, "jclass") != "org/openscience/cdk/formula/IsotopePattern") {
        stop("Must supply an IsotopePattern")
    }
    if (is.null(ips)) ips <- get.isotope.pattern.similarity()
    return(ips$compare(iso1, iso2))
}

.packageName <- "rcdk"

## draw.molecule <- function(molecule = NA) {
##   if (is.na(molecule)) {
##     editor <- .jnew("org/guha/rcdk/draw/Get2DStructureFromJCP")
##   } else {
##     if (attr(molecule, "jclass") != 'org/openscience/cdk/interfaces/IAtomContainer') {
##       stop("Supplied object should be a Java reference to an IAtomContainer")
##     }
##     editor <- .jnew("org/guha/rcdk/draw/Get2DStructureFromJCP", molecule)
##   }
##   .jcall(editor, "V", "showWindow")
##   molecule <- .jcall(editor, "[Lorg/openscience/cdk/interfaces/IAtomContainer;", "getMolecules")
##   return(molecule)
## }

## script should be a valid Jmol script string
## view.molecule.3d <- function(molecule, ncol = 4, cellx = 200, celly = 200, script = NA) {

##   if (class(molecule) != 'character' &&
##       class(molecule) != 'list' &&
##       class(molecule) != 'jobjRef') {
##     stop("Must supply a filename, single molecule object or list of molecule objects")
##   }

##   if (class(molecule) == 'character') {
##     molecule <- load.molecules(molecule)
##     if (length(molecule) == 1) molecule <- molecule[[1]]
##   }

##   if (class(molecule) != 'list') { ## single molecule
##     if (attr(molecule, "jclass") != 'org/openscience/cdk/interfaces/IAtomContainer') {
##       stop("Supplied object should be a Java reference to an IAtomContainer")
##     }
##     viewer <- .jnew("org/guha/rcdk/view/ViewMolecule3D", molecule)
##     .jcall(viewer, 'V', 'show')
##     if (!is.na(script)) {
##       .jcall(viewer, "V", "setScript", script)
##     }
##   } else { ## script is not run for the grid case
##     array <- .jarray(molecule, contents.class="org/openscience/cdk/interfaces/IAtomContainer")
##     v3d <- .jnew("org/guha/rcdk/view/ViewMolecule3DTable", array,
##                  as.integer(ncol), as.integer(cellx), as.integer(celly))
##     .jcall(v3d, 'V', 'show')    
##   }
## }

#' get.depictor
#' 
#' return an RcdkDepictor.
#' 
#' @param width Default. \code{200}
#' @param height Default. \code{200}
#' @param zoom Default. \code{1.3}
#' @param style Default. \code{cow}
#' @param annotate Default. \code{off}
#' @param abbr Default. \code{on}
#' @param suppressh Default. \code{TRUE}
#' @param showTitle Default. \code{FALSE}
#' @param smaLimit Default. \code{100}
#' @param sma Default. \code{NULL}
#' 
#' @export
#' 
get.depictor <- function(width = 200, height = 200, zoom = 1.3, style = "cow", annotate = "off", abbr = "on",
                         suppressh = TRUE, showTitle = FALSE, smaLimit = 100, sma = NULL) {
  if (is.null(sma)) sma <- ""
  return(.jnew("org/guha/rcdk/view/RcdkDepictor",
               as.integer(width),
               as.integer(height),
               as.double(zoom),
               as.character(style),
               as.character(annotate),
               as.character(abbr),
               as.logical(suppressh),
               as.logical(showTitle),
               as.integer(smaLimit),
               as.character(sma)))
}

#' view.molecule.2d
#' 
#' Create a 2D depiction of a molecule. If there are more than
#' one molecules supplied, return a grid woth \code{ncol} columns,.
#' 
#' @param molecule The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @param ncol Default \code{4}
#' @param width Default \code{200}
#' @param height Default \code{200}
#' @param depictor Default \code{NULL}
#' 
#' @importFrom utils write.table
#' @export 
view.molecule.2d <- function(molecule, ncol = 4, width = 200, height = 200, depictor = NULL) {
  
  if (class(molecule) != 'character' &&
      class(molecule) != 'list' &&
      class(molecule) != 'jobjRef') {
    stop("Must supply a filename, single molecule object or list of molecule objects")
  }

  if (is.null(depictor))
    depictor <- get.depictor()
  
  ## in case we're on OS X we need to prep some stuff
  ## so we can shell out 
  is.osx <- Sys.info()[1] == 'Darwin'
  jarfile <- NULL
  rcdklibs <- NULL
  if (is.osx) { 
    jarfile <- system.file(package='rcdk')
    rcdklibs <- system.file(package='rcdklibs')
  }

  ## if we got a file name, lets load all the molecules
  if (class(molecule) == 'character') {
    molecule <- load.molecules(molecule)
    if (length(molecule) == 1) molecule <- molecule[[1]]
  }

  if (class(molecule) != 'list') { ## single molecule
    if (attr(molecule, "jclass") != 'org/openscience/cdk/interfaces/IAtomContainer') {
      stop("Supplied object should be a Java reference to an IAtomContainer")
    }

    if (is.osx) {
      smi <- get.smiles(molecule)
      if (depictor$getSma() == "")
        sma = "\"\""
      else
        sma = depictor$getSma()

      cmd <- sprintf('java -cp \"%s/cont/*:%s/cont/rcdk.jar\" org.guha.rcdk.app.OSXHelper viewMolecule2D "%s" %d %d %f %s %s %s %s %s %d "%s" &', rcdklibs, jarfile, smi,
                     depictor$getWidth(), depictor$getHeight(),
                     depictor$getZoom(), depictor$getStyle(), depictor$getAnnotate(),
                     depictor$getAbbr(), depictor$isSuppressh(), depictor$isShowTitle(),
                     depictor$getSmaLimit(), sma)
      return(system(cmd))
    } else {
      v2d <- .jnew("org/guha/rcdk/view/ViewMolecule2D", molecule, as.integer(width), as.integer(height), depictor)
      ret <- .jcall(v2d, "V", "draw")
    }
  } else { ## multiple molecules
    if (is.osx) {
      ## write out mols as a temp smi file
      smi <- sapply(molecule, get.smiles, flavor = smiles.flavors(c('Isomeric')))
      titles <- sapply(molecule, get.title)
      tmp <- data.frame(smi, titles)
      tf <- tempfile(pattern='rcdkv-', fileext='.smi')
      write.table(tmp, file=tf, sep='\t', row.names=FALSE, col.names=FALSE, quote=FALSE)

      if (depictor$getSma() == "") {
        sma = "\"\""
      } else {
        sma = depictor$getSma()
      }
      cmd <- sprintf('java -cp \"%s/cont/*:%s/cont/rcdk.jar\" org.guha.rcdk.app.OSXHelper viewMolecule2Dtable "%s" %d %d %f %s %s %s %s %s %d "%s" %d &',
                     rcdklibs, jarfile, tf,
                     depictor$getWidth(), depictor$getHeight(),
                     depictor$getZoom(), depictor$getStyle(), depictor$getAnnotate(),
                     depictor$getAbbr(), depictor$isSuppressh(), depictor$isShowTitle(),
                     depictor$getSmaLimit(), sma,
                     ncol)
      return(system(cmd))
    } else {
      array <- .jarray(molecule, contents.class="org/openscience/cdk/interfaces/IAtomContainer")
      v2d <- .jnew("org/guha/rcdk/view/ViewMolecule2DTable", array,
                   as.integer(ncol), depictor)
    }
  }
}

view.table <- function(molecules, dat, depictor = NULL) {

  if (!is.list(molecules)) {
    stop("Must provide a list of molecule objects")
  }

  if (!is.matrix(dat) && !is.data.frame(dat)) {
    stop("datatable must be a matrix or data.frame")
  }

  if (length(molecules) != nrow(dat)) {
    stop("The number of rows in datatable must be the same as the number of molecules")
  }

  if (is.null(depictor))
    depictor <- get.depictor()
  
  if (is.null(names(dat))) cnames <- c('Molecule', paste('V',1:ncol(dat)), sep='')
  else cnames <- c('Molecule', names(dat))

  ## we need to convert the R vectors to Java arrays
  ## and the datatable data.frame to an Object[][]
  molecules <- .jarray(molecules, "org/openscience/cdk/interfaces/IAtomContainer")
  carr <- .jarray(cnames)

  rows <- list()
  for (i in 1:nrow(dat)) {
    row <- list()
    
    ## for a given row we have to construct a Object[] and add
    ## it to our list
    for (j in 1:ncol(dat)) {
      if (is.numeric(dat[i,j])) {
        row[j] <- .jnew("java/lang/Double", dat[i,j])
      }
      else if (is.character(dat[i,j]) || is.factor(dat[i,j]) || is.logical(dat[i,j])) {
        row[j] <- .jnew("java/lang/String", as.character(dat[i,j]))
      }
    }
    rows[i] <- .jarray(row, "java/lang/Object")
  }

  ## now make our object table
  xval.arr <- .jarray(rows, "[Ljava/lang/Object;")
  obj <- .jnew("org/guha/rcdk/view/ViewMolecule2DDataTable",
               molecules, carr, xval.arr, depictor)
  .jcall(obj, "V", "display")
}


#' view.image.2d
#' 
#' @param molecule The molecule to display Should be a `jobjRef` representing an `IAtomContainer`
#' @param depictor Default \code{NULL}
#' 
#' @export
view.image.2d <- function(molecule, depictor = NULL) {
  if (is.null(depictor))
    depictor <- get.depictor()
  if (attr(molecule,"jclass") != "org/openscience/cdk/interfaces/IAtomContainer")
    stop("Must supply an IAtomContainer object")
  mi <- .jnew("org/guha/rcdk/view/MoleculeImage", molecule, depictor)
  bytes <- .jcall(mi, "[B", "getBytes", as.integer(depictor$getWidth()), as.integer(depictor$getHeight()), "png")
  return(readPNG(bytes))
}

#' copy.image.to.clipboard
#' 
#' generate an image and make it available to the system
#' clipboard.
#' 
#' @param molecule The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @param depictor Optional. Default \code{NULL}. Depictor from \code{get.depictor}
#' @export
copy.image.to.clipboard <-  function(molecule, depictor = NULL) {
  if (is.null(depictor))
    depictor <- get.depictor()

  if (Sys.info()[1] == 'Darwin') { ## try the standalone helper
    smi <- get.smiles(molecule)
    jarfile <- system.file(package='rcdk')
    rcdklibs <- system.file(package='rcdklibs')
    if (depictor$getSma() == "") {
      sma = "\"\""
    } else {
      sma = depictor$getSma()
    }
    cmd <- sprintf('java -cp %s/cont/*:%s/cont/rcdk.jar org.guha.rcdk.app.OSXHelper copyToClipboard "%s" %d %d %f %s %s %s %s %s %d %s',
                   rcdklibs, jarfile, smi,
                   depictor$getWidth(), depictor$getHeight(),
                   depictor$getZoom(), depictor$getStyle(), depictor$getAnnotate(),
                   depictor$getAbbr(), depictor$isSuppressh(), depictor$isShowTitle(),
                   depictor$getSmaLimit(), sma)
    return(system(cmd))
  }
  if (attr(molecule,"jclass") != "org/openscience/cdk/interfaces/IAtomContainer")
    stop("Must supply an IAtomContainer object")
  .jcall('org/guha/rcdk/view/MoleculeImageToClipboard',
         'V',
         'copyImageToClipboard',
         molecule, depictor);
}

## test.view.image <- function()
## {
##   m <- parse.smiles('c1ccccc1C(=O)NC')
##   img <- view.image.2d(m, 100,100)
##   dims <- dim(img)  
##   checkEquals(dims[1], 100)
##   checkEquals(dims[2], 100)
##   checkEquals(dims[2], 3)
## }

#' Get adjacency matrix for a molecule.
#'
#' The adjacency matrix for a molecule with \eqn{N} non-hydrogen atoms is an
#' \eqn{N \times N} matrix where the element [\eqn{i},\eqn{j}] is set to 1
#' if atoms \eqn{i} and \eqn{j} are connected by a bond, otherwise set to 0.
#'
#' @param mol A \code{jobjRef} object with Java class \code{IAtomContainer}
#' @return A \eqn{N \times N} numeric matrix
#' @author Rajarshi Guha \email{rajarshi.guha@gmail.com}
#' @seealso \code{\link{get.connection.matrix}}
#' @examples
#' m <- parse.smiles("CC=C")[[1]]
#' get.adjacency.matrix(m)
#' @export
get.adjacency.matrix <- function(mol) {
    am <- .jcall('org.openscience.cdk.graph.matrix.AdjacencyMatrix','[[I','getMatrix', mol)
    do.call(rbind, lapply(am, .jevalArray))
}

#' Get connection matrix for a molecule.
#'
#' The connection matrix for a molecule with \eqn{N} non-hydrogen atoms is an
#' \eqn{N \times N} matrix where the element [\eqn{i},\eqn{j}] is set to the 
#' bond order if atoms \eqn{i} and \eqn{j} are connected by a bond, otherwise set to 0.
#'
#' @param mol A \code{jobjRef} object with Java class \code{IAtomContainer}
#' @return A \eqn{N \times N} numeric matrix
#' @author Rajarshi Guha \email{rajarshi.guha@gmail.com}
#' @seealso \code{\link{get.adjacency.matrix}}
#' @examples
#' m <- parse.smiles("CC=C")[[1]]
#' get.connection.matrix(m)
#' @export
get.connection.matrix <- function(mol) {
    cm <- .jcall('org.openscience.cdk.graph.matrix.ConnectionMatrix','[[D','getMatrix', mol)
    do.call(rbind, lapply(cm, .jevalArray))
}

#' Generate molecular fingerprints
#' 
#' `get.fingerprint` returns a `fingerprint` object representing molecular fingerprint of
#' the input molecule.
#' 
#' @param molecule A \code{jobjRef} object to an \code{IAtomContaine}
#' @param type The type of fingerprint. Possible values are:
#'   \itemize{
#' \item standard - Considers paths of a given length. The default is
#' but can be changed. These are hashed fingerprints, with a
#' default length of 1024
#' \item extended - Similar to the standard type, but takes rings and
#' atomic properties into account into account
#' \item graph - Similar to the standard type by simply considers connectivity
#' \item hybridization - Similar to the standard type, but only consider hybridization state
#' \item maccs - The popular 166 bit MACCS keys described by MDL
#' \item estate - 79 bit fingerprints corresponding to the E-State atom types described by Hall and Kier
#' \item pubchem - 881 bit fingerprints defined by PubChem
#' \item kr - 4860 bit fingerprint defined by Klekota and Roth
#' \item shortestpath - A fingerprint based on the shortest paths between pairs of atoms and takes into account ring systems, charges etc.
#' \item signature - A feature,count type of fingerprint, similar in nature to circular fingerprints, but based on the signature 
#' descriptor
#' \item circular - An implementation of the ECFP6 (default) fingerprint. Other circular types can be chosen by modifying the \code{circular.type} parameter.
#' \item substructure - Fingerprint based on list of SMARTS pattern. By default a set of functional groups is tested.
#' }
#' @param fp.mode The style of fingerprint. Specifying "`bit`" will return a binary fingerprint,
#' "`raw`" returns the the original representation (usually sequence of integers) and 
#' "`count`" returns the fingerprint as a sequence of counts.
#' @param depth The search depth. This argument is ignored for the
#' `pubchem`, `maccs`, `kr` and `estate` fingerprints
#' @param size The final length of the fingerprint. 
#' This argument is ignored for the `pubchem`, `maccs`, `kr`, `signature`, `circular` and 
#' `estate` fingerprints
#' @param substructure.pattern List of characters containing the SMARTS pattern to match. If the an empty list is provided (default) than the functional groups substructures (default in CDK) are used.
#' @param circular.type Name of the circular fingerprint type that should be computed given as string. Possible values are: 'ECFP0', 'ECFP2', 'ECFP4', 'ECFP6' (default), 'FCFP0', 'FCFP2', 'FCFP4' and 'FCFP6'.
#' @param verbose Verbose output if \code{TRUE}
#' @return an S4 object of class \code{\link{fingerprint-class}} or \code{\link{featvec-class}}, 
#' which can be manipulated with the fingerprint package.
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @examples 
#' ## get some molecules
#' sp <- get.smiles.parser()
#' smiles <- c('CCC', 'CCN', 'CCN(C)(C)', 'c1ccccc1Cc1ccccc1','C1CCC1CC(CN(C)(C))CC(=O)CC')
#' mols <- parse.smiles(smiles)
#' 
#' ## get a single fingerprint using the standard
#' ## (hashed, path based) fingerprinter
#' fp <- get.fingerprint(mols[[1]])
#' 
#' ## get MACCS keys for all the molecules
#' fps <- lapply(mols, get.fingerprint, type='maccs')
#' 
#' ## get Signature fingerprint
#' ## feature, count fingerprinter
#' fps <- lapply(mols, get.fingerprint, type='signature', fp.mode='raw')
#' ## get Substructure fingerprint for functional group fragments
#' fps <- lapply(mols, get.fingerprint, type='substructure')
#' 
#' ## get Substructure count fingerprint for user defined fragments
#' mol1 <- parse.smiles("c1ccccc1CCC")[[1]]
#' smarts <- c("c1ccccc1", "[CX4H3][#6]", "[CX2]#[CX2]")
#' fps <- get.fingerprint(mol1, type='substructure', fp.mode='count',
#'     substructure.pattern=smarts)
#' 
#' ## get ECFP0 count fingerprints 
#' mol2 <- parse.smiles("C1=CC=CC(=C1)CCCC2=CC=CC=C2")[[1]]
#' fps <- get.fingerprint(mol2, type='circular', fp.mode='count', circular.type='ECFP0')
get.fingerprint <- function(molecule, type = 'standard', fp.mode = 'bit', depth=6, size=1024, substructure.pattern=character(), circular.type = "ECFP6", verbose=FALSE) {
  if (is.null(attr(molecule, 'jclass'))) stop("Must supply an IAtomContainer or something coercable to it")
  if (attr(molecule, "jclass") != "org/openscience/cdk/interfaces/IAtomContainer") {
    ## try casting it
    molecule <- .jcast(molecule, "org/openscience/cdk/interfaces/IAtomContainer")
  }

  mode(size) <- 'integer'
  mode(depth) <- 'integer'
  
  # Determine integer ID for the circular fingerprint given its desired type.
  # This allows us to use also ECFP4, ... 
  if (type == 'circular') {
        circular.type.id <- switch(circular.type, 
             ECFP0 = 1, ECFP2 = 2, ECFP4 = 3, ECFP6 = 4,
             FCFP0 = 5, FCFP2 = 6, FCFP4 = 7, FCFP6 = 8,
             NULL)
        
        if (is.null(circular.type.id)) stop(paste('Invalid circular fingerprint type: ', circular.type))
        
        mode(circular.type.id) <- 'integer'
  }

  fingerprinter <-
    switch(type,
           standard = .jnew('org/openscience/cdk/fingerprint/Fingerprinter', size, depth),
           extended = .jnew('org/openscience/cdk/fingerprint/ExtendedFingerprinter', size, depth),
           graph = .jnew('org/openscience/cdk/fingerprint/GraphOnlyFingerprinter', size, depth),
           maccs = .jnew('org/openscience/cdk/fingerprint/MACCSFingerprinter'),
           pubchem = .jnew('org/openscience/cdk/fingerprint/PubchemFingerprinter', get.chem.object.builder()),
           estate = .jnew('org/openscience/cdk/fingerprint/EStateFingerprinter'),
           hybridization = .jnew('org/openscience/cdk/fingerprint/HybridizationFingerprinter', size, depth),
           lingo = .jnew('org/openscience/cdk/fingerprint/LingoFingerprinter', depth),
           kr = .jnew('org/openscience/cdk/fingerprint/KlekotaRothFingerprinter'),
           shortestpath = .jnew('org/openscience/cdk/fingerprint/ShortestPathFingerprinter', size),
           signature = .jnew('org/openscience/cdk/fingerprint/SignatureFingerprinter', depth),
           circular = .jnew('org/openscience/cdk/fingerprint/CircularFingerprinter', circular.type.id),
           substructure = 
               if (length(substructure.pattern) == 0) 
                   # Loads the default group substructures
                   { .jnew('org/openscience/cdk/fingerprint/SubstructureFingerprinter') }
               else
                   # Loads the substructures defined by the user
                   { .jnew('org/openscience/cdk/fingerprint/SubstructureFingerprinter', .jarray(substructure.pattern)) },
           )
  if (is.null(fingerprinter)) stop("Invalid fingerprint type specified")

  jfp <- NA
  if (fp.mode == 'bit') {
    jfp <- .jcall(fingerprinter,
                  "Lorg/openscience/cdk/fingerprint/IBitFingerprint;",
                  "getBitFingerprint", molecule, check=FALSE)
  } else if (fp.mode == 'raw') {
    jfp <- .jcall(fingerprinter,
                  "Ljava/util/Map;",
                  "getRawFingerprint", molecule, check=FALSE)
  } else if (fp.mode == 'count') {
    jfp <- .jcall(fingerprinter,
                  "Lorg/openscience/cdk/fingerprint/ICountFingerprint;",
                  "getCountFingerprint", molecule, check=FALSE)    
  }
  
  e <- .jgetEx()
  if (.jcheck(silent=TRUE)) { 
    if (verbose) print(e)
    return(NULL)
  }

  moltitle <- get.property(molecule, 'Title')
  if (is.na(moltitle)) moltitle <- ''

  if (fp.mode == 'bit') {
    bitset <- .jcall(jfp, "Ljava/util/BitSet;", "asBitSet")
    
    if (type == 'maccs') nbit <- 166
    else if (type == 'estate') nbit <- 79
    else if (type == 'pubchem') nbit <- 881
    else if (type == 'kr') nbit <- 4860
    else if (type == 'substructure') nbit <- .jcall(fingerprinter, "I", "getSize")
    else if (type == 'circular') nbit <- .jcall(fingerprinter, "I", "getSize")
    else nbit <- size
    
    bitset <- .jcall(bitset, "S", "toString")
    s <- gsub('[{}]','', bitset)
    s <- strsplit(s, split=',')[[1]]
    return(new("fingerprint", nbit=nbit, bits=as.numeric(s)+1, provider="CDK", name=moltitle))
  } else if (fp.mode == 'raw') {
    keySet <- .jcall(jfp, "Ljava/util/Set;", method="keySet")
    size <- .jcall(jfp, "I", method="size")
    if (size == 0) return(new('featvec', provider='CDK', name=moltitle))
    keyIter <- .jcall(keySet, "Ljava/util/Iterator;", method="iterator")
    keys <- list()
    for (i in 1:size) {
      keys[[i]] <- J(keyIter, "next")
    }

    values <- list()
    for (i in 1:length(keys)) {
      tmp <- .jcall(jfp, "Ljava/lang/Object;", "get", .jcast(new(J("java/lang/String"),keys[[i]]),"java/lang/Object") )
      values[[i]] <- .jsimplify(tmp)
    }
    ## now make our features and create the featvec fp
    if (length(keys) != length(values))
      stop("Feature length did not match count length. This is a problem")
    features <- lapply(1:length(keys), function(i) new("feature", feature=keys[[i]], count=as.integer(values[[i]])))
    return(new("featvec", features=features, provider="CDK", name=moltitle))
  } else if (fp.mode == 'count') {
    fpsize <- .jcall(jfp, "J", "size")
    nbin <- .jcall(jfp, "I", "numOfPopulatedbins")
    ## get hash values
    hvals <- sapply(0:(nbin-1), function(i) .jcall(jfp, "I", "getHash", as.integer(i)) )
    cvals <- sapply(0:(nbin-1), function(i) .jcall(jfp, "I", "getCount", as.integer(i)) )
    features <- lapply(1:length(hvals), function(i) new("feature", feature=as.character(hvals[i]), count=as.integer(cvals[i])))
    return(new("featvec", features=features, provider="CDK", name=moltitle))
  }
}

test.fp <- function() {
    mol <- parse.smiles("CCCCC")[[1]]
    fp <- get.fingerprint(mol, type='maccs')
    checkTrue(length(fp@bits) > 0)

    fp <- get.fingerprint(mol, type='kr')
    checkTrue(length(fp@bits) > 0)
    fp <- get.fingerprint(mol, type='shortestpath')
    checkTrue(length(fp@bits) > 0)
}

# Substructure test are inspired by the test for the substructure fingerprints in CDK
test.fp.substructures.binary <- function() {
    # Default patterns: functional groups
    mol <- parse.smiles("c1ccccc1CCC")[[1]]
    fp <- get.fingerprint(mol, type="substructure", fp.mode="bit")
    fp_bits <- fingerprint::fp.to.matrix(list(fp))

    checkEquals(length(fp), 307)
    checkEquals(fp_bits[1], 1)
    checkEquals(fp_bits[2], 1)
    checkEquals(fp_bits[274], 1)
    checkEquals(fp_bits[101], 0)

    # User defined patterns
    smarts <- c("c1ccccc1", "[CX4H3][#6]", "[CX2]#[CX2]")
    mol <- parse.smiles("c1ccccc1CCC")[[1]]
    fp <- get.fingerprint(mol, type="substructure", fp.mode="bit",
                          substructure.pattern = smarts)
    fp_bits <- fingerprint::fp.to.matrix(list(fp))

    checkEquals(length(fp), 3)
    checkEquals(length(fp@bits), 2)
    checkEquals(fp_bits[1], 1)
    checkEquals(fp_bits[2], 1)
    checkEquals(fp_bits[3], 0)

    mol <- parse.smiles("C=C=C")[[1]]
    fp <- get.fingerprint(mol, type="substructure", fp.mode="bit",
                          substructure.pattern = smarts)
    fp_bits <- fingerprint::fp.to.matrix(list(fp))

    checkEquals(length(fp), 3)
    checkEquals(length(fp@bits), 0)
    for (i_fp in 1:3) {
        checkEquals(fp_bits[i_fp], 0)
    }

    # Check for aromatic ring
    smarts <- "a:1:a:a:a:a:a1"
    mol <- parse.smiles("C1=CC=CC(=C1)CCCC2=CC=CC=C2")[[1]]
    set.atom.types(mol)
    do.aromaticity(mol)
    fp <- get.fingerprint(mol, type="substructure", fp.mode="bit",
                          substructure.pattern = smarts)
    fp_bits <- fingerprint::fp.to.matrix(list(fp))
    checkEquals(length(fp), 1)
    checkEquals(fp_bits[1], 1)
}

test.fp.substructures.count <- function() {
    # Default patterns: functional groups
    mol <- parse.smiles("c1ccccc1CCC")[[1]]
    fp <- get.fingerprint(mol, type="substructure", fp.mode="count")

    checkEquals(length(fp), 307)
    checkTrue(fingerprint::count(fp@features[[1]]) > 0)
    checkTrue(fingerprint::count(fp@features[[2]]) > 0)
    checkTrue(fingerprint::count(fp@features[[274]]) > 0)
    checkTrue(fingerprint::count(fp@features[[101]]) == 0)

    # User defined patterns
    smarts <- c("c1ccccc1", "[CX4H3][#6]", "[CX2]#[CX2]")
    mol <- parse.smiles("c1ccccc1CCC")[[1]]
    fp <- get.fingerprint(mol, type="substructure", fp.mode="count",
                          substructure.pattern = smarts)

    checkEquals(length(fp), 3)
    checkEquals(fingerprint::count(fp@features[[1]]), 1)
    checkEquals(fingerprint::count(fp@features[[2]]), 1)
    checkEquals(fingerprint::count(fp@features[[3]]), 0)

    mol <- parse.smiles("C=C=C")[[1]]
    fp <- get.fingerprint(mol, type="substructure", fp.mode="count",
                          substructure.pattern = smarts)

    checkEquals(length(fp), 3)
    for (i_fp in 1:3) {
        checkEquals(fingerprint::count(fp@features[[i_fp]]), 0)
    }

    # Check for aromatic ring
    smarts <- "a:1:a:a:a:a:a1"
    mol <- parse.smiles("C1=CC=CC(=C1)CCCC2=CC=CC=C2")[[1]]
    set.atom.types(mol)
    do.aromaticity(mol)
    fp <- get.fingerprint(mol, type="substructure", fp.mode="count",
                          substructure.pattern = smarts)

    checkEquals(length(fp), 1)
    checkEquals(fingerprint::count(fp@features[[1]]), 2)
}

#' matches 
#' 
#' @param query Required. A SMARTSQuery
#' @param target Required. The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @param return.matches Optional. Default \code{FALSE}
#' @export
matches <- function(query, target, return.matches=FALSE) {
  if (!is.list(target)) target <- list(target)
  if (!all(unlist(lapply(target, class)) == 'jobjRef'))
    stop("targets must be a list of IAtomContainer objects or a single IAtomContainer object")

  dcob <- get.chem.object.builder()

  ## make an SQT
  sqt <- new(J("org/openscience/cdk/smiles/smarts/SMARTSQueryTool"), query, dcob)
  matchings <- unlist(lapply(target, function(z) sqt$matches(z)))

  matchings <- lapply(target, function(z) {
    status <- sqt$matches(z)
    if (status) {
      mappings <- sqt$getUniqueMatchingAtoms()
      mappings <- lapply(1:mappings$size(), function(i) {
        atomIndices <- mappings$get(as.integer(i-1))
        atomIndinces <- .javalist.to.rlist(atomIndices)
        sapply(atomIndices, .jsimplify)
      })
    } else{
      mappings <- NULL
    }
    return(list(match=status, mapping=mappings))
  })

  if (!return.matches) return(unlist(lapply(matchings, "[", 1)))
  else {
    return(matchings)
  }
}

test.match1 <- function()
{
  m <- parse.smiles('CCCCc1cccc(Cl)c1')[[1]]
  q <- 'cCl'
  checkTrue(matches(q,m))
}

test.match2 <- function()
{
  m <- parse.smiles('CCCCc1cccc(Cl)c1')[[1]]
  q <- 'CCCCc'
  checkTrue(matches(q,m))
}

test.match3 <- function()
{
  m1 <- parse.smiles('CCCCc1cccc(Cl)c1')[[1]]
  m2 <- parse.smiles('CC(N)(N)CC=O')[[1]]
  q <- '[CD2]'
  checkTrue(all(matches(q,list(m1,m2))))
  checkEquals(2, length(matches(q,list(m1,m2))))
}

test.match4 <- function()
{

  smiles <- c('CCC', 'c1ccccc1', 'C(C)(C=O)C(CCNC)C1CC1C(=O)')
  mols <- sapply(smiles, parse.smiles)
  mappings <- matches("CCCC", mols, TRUE)
  checkEquals(3, length(mappings))
  checkTrue(!mappings[[1]]$match)
  checkTrue(!mappings[[2]]$match)
  checkTrue(mappings[[3]]$match)

  checkTrue(is.null(mappings[[1]]$mapping))
  checkTrue(is.null(mappings[[2]]$mapping))

  checkEquals(13, length(mappings[[3]]$mapping))
  m <- sapply(mappings[[3]]$mapping, function(x) length(x))
  checkTrue(all(m == 4))
}


test.mcs1 <- function() {
  # Skip on CRAN - MCS algorithms are computationally intensive and trigger multi-threading
  if (Sys.getenv("_R_CHECK_PACKAGE_NAME_", "") != "" || Sys.getenv("_R_CHECK_TIMINGS_", "") != "") {
    return(TRUE)
  }
  mols <- parse.smiles(c("NCc1ccccc1OC(=N)CCN", "c1ccccc1OC(=N)"))
  lapply(mols, do.aromaticity)
  lapply(mols, set.atom.types)
  mcs <- get.mcs(mols[[1]], mols[[2]], TRUE)
  checkTrue(.jclass(mcs) %in% c("org.openscience.cdk.silent.AtomContainer", "org.openscience.cdk.silent.AtomContainer2"))
  checkEquals(9, get.atom.count(mcs))
}

test.mcs3 <- function() {
  # Skip on CRAN - MCS algorithms are computationally intensive and trigger multi-threading
  if (Sys.getenv("_R_CHECK_PACKAGE_NAME_", "") != "" || Sys.getenv("_R_CHECK_TIMINGS_", "") != "") {
    return(TRUE)
  }
  mols <- parse.smiles(c("c1cccc(COC(=O)NC(CC(C)C)C(=O)NC(CCc2ccccc2)C(=O)COC)c1", "c1cccc(COC(=O)NC(CC(C)C)C(=O)NCC#N)c1"))
  lapply(mols, do.aromaticity)
  lapply(mols, set.atom.types)
  mcs <- get.mcs(mols[[1]], mols[[2]], TRUE)
  checkTrue(.jclass(mcs) %in% c("org.openscience.cdk.silent.AtomContainer", "org.openscience.cdk.silent.AtomContainer2"))
  checkEquals(21, get.atom.count(mcs))
}

test.mcs2 <- function() {
  # Skip on CRAN - MCS algorithms are computationally intensive and trigger multi-threading
  if (Sys.getenv("_R_CHECK_PACKAGE_NAME_", "") != "" || Sys.getenv("_R_CHECK_TIMINGS_", "") != "") {
    return(TRUE)
  }
  mols <- parse.smiles(c("NCc1ccccc1OC(=N)CCN", "c1ccccc1OC(=N)"))
  lapply(mols, do.aromaticity)
  lapply(mols, set.atom.types)
  mcs <- get.mcs(mols[[1]], mols[[2]], FALSE)
  checkTrue(inherits(mcs, "matrix"))
  checkEquals(9, nrow(mcs))
  checkEquals(2, ncol(mcs))
}

test.aromaticity.match <- function() {
  smiles <- c("CCC", "c1ccccc1", "C(C)(C=O)C(CCNC)C1CC1C(=O)")
  mols <- parse.smiles(smiles)
  query <- "[#6]=O"
doesMatch <- matches(query, mols)
  checkTrue(all(doesMatch == c(FALSE,FALSE,TRUE)))
}

test.match1 <- function()
{
  m <- parse.smiles('CCCCc1cccc(Cl)c1')[[1]]
  q <- 'cCl'
  checkTrue(matches(q,m))
}

test.match2 <- function()
{
  m <- parse.smiles('CCCCc1cccc(Cl)c1')[[1]]
  q <- 'CCCCc'
  checkTrue(matches(q,m))
}

test.match3 <- function()
{
  m1 <- parse.smiles('CCCCc1cccc(Cl)c1')[[1]]
  m2 <- parse.smiles('CC(N)(N)CC=O')[[1]]
  q <- '[CD2]'
  checkTrue(all(matches(q,list(m1,m2))))
  checkEquals(2, length(matches(q,list(m1,m2))))
}

test.match4 <- function()
{

  smiles <- c('CCC', 'c1ccccc1', 'C(C)(C=O)C(CCNC)C1CC1C(=O)')
  mols <- sapply(smiles, parse.smiles)
  mappings <- matches("CCCC", mols, TRUE)
  checkEquals(3, length(mappings))
  checkTrue(!mappings[[1]]$match)
  checkTrue(!mappings[[2]]$match)
  checkTrue(mappings[[3]]$match)

  checkTrue(is.null(mappings[[1]]$mapping))
  checkTrue(is.null(mappings[[2]]$mapping))

  checkEquals(13, length(mappings[[3]]$mapping))
  m <- sapply(mappings[[3]]$mapping, function(x) length(x))
  checkTrue(all(m == 4))
}


test.mcs1 <- function() {
  mols <- parse.smiles(c("NCc1ccccc1OC(=N)CCN", "c1ccccc1OC(=N)"))
  lapply(mols, do.aromaticity)
  lapply(mols, do.typing) 
  mcs <- get.mcs(mols[[1]], mols[[2]], TRUE)
  checkEquals("org.openscience.cdk.AtomContainer2", .jclass(mcs))
  checkEquals(9, get.atom.count(mcs))
}

test.mcs3 <- function() {
  mols <- parse.smiles(c("c1cccc(COC(=O)NC(CC(C)C)C(=O)NC(CCc2ccccc2)C(=O)COC)c1", "c1cccc(COC(=O)NC(CC(C)C)C(=O)NCC#N)c1"))
  lapply(mols, do.aromaticity)
  lapply(mols, do.typing) 
  mcs <- get.mcs(mols[[1]], mols[[2]], TRUE)
  checkEquals("org.openscience.cdk.AtomContainer2", .jclass(mcs))
  checkEquals(21, get.atom.count(mcs))
}

test.mcs2 <- function() {
  mols <- parse.smiles(c("NCc1ccccc1OC(=N)CCN", "c1ccccc1OC(=N)"))
  lapply(mols, do.aromaticity)
  lapply(mols, do.typing) 
  mcs <- get.mcs(mols[[1]], mols[[2]], FALSE)
  checkTrue(inherits(mcs, "matrix"))
  checkEquals(9, nrow(mcs))
  checkEquals(2, ncol(mcs))
}

test.aromaticity.match <- function() {
  smiles <- c("CCC", "c1ccccc1", "C(C)(C=O)C(CCNC)C1CC1C(=O)")
  mols <- parse.smiles(smiles)
  query <- "[#6]=O"
doesMatch <- matches(query, mols)
  checkTrue(all(doesMatch == c(FALSE,FALSE,TRUE)))
}

#' Generate Bemis-Murcko Fragments
#' 
#' Fragment the input molecule using the Bemis-Murcko scheme
#' 
#' A variety of methods for fragmenting molecules are available ranging from
#' exhaustive, rings to more specific methods such as Murcko frameworks. Fragmenting a
#' collection of molecules can be a useful for a variety of analyses. In addition
#' fragment based analysis can be a useful and faster alternative to traditional 
#' clustering of the whole collection, especially when it is large.
#' 
#' Note that exhaustive fragmentation of large molecules (with many single bonds) can become
#' time consuming.
#' 
#' @param mols A list of `jobjRef` objects of Java class `IAtomContainer`
#' @param min.frag.size The smallest fragment to consider (in terms of heavy atoms)
#' @param as.smiles If `TRUE` return the fragments as SMILES strings. If not, then fragments
#' are returned as `jobjRef` objects
#' @param single.framework If `TRUE`, then a single framework (i.e., the framework consisting of the
#' union of all ring systems and linkers) is returned for each molecule. Otherwise, all combinations
#' of ring systems and linkers are returned
#' @return Returns a list with each element being a list with two elements: `rings` and
#' `frameworks`. Each of these elements is either a character vector of SMILES strings or a list of
#' `IAtomContainer` objects.
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @seealso [get.exhuastive.fragments()]
#' @export
#' @examples 
#' mol <- parse.smiles('c1ccc(cc1)CN(c2cc(ccc2[N+](=O)[O-])c3c(nc(nc3CC)N)N)C')[[1]]
#' mf1 <- get.murcko.fragments(mol, as.smiles=TRUE, single.framework=TRUE)
#' mf1 <- get.murcko.fragments(mol, as.smiles=TRUE, single.framework=FALSE)
get.murcko.fragments <- function(mols, min.frag.size = 6, as.smiles = TRUE, single.framework = FALSE) {
  if (!is.list(mols)) mols <- list(mols)
  klasses <- unlist(lapply(mols, function(x) attr(x, "jclass")))
  if (!all(klasses ==  "org/openscience/cdk/interfaces/IAtomContainer")) {
    stop("Must supply an IAtomContainer object")
  }

  fragmenter <- .jnew("org/openscience/cdk/fragment/MurckoFragmenter",
                      single.framework, as.integer(min.frag.size))
  
  ret <- lapply(mols, function(x) {
    .jcall(fragmenter, "V", "generateFragments", x)
    if (as.smiles) {
      rings <- .jcall(fragmenter, "[S", "getRingSystems")
      frames <- .jcall(fragmenter, "[S", "getFrameworks")
    } else {
      rings <- .jcall(fragmenter, "[Lorg/openscience/cdk/interfaces/IAtomContainer;", "getRingSystemsAsContainers")
      frames <- .jcall(fragmenter, "[Lorg/openscience/cdk/interfaces/IAtomContainer;", "getFrameworksAsContainers")
    }
    return(list(rings = rings, frameworks = frames))    
  })
  return(ret)
}

#' @inherit get.murcko.fragments 
#' @return returns a list of length equal to the number of input molecules. Each
#' element is a character vector of SMILES strings or a list of `jobjRef` objects.
get.exhaustive.fragments <- function(mols, min.frag.size = 6, as.smiles = TRUE) {
  if (!is.list(mols)) mols <- list(mols)
  klasses <- unlist(lapply(mols, function(x) attr(x, "jclass")))
  if (!all(klasses ==  "org/openscience/cdk/interfaces/IAtomContainer")) {
    stop("Must supply an IAtomContainer object")
  }

  fragmenter <- .jnew("org/openscience/cdk/fragment/ExhaustiveFragmenter", as.integer(min.frag.size))

  ret <- lapply(mols, function(x) {
    .jcall(fragmenter, "V", "generateFragments", x)
    if (as.smiles) {
      fragments <- .jcall(fragmenter, "[S", "getFragments")
    } else {
      fragments <- .jcall(fragmenter, "[Lorg/openscience/cdk/interfaces/IAtomContainer;", "getFragmentsSystemsAsContainers")
    }
    return(fragments)    
  })
  return(ret)
}

test.frag1 <- function() {
  m <- parse.smiles("c1(ccc(cc1C)CCC(C(CCC)C2C(C2)CC)C3C=C(C=C3)CC)C")[[1]]
  do.aromaticity(m)
  set.atom.types(m)
  f <- get.murcko.fragments(m, as.smiles=TRUE, min.frag.size = 6, single.framework = TRUE)
  checkEquals(length(f), 1)
  checkEquals(length(f[[1]]$rings), 1)
  checkEquals(f[[1]]$rings, "c1ccccc1")
  checkEquals(f[[1]]$frameworks, "c1ccc(cc1)CCC(CC2CC2)C3C=CC=C3")
}

test.frag2 <- function() {
  ms <- parse.smiles(c('c1(ccc(cc1C)CCC(C(CCC)C2C(C2)CC)C3C=C(C=C3)CC)C',
                       'c1ccc(cc1)c2c(oc(n2)N(CCO)CCO)c3ccccc3',
                       'COc1ccc(cc1OCc2ccccc2)C(=S)N3CCOCC3'))
  lapply(ms, do.aromaticity)
  lapply(ms, set.atom.types)
  f <- get.murcko.fragments(ms, as.smiles=TRUE, min.frag.size = 6, single.framework = TRUE)
  checkEquals(length(f), 3)

  fworks <- unlist(lapply(f, function(x) length(x$frameworks)))
  checkTrue(all(fworks == 1))
}

test.frag3 <- function() {
  ms <- parse.smiles(c('c1(ccc(cc1C)CCC(C(CCC)C2C(C2)CC)C3C=C(C=C3)CC)C',
                       'c1ccc(cc1)c2c(oc(n2)N(CCO)CCO)c3ccccc3',
                       'COc1ccc(cc1OCc2ccccc2)C(=S)N3CCOCC3'))
  lapply(ms, do.aromaticity)
  lapply(ms, set.atom.types)
  f <- get.murcko.fragments(ms, as.smiles=FALSE, min.frag.size = 6, single.framework = TRUE)
  checkEquals(length(f), 3)

  fworks <- unlist(lapply(f, function(x) unlist(lapply(x$frameworks, .jclass))))
  checkTrue(all(fworks %in% c("org.openscience.cdk.silent.AtomContainer", "org.openscience.cdk.silent.AtomContainer2")))
}

################################################################################
#' Deprecated functions in the rcdk package.
#' 
#' These functions are provided for compatibility with older version of
#' the phyloseq package.  They may eventually be completely
#' removed.
#' 
#' @usage deprecated_rcdk_function(x, value, ...)
#' @rdname rcdk-deprecated
#' @name rcdk-deprecated
#' @param x For assignment operators, the object that will undergo a replacement
#'  (object inside parenthesis).
#' @param value For assignment operators, the value to replace with 
#'  (the right side of the assignment).
#' @param ... For functions other than assignment operators, 
#'  parameters to be passed to the modern version of the function (see table).
#' @docType package
#' @export do.typing
#' @aliases deprecated_rcdk_function do.typing
#' @details
#' \tabular{rl}{
#'   \code{do.typing} \tab now a synonym for \code{\link{set.atom.types}}\cr
#' }
#'
deprecated_rcdk_function <- function(x, value, ...){return(NULL)}
do.typing <- function(...){.Deprecated("set.atom.types", package="rcdk");return(set.atom.types(...))}
################################################################################

#' Set a property value of the molecule.
#' 
#' This function sets the value of a keyed property on the molecule. 
#' Properties enable us to associate arbitrary pieces of data with a 
#' molecule. Such data can be text, numeric or a Java object 
#' (represented as a `jobjRef`).
#' 
#' @param molecule The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @param key The property key as a character string
#' @param value The value of the property. This can be a character, numeric or 
#' `jobjRef` R object
#' @seealso \code{\link{get.property}}, \code{\link{get.properties}}, \code{\link{remove.property}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @examples
#' mol <- parse.smiles("CC1CC(C=O)CCC1")[[1]]
#' set.property(mol, 'prop1', 23.45)
#' set.property(mol, 'prop2', 'inactive')
#' get.property(mol, 'prop1')
#' 
set.property <- function(molecule, key, value) {
  if (!is.character(key)) {
    stop("The property key must be a character")
  }
  if (!.check.class(molecule, "org/openscience/cdk/interfaces/IAtomContainer") &&
      !.check.class(molecule, "org/openscience/cdk/AtomContainer"))
    stop("Must supply an AtomContainer or IAtomContainer object")
  if (.check.class(molecule, "org/openscience/cdk/AtomContainer"))
    atom <- .jcast(molecule, "org/openscience/cdk/interfaces/IAtomContainer")
  
  if (is.character(value)) {
    value <- .jcall('org/guha/rcdk/util/Misc', 'V', 'setProperty',
                    molecule, as.character(key),
                    .jcast( .jnew("java/lang/String", value), "java/lang/Object"))
  } else if (is.integer(value)) {
    value <-.jcall('org/guha/rcdk/util/Misc', 'V', 'setProperty',
                   molecule, as.character(key), as.integer(value))
  } else if (is.double(value)) {
    value <-.jcall('org/guha/rcdk/util/Misc', 'V', 'setProperty',
                   molecule, as.character(key), as.double(value))
  } else if (class(value) == 'jobjRef') {
    value <-.jcall('org/guha/rcdk/util/Misc', 'V', 'setProperty',
                   molecule, as.character(key),
                   .jcast(value, 'java/lang/Object'))
  }
  
}

#' Get a property value of the molecule.
#' 
#' This function retrieves the value of a keyed property that has
#' previously been set on the molecule. Properties enable us to 
#' associate arbitrary pieces of data with a molecule. Such data
#' can be text, numeric or a Java object (represented as a `jobjRef`).
#' 
#' @param molecule The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @param key The property key as a character string
#' @return The value of the property. If there is no property with the specified key, `NA` is returned
#' @seealso \code{\link{set.property}}, \code{\link{get.properties}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @examples 
#' mol <- parse.smiles("CC1CC(C=O)CCC1")[[1]]
#' set.property(mol, 'prop1', 23.45)
#' set.property(mol, 'prop2', 'inactive')
#' get.property(mol, 'prop1')
get.property <- function(molecule, key) {
  if (is.jnull(molecule)) {
    warning("Molecule object was null")
    return(NA)
  }
  if (is.null(key) || is.na(key) || !is.character(key)) {
    stop("The property key must be a character")
  }
  if (!.check.class(molecule, "org/openscience/cdk/interfaces/IAtomContainer") &&
      !.check.class(molecule, "org/openscience/cdk/AtomContainer"))
    stop("Must supply an AtomContainer or IAtomContainer object")
  if (.check.class(molecule, "org/openscience/cdk/AtomContainer"))
    atom <- .jcast(molecule, "org/openscience/cdk/interfaces/IAtomContainer")

  value <- .jcall('org/guha/rcdk/util/Misc', 'Ljava/lang/Object;', 'getProperty',
                  molecule, as.character(key), check=FALSE)
  e <- .jgetEx()
  if (.jcheck(silent=TRUE)) {
    return(NA)
  }

  if (is.jnull(value)) return(NA)
  else return(.jsimplify(value))
}

#' Get all properties associated with a molecule.
#' 
#' In this context a property is a value associated with a key and stored
#' with the molecule. This methd returns a list of all the properties of 
#' a molecule. The names of the list are set to the property names.
#' 
#' @param molecule The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @return A named `list` with the property values. Element names are the keys 
#' for each property. If no properties have been defined, an empty list.
#' @seealso \code{\link{set.property}}, \code{\link{get.property}}, \code{\link{remove.property}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @examples 
#' mol <- parse.smiles("CC1CC(C=O)CCC1")[[1]]
#' set.property(mol, 'prop1', 23.45)
#' set.property(mol, 'prop2', 'inactive')
#' get.properties(mol)
get.properties <- function(molecule) {
  if (!.check.class(molecule, "org/openscience/cdk/interfaces/IAtomContainer") &&
      !.check.class(molecule, "org/openscience/cdk/AtomContainer"))
    stop("Must supply an AtomContainer or IAtomContainer object")
  if (.check.class(molecule, "org/openscience/cdk/AtomContainer"))
    atom <- .jcast(molecule, "org/openscience/cdk/interfaces/IAtomContainer")

  map <- .jcall(molecule, "Ljava/util/Map;", method = "getProperties")
  keySet <- .jcall(map, "Ljava/util/Set;", method="keySet")
  size <- .jcall(map, "I", method="size")
  if (size == 0) return(list())
  keyIter <- .jcall(keySet, "Ljava/util/Iterator;", method="iterator")
  keys <- list()
  for (i in 1:size) {
    ##    keys[[i]] <-.jcall(keyIter, "Ljava/lang/Object;", method="next")
    keys[[i]] <- J(keyIter, "next")
  }

  
  values <- list()
  for (i in 1:length(keys)) {
    the.value <- .jcall(map, "Ljava/lang/Object;", "get", .jcast(new(J("java/lang/String"),keys[[i]]),"java/lang/Object") )
    if (is.jnull(the.value)) values[[i]] <- NA
    else values[[i]] <- the.value
  }
  
  ret <- list()
  for (i in 1:length(keys)) {
    k <- keys[[i]]
    if ('jobjRef' %in% class(values[[i]])) ret[[k]] <- .jsimplify(values[[i]])
    else if (is.na(values[[i]])) ret[[k]] <- NA
    else ret[[k]] <- values[[i]]
  }
  ret
}

#' Remove a property associated with a molecule.
#' 
#' In this context a property is a value associated with a key and stored
#' with the molecule. This methd will remove the property defined by the key.
#' If there is such key, a warning is raised.
#' 
#' @param molecule The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @param key The property key as a character string
#' @seealso \code{\link{set.property}}, \code{\link{get.property}}, \code{\link{get.properties}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @examples 
#' mol <- parse.smiles("CC1CC(C=O)CCC1")[[1]]
#' set.property(mol, 'prop1', 23.45)
#' set.property(mol, 'prop2', 'inactive')
#' get.properties(mol)
#' remove.property(mol, 'prop2')
#' get.properties(mol)
remove.property <- function(molecule, key) {
  if (!is.character(key)) {
    stop("The property key must be a character")
  }
  if (!.check.class(molecule, "org/openscience/cdk/interfaces/IAtomContainer") &&
      !.check.class(molecule, "org/openscience/cdk/AtomContainer"))
    stop("Must supply an AtomContainer or IAtomContainer object")
  if (.check.class(molecule, "org/openscience/cdk/AtomContainer"))
    atom <- .jcast(molecule, "org/openscience/cdk/interfaces/IAtomContainer")

  if (is.na(get.property(molecule, key))) {
    warning("No such key exists")
  } else {
    value <- .jcall('org/guha/rcdk/util/Misc', 'V', 'removeProperty',
                    molecule, as.character(key))
  }
}

test.set.props <- function() {
  m <- parse.smiles("CCCC")[[1]]
  set.property(m, "foo", "bar")
  checkEquals(get.property(m,"foo"), "bar")
}

test.get.properties <- function() {
  m <- parse.smiles("CCCC")[[1]]
  set.property(m, "foo", "bar")
  set.property(m, "baz", 1.23)  
  props <- get.properties(m)
  checkEquals(length(props), 3)
  checkTrue(all(sort(names(props)) == c('baz','cdk:Title','foo')))
  checkEquals(props$foo,'bar')
  checkEquals(props$baz,1.23)  
}

## test.props.from.file <- function() {
##   print(getwd())
##   kegg_file <- system.file("molfiles/kegg.sdf", packge="rcdk")
##   f <- load.molecules(kegg_file)
##   checkEquals(length(f), 10)
##   proplens <- unlist(lapply(lapply(f, get.properties), length))
##   checkEquals(proplens, c(5,5,5,5,5,5,5,3,5,5))
## }

#' Boiling Point Data
#'
#' A dataset containing the structures and associated boiling 
#' points for 277 molecules, primarily alkanes and substituted alkanes.
#'
#' @format A data frame with 277 rows and 2 columns.:
#' \describe{
#'   \item{SMILES}{Structure in SMILES format}
#'   \item{BP}{Boiling point in Kelvin}
#'   
#' } The names of the molecules are used as the row names.
#' 
#' @references 
#'  Goll, E.S. and Jurs, P.C.; "Prediction of the Normal Boiling Points of
#'  Organic Compounds From Molecular Structures with a Computational
#'  Neural Network Model", \emph{J. Chem. Inf. Comput. Sci.}, 1999, 
#'  \emph{39}, 974-983.
#'  
#' @docType data
"bpdata"

#' Generate flag for customizing SMILES generation.
#'
#' The CDK supports a variety of customizations for SMILES generation including
#' the use of lower case symbols for aromatic compounds to the use of the ChemAxon
#' \href{http://www.chemeddl.org/tools/marvin/help/formats/cxsmiles-doc.html}{CxSmiles}
#' format. Each 'flavor' is represented by an integer and multiple
#' customizations are bitwise OR'ed. This method accepts the names of one or
#' more customizations and returns the bitwise OR of them.
#' See \href{https://cdk.github.io/cdk/2.3/docs/api/index.html?org/openscience/cdk/smiles/SmiFlavor.html}{CDK documentation}
#' for the list of flavors and what they mean.
#'
#' @param flavors A character vector of flavors. The default is \code{Generic} 
#' (output non-canonical SMILES without stereochemistry, atomic masses). Possible 
#' values are
#' * Absolute
#' * AtomAtomMap
#' * AtomicMass
#' * AtomicMassStrict
#' * Canonical
#' * Cx2dCoordinates
#' * Cx3dCoordinates
#' * CxAtomLabel
#' * CxAtomValue
#' * CxCoordinates
#' * CxFragmentGroup
#' * CxMulticenter
#' * CxPolymer
#' * CxRadical
#' * CxSmiles
#' * CxSmilesWithCoords
#' * Default
#' * Generic
#' * InChILabelling
#' * Isomeric
#' * Stereo
#' * StereoCisTrans
#' * StereoExTetrahedral
#' * StereoTetrahedral
#' * Unique
#' * UniversalSmiles
#' * UseAromaticSymbols
#' @md
#' @return A numeric representing the bitwise `OR`` of the specified flavors
#' @seealso \code{\link{get.smiles}}
#' @references \href{https://cdk.github.io/cdk/2.3/docs/api/index.html?org/openscience/cdk/smiles/SmiFlavor.html}{CDK documentation}
#' @examples
#' m <- parse.smiles('C1C=CCC1N(C)c1ccccc1')[[1]]
#' get.smiles(m)
#' get.smiles(m, smiles.flavors(c('Generic','UseAromaticSymbols')))
#'
#' m <- parse.smiles("OS(=O)(=O)c1ccc(cc1)C(CC)CC |Sg:n:13:m:ht,Sg:n:11:n:ht|")[[1]]
#' get.smiles(m,flavor = smiles.flavors(c("CxSmiles")))
#' get.smiles(m,flavor = smiles.flavors(c("CxSmiles","UseAromaticSymbols")))
#'
#' @export
#' @author Rajarshi Guha \email{rajarshi.guha@@gmail.com}
smiles.flavors <- function(flavors = c('Generic')) {
    valid.flavors <- c('Absolute',
                       'AtomAtomMap',
                       'AtomicMass',
                       'AtomicMassStrict',
                       'Canonical',
                       'Cx2dCoordinates',
                       'Cx3dCoordinates',
                       'CxAtomLabel',
                       'CxAtomValue',
                       'CxCoordinates',
                       'CxFragmentGroup',
                       'CxMulticenter',
                       'CxPolymer',
                       'CxRadical',
                       'CxSmiles',
                       'CxSmilesWithCoords',
                       'Default',
                       'Generic',
                       'InChILabelling',
                       'Isomeric',
                       'Stereo',
                       'StereoCisTrans',
                       'StereoExTetrahedral',
                       'StereoTetrahedral',
                       'Unique',
                       'UniversalSmiles',
                       'UseAromaticSymbols')
    if (any(is.na(match(flavors, valid.flavors)))) {
        stop("Invalid flavor specified")
    }
    vals <- sapply(flavors, function(x) {
        .jfield('org.openscience.cdk.smiles.SmiFlavor', 'I', x)
    })
    Reduce(bitwOr, vals, 0)
}

#' Generate a SMILES representation of a molecule.
#' 
#' The function will generate a SMILES representation of an
#' `IAtomContainer` object. The default parameters of the CDK SMILES
#' generator are used. This can mean that for large ring systems the
#' method may fail. See CDK \href{http://cdk.github.io/cdk/2.2/docs/api/org/openscience/cdk/smiles/SmilesGenerator.html}{Javadocs}
#' for more information
#' @param molecule The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @param flavor The type of SMILES to generate. See \code{\link{smiles.flavors}}. Default is `Generic`
#' SMILES
#' @param smigen A pre-existing SMILES generator object. By default, a new one is created from the specified flavor
#' @return A character string containing the generated SMILES
#' @seealso \code{\link{parse.smiles}}, \code{\link{smiles.flavors}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @references \href{http://cdk.github.io/cdk/2.2/docs/api/org/openscience/cdk/smiles/SmilesGenerator.html}{SmilesGenerator} 
#' @examples 
#' m <- parse.smiles('C1C=CCC1N(C)c1ccccc1')[[1]]
#' get.smiles(m)
#' get.smiles(m, smiles.flavors(c('Generic','UseAromaticSymbols')))
get.smiles <- function(molecule, flavor = smiles.flavors(c('Generic')), smigen = NULL) {
    if (is.null(smigen))
        smigen <- .jnew("org/openscience/cdk/smiles/SmilesGenerator", flavor)
    return(.jcall(smigen, 'S', 'create', molecule))
}

#' Get a SMILES parser object.
#' 
#' This function returns a reference to a SMILES parser
#' object. If you are parsing multiple SMILES strings using multiple
#' calls to \code{\link{parse.smiles}}, it is
#' preferable to create your own parser and supply it to
#' \code{\link{parse.smiles}} rather than forcing that function
#' to instantiate a new parser for each call
#' 
#' @return A `jobjRef` object corresponding to the CDK 
#' \href{http://cdk.github.io/cdk/2.2/docs/api/org/openscience/cdk/smiles/SmilesParser.html}{SmilesParser} class
#' @seealso \code{\link{get.smiles}}, \code{\link{parse.smiles}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.smiles.parser <- function() {
    .jnew("org/openscience/cdk/smiles/SmilesParser", get.chem.object.builder())
}

#' Parse SMILES strings into molecule objects.
#' 
#' This function parses a vector of SMILES strings to generate a list of
#' `IAtomContainer` objects. Note that the resultant molecule will
#' not have any 2D or 3D coordinates.
#' Note that the molecules obtained from this method will not have any
#' aromaticity perception (unless aromatic symbols are encountered, in which 
#' case the relevant atoms are automatically set to aromatic), atom typing or 
#' isotopic configuration done on them. This is in contrast to the 
#' \code{\link{load.molecules}} method. Thus, you should
#' perform these steps manually on the molecules.
#' @param smiles A single SMILES string or a vector of SMILES strings
#' @param kekulise If set to `FALSE` disables electron checking and
#' allows for parsing of incorrect SMILES. If a SMILES does not parse by default, try
#' setting this to `FALSE` - though the resultant molecule may not have consistent
#' bonding. As an example, `c4ccc2c(cc1=Nc3ncccc3(Cn12))c4` will not be parsed by default
#' because it is missing a nitrogen. With this argument set to `FALSE` it will parse
#' successfully, but this is a hack to handle an incorrect SMILES
#' @param omit.nulls If set to `TRUE`, omits SMILES which were parsed as `NULL`
#' @param smiles.parser A SMILES parser object obtained from \code{\link{get.smiles.parser}}
#' @return A `list` of `jobjRef`s to their corresponding CDK `IAtomContainer` 
#' objects. If a SMILES string could not be parsed and `omit.nulls=TRUE` it 
#' is omited from the output list.
#' @seealso \code{\link{get.smiles}}, \code{\link{parse.smiles}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
parse.smiles <-
  function(smiles,
           kekulise = TRUE,
           omit.nulls = FALSE,
           smiles.parser = NULL) {
    if (!is.character(smiles)) {
      stop("Must supply a character vector of SMILES strings")
    }
    if (!is.null(smiles.parser)) {
      parser <- smiles.parser
    } else {
      parser <- get.smiles.parser()
    }
    .jcall(parser, "V", "kekulise", kekulise)
    returnValue_withnulls <- sapply(smiles,
                          function(x) {
                              mol <- tryCatch(
                              {
                                  .jcall(parser, "Lorg/openscience/cdk/interfaces/IAtomContainer;", "parseSmiles", x)
                              }, error = function(e) {
                                  return(NULL)
                              }
                              )
                              if (is.null(mol)){
                                  return(NULL)
                              } else {
                                  return(.jcast(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
                              }
                          })
    returnValue_nonulls <- Filter(Negate(is.null), returnValue_withnulls)
    returnValue <- returnValue_withnulls

    if (omit.nulls==TRUE) {
        returnValue <- returnValue_nonulls
    }

    nulls_count <- length(returnValue_withnulls)-length(returnValue_nonulls)

    if (nulls_count > 0) {
        warning(paste(nulls_count)," out of ",paste(length(returnValue_withnulls)),
        " SMILES were not successfully parsed, resulting in NULLs.")
    }
    return(returnValue)
}

test.get.smiles <- function()
{
  m <- parse.smiles('CCCC')[[1]]
  s <- get.smiles(m)
  checkEquals(s, 'CCCC')
}

test.get.smiles2 <- function() {
  m1 <- parse.smiles("CCCNCC")[[1]]
  m2 <- parse.smiles("CCNCCS")[[1]]
  mcs <- get.mcs(m1, m2)
  checkEquals("[CH2]CNCC", get.smiles(mcs, smiles.flavors(c('Unique'))))
}

library(rJava)

test.depictiongenerator <- function()
{
  
  mol <- parse.smiles("CC")
  
  #create a depiction  and write to SVG
  dg  <- .jnew("org.openscience.cdk.depict.DepictionGenerator")
  dg$withSize(512,512)$withAtomColors()
  temp1 <- paste0(tempfile(), ".svg")
  dg$depict(mol[[1]])$writeTo(temp1)
  
  #check SVG Output
  checkEquals("org.openscience.cdk.depict.DepictionGenerator", dg$getClass()$getName())
}

test.get.smiles <- function()
{
  m <- parse.smiles('CCCC')[[1]]
  s <- get.smiles(m)
  checkEquals(s, 'CCCC')
}

test.get.smiles2 <- function() {
  m1 <- parse.smiles("CCCNCC")[[1]]
  m2 <- parse.smiles("CNCCS")[[1]]
  mcs <- get.mcs(m1, m2)
  checkEquals("[CH2]NCC", get.smiles(mcs, smiles.flavors(c('Unique'))))
}

#' @name Molecule
#' 
#' @title Operations on molecules
#' 
#' @description 
#' Various functions to perform operations on molecules.
#' 
#' \code{\link{get.exact.mass}} returns the exact mass of a molecule
#' \code{\link{get.natural.mass}} returns the natural exact mass of a molecule
#' \code{\link{convert.implicit.to.explicit}} converts implicit hydrogens to 
#'     explicit hydrogens. This function does not return any value but rather 
#'     modifies the molecule object passed to it 
#' \code{\link{is.neutral}} returns \code{TRUE} if all atoms in the molecule have
#'     a formal charge of \code{0}, otherwise \code{FALSE}
#' 
#' @section Usage:
#' get.exact.mass(mol)
#' get.natural.mass(mol)
#' convert.implicit.to.explicit(mol)
#' is.neutral(mol)
#' 
#' @section Arguments:
#'  mol  A jobjRef representing an IAtomContainer or IMolecule object
#' 
#' @details 
#' 
#' In some cases, a molecule may not have any hydrogens (such as when read in 
#' from an MDL MOLfile that did not have hydrogens). In such cases,
#' \code{\link{convert.implicit.to.explicit}} will add implicit hydrogens and 
#' then convert them to explicit ones.  In addition, for such cases, make sure 
#' that the molecule has been typed beforehand.
#' 
#' @section Value:
#'  \code{\link{get.exact.mass}} returns a numeric 
#'  \code{\link{get.natural.mass}} returns a numeric
#'  \code{\link{convert.implicit.to.explicit}} has no return value
#'  \code{\link{is.neutral}} returns a boolean.
#'  
#' @seealso 
#' \code{\link{get.atoms}}, \code{\link{do.typing}}
#' @aliases Molecule
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
NULL

#' get.mcs
#' 
#' 
#' @param mol1 Required. First molecule to compare. Should be a `jobjRef` representing an `IAtomContainer`
#' @param mol2 Required. Second molecule to compare. Should be a `jobjRef` representing an `IAtomContainer`
#' @param as.molecule Optional. Default \code{TRUE}.
#' @export
get.mcs <- function(mol1, mol2, as.molecule = TRUE) {
  if (as.molecule) {
    return(.jcall("org.guha.rcdk.util.Misc",
           "Lorg/openscience/cdk/interfaces/IAtomContainer;",
           "getMcsAsNewContainer", mol1, mol2))
  } else {
    arr <- .jcall("org.guha.rcdk.util.Misc",
           "[[I",
           "getMcsAsAtomIndexMapping", mol1, mol2)
    do.call('rbind', lapply(arr, .jevalArray, rawJNIRefSignature = "[I"))
  }
}

is.subgraph <- function(query, target) {
  if (class(query) == 'character') {
    query <- parse.smiles(query)
    do.aromaticity(query)
    do.typing(query)
  }

  if (!is.list(target)) target <- list(target)
  if (!all(unlist(lapply(target, class)) == 'jobjRef'))
    stop("targets must be a list of IAtomContainer objects or a single IAtomContainer object")
  
  method <- 'TurboSubStructure'
  isoType <- .jcall("org.openscience.cdk.smsd.interfaces.Algorithm",
                    "Lorg/openscience/cdk/smsd/interfaces/Algorithm;",
                    "valueOf", method)
  iso <- .jnew("org.openscience.cdk.smsd.Isomorphism",
               isoType, TRUE);
  unlist(lapply(target, function(x) {
    .jcall(iso, "V", "init", query, x, TRUE, TRUE)
    .jcall(iso, "V", "setChemFilters", FALSE, FALSE, FALSE)
    .jcall(iso, "Z", "isSubgraph")
  }))
}

.packageName <- "rcdk"

#' Get the default chemical object builder.
#' 
#' The CDK employs a builder design pattern to construct
#' instances of new chemical objects (e.g., atoms, bonds, parsers
#' and so on). Many methods require an instance of a builder 
#' object to function. While most functions in this package handle
#' this internally, it is useful to be able to get an instance of
#' a builder object when directly working with the CDK API via
#' `rJava`.
#' 
#' This method returns an instance of the \href{http://cdk.github.io/cdk/2.2/docs/api/org/openscience/cdk/silent/SilentChemObjectBuilder.html}{SilentChemObjectBuilder}. 
#' Note that this is a static object that is created at package load time, 
#' and the same instance is returned whenever this function is called.
#' 
#' @return An instance of \href{http://cdk.github.io/cdk/2.3/docs/api/org/openscience/cdk/silent/SilentChemObjectBuilder.html}{SilentChemObjectBuilder}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.chem.object.builder <- function() {
  return(get("dcob", envir = .rcdk.GlobalEnv))
}

.check.class <- function(obj, klass) {
  !is.null(attr(obj, 'jclass')) && attr(obj, "jclass") == klass
}

.trim.whitespace <- function(x) {
  x <- gsub('^[[:space:]]+', '', x)
  gsub('[[:space:]]+$', '',x)
}

.javalist.to.rlist <- function(l) {
  size <- .jcall(l, "I", "size")
  if (size == 0) return(list())
  rl <- list()
  for (i in 1:size)
    rl[[i]] <- .jcall(l, "Ljava/lang/Object;", "get", as.integer(i-1))
  return(rl)
}

#'
#' @import fingerprint
#' @import methods
#' @import rJava
#' @import png
#' @import iterators
#' @import itertools
#' @import rcdklibs
#'
.onLoad <- function(lib, pkg) {
  dlp<-Sys.getenv("DYLD_LIBRARY_PATH")
  if (dlp!="") { # for Mac OS X we need to remove X11 from lib-path
    Sys.setenv("DYLD_LIBRARY_PATH"=sub("/usr/X11R6/lib","",dlp))
  }

  jar.rcdk <- paste(lib,pkg,"cont","rcdk.jar",sep=.Platform$file.sep)
  jar.png <- paste(lib,pkg,"cont","com.objectplanet.image.PngEncoder.jar",sep=.Platform$file.sep)
  .jinit(classpath=c(jar.rcdk,jar.png))
  
  # check Java Version 
  jv <- .jcall("java/lang/System", "S", "getProperty", "java.runtime.version")
  if(substr(jv, 1L, 2L) == "1.") {
    jvn <- as.numeric(paste0(strsplit(jv, "[.]")[[1L]][1:2], collapse = "."))
    if(jvn < 1.8) stop("Java >= 8 is needed for this package but not available")
  }

  ## generate some Java objects which get reused, so as to avoid repeated .jnew()
  nRule <- .jnew("org/openscience/cdk/formula/rules/NitrogenRule");
  rdbeRule <- .jnew("org/openscience/cdk/formula/rules/RDBERule");
  assign(".rcdk.GlobalEnv", new.env(parent = emptyenv()), envir = topenv())
  assign("nRule", nRule, envir = .rcdk.GlobalEnv)
  assign("rdbeRule", rdbeRule, envir = .rcdk.GlobalEnv)
  assign("dcob", .jcall("org/openscience/cdk/silent/SilentChemObjectBuilder",
                        "Lorg/openscience/cdk/interfaces/IChemObjectBuilder;",
                        "getInstance"), envir = .rcdk.GlobalEnv)
  assign("mfManipulator", .jnew("org/openscience/cdk/tools/manipulator/MolecularFormulaManipulator"), envir = .rcdk.GlobalEnv)
}

#' Get the current CDK version used in the package.
#' 
#' @return Returns a character containing the version of the CDK used in this package
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
cdk.version <- function() {
  .jcall("org.openscience.cdk.CDK", "S", "getVersion")
}

#' Remove explicit hydrogens.
#' 
#' Create an copy of the original structure with explicit hydrogens removed. 
#' Stereochemistry is updated but up and down bonds in a depiction may need 
#' to be recalculated. This can also be useful for descriptor calculations.
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @return A copy of the original molecule, with explicit hydrogens removed
#' @seealso \code{\link{get.hydrogen.count}}, \code{\link{get.total.hydrogen.count}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
remove.hydrogens <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  newmol <- .jcall('org/openscience/cdk/tools/manipulator/AtomContainerManipulator',
                   'Lorg/openscience/cdk/interfaces/IAtomContainer;',
                   'removeHydrogens',
                   mol);
  newmol
}

#' Get total number of implicit hydrogens in the molecule.
#' 
#' Counts the number of hydrogens on the provided molecule. As this method 
#' will sum all implicit hydrogens on each atom it is important to ensure 
#' the molecule has already been configured (and thus each atom has an 
#' implicit hydrogen count). 
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @return An integer representing the total number of implicit hydrogens
#' @seealso \code{\link{get.hydrogen.count}}, \code{\link{remove.hydrogens}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.total.hydrogen.count <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  .jcall('org/openscience/cdk/tools/manipulator/AtomContainerManipulator',
         'I',
         'getTotalHydrogenCount',
         mol);
}

#' get.exact.mass
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @export
get.exact.mass <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  
  
  formulaJ <- .jcall('org/openscience/cdk/tools/manipulator/MolecularFormulaManipulator',
                     "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
                     "getMolecularFormula",
                     mol,
                     use.true.class=FALSE);
  
  
  ret <- .jcall('org/openscience/cdk/tools/manipulator/MolecularFormulaManipulator',
                'D',
                'getTotalExactMass',
                formulaJ,
                check=FALSE)
  
  ex <- .jgetEx(clear=TRUE)
  
  
  if (is.null(ex)) return(ret)
  else{
    print(ex)
    stop("Couldn't get exact mass. Maybe you have not performed aromaticity, atom type or isotope configuration?")
  }
}
  

#' get.natural.mass
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @export
get.natural.mass <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  ret <- .jcall('org/openscience/cdk/tools/manipulator/AtomContainerManipulator',
                'D',
                'getNaturalExactMass',
                mol,
                check=FALSE)
  ex <- .jgetEx(clear=TRUE)
  if (is.null(ex)) return(ret)
  else{
    print(ex)
    stop("Couldn't get natural mass. Maybe you have not performed aromaticity, atom type or isotope configuration?")
  }  
}

#' get.total.charge
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @export
get.total.charge <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  
  ## check to see if we have partial charges
  atoms <- get.atoms(mol)
  pcharges <- unlist(lapply(atoms, get.charge))

  ## If any are null, partial charges were not set, so
  ## just return the total formal charge
  if (any(is.null(pcharges))) return(get.total.formal.charge(mol))
  else {
    .jcall('org/openscience/cdk/tools/manipulator/AtomContainerManipulator',
           'D',
           'getTotalCharge',
           mol);
  }
}

#' get.total.formal.charge
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @export
get.total.formal.charge <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  .jcall('org/openscience/cdk/tools/manipulator/AtomContainerManipulator',
         'I',
         'getTotalFormalCharge',
         mol);
}

#' Convert implicit hydrogens to explicit.
#' 
#' In some cases, a molecule may not have any hydrogens (such as when read
#' in from an MDL MOL file that did not have hydrogens or SMILES with no
#' explicit hydrogens). In such cases, this method
#' will add implicit hydrogens and then convert them to explicit ones. The 
#' newly added H's will not have any 2D or 3D coordinates associated with them.
#' Ensure that the molecule has been typed beforehand.
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @seealso \code{\link{get.hydrogen.count}}, \code{\link{remove.hydrogens}}, \code{\link{do.typing}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
convert.implicit.to.explicit <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")

    if (any(is.null(unlist(lapply(get.atoms(mol), .jcall, returnSig = "Ljava/lang/Integer;", method="getImplicitHydrogenCount"))))) {
    ## add them in
    dcob <- get.chem.object.builder()
    hadder <- .jcall("org/openscience/cdk/tools/CDKHydrogenAdder", "Lorg/openscience/cdk/tools/CDKHydrogenAdder;",
                     "getInstance", dcob)
    .jcall(hadder, "V", "addImplicitHydrogens", mol)
  }
  .jcall('org/openscience/cdk/tools/manipulator/AtomContainerManipulator', 'V', 'convertImplicitToExplicitHydrogens', mol)
}


#' Get the atoms from a molecule or bond.
#' 
#' @param object A `jobjRef` representing either a molecule (`IAtomContainer`) or 
#' bond (`IBond`) object.
#' @return A list of `jobjRef` representing the `IAtom` objects in the molecule or bond
#' @seealso \code{\link{get.bonds}}, \code{\link{get.connected.atoms}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.atoms <- function(object) {
  if (is.null(attr(object, 'jclass')))
    stop("object must be of class IAtomContainer or IObject or IBond")
  
  if (attr(object, 'jclass') != "org/openscience/cdk/interfaces/IAtomContainer" &&
      attr(object, 'jclass') != "org/openscience/cdk/interfaces/IObject" &&
      attr(object, 'jclass') != "org/openscience/cdk/interfaces/IBond")
    stop("object must be of class IAtomContainer or IObject or IBond")

  natom <- .jcall(object, "I", "getAtomCount")
  atoms <- list()
  for (i in 0:(natom-1))
    atoms[[i+1]] <- .jcall(object, "Lorg/openscience/cdk/interfaces/IAtom;", "getAtom", as.integer(i))
  atoms
}

#' Get the bonds in a molecule.
#' 
#' @param mol A `jobjRef` representing the molecule (`IAtomContainer`) object.
#' @return A list of `jobjRef` representing the bonds (`IBond`) objects in the molecule
#' @seealso \code{\link{get.atoms}}, \code{\link{get.connected.atoms}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.bonds <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  
  nbond <- .jcall(mol, "I", "getBondCount")
  bonds <- list()
  for (i in 0:(nbond-1))
    bonds[[i+1]] <- .jcall(mol, "Lorg/openscience/cdk/interfaces/IBond;", "getBond", as.integer(i))
  bonds
}

#' do.aromaticity
#' 
#' detect aromaticity of an input compound
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @export
do.aromaticity <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  
  model <- .jcall("org/openscience/cdk/aromaticity/ElectronDonation",
                  "Lorg/openscience/cdk/aromaticity/ElectronDonation;",
                  "daylight")
  cycles.all <- .jcall("org/openscience/cdk/graph/Cycles", 
                      "Lorg/openscience/cdk/graph/CycleFinder;",
                      "all")
  cycles.6 <- .jcall("org.openscience.cdk.graph.Cycles", 
                    "Lorg/openscience/cdk/graph/CycleFinder;",
                    "all", as.integer(6))
  cycles <- .jcall("org.openscience.cdk.graph.Cycles", 
                  "Lorg/openscience/cdk/graph/CycleFinder;",
                  "or", cycles.all, cycles.6)
  aromaticity <- .jnew("org/openscience/cdk.aromaticity/Aromaticity",
                       model, cycles)
  .jcall(aromaticity, "Z", "apply", mol)
}

#' do.typing
#' 
#' configure atom typings.
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @export
do.typing <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  
  .jcall("org.openscience.cdk.tools.manipulator.AtomContainerManipulator",
         "V", "percieveAtomTypesAndConfigureAtoms", mol)
}

#' do.isotopes
#' 
#' configure isotopes
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @export
do.isotopes <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  ifac <- .jcall('org.openscience.cdk.config.Isotopes',
                 'Lorg/openscience/cdk/config/Isotopes;',
                 'getInstance')
  .jcall(ifac, 'V', 'configureAtoms', mol)
}

#' Tests whether the molecule is neutral.
#' 
#' The test checks whether all atoms in the molecule have a formal charge of 0.
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @return `TRUE` if molecule is neutral, `FALSE` otherwise
#' @aliases charge
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
is.neutral <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  
  atoms <- get.atoms(mol)
  fc <- unlist(lapply(atoms, get.formal.charge))
  return(all(fc == 0))
}

#' Tests whether the molecule is fully connected.
#' 
#' A single molecule will be represented as a 
#' \href{https://en.wikipedia.org/wiki/Complete_graph}{complete} graph. 
#' In some cases, such as for molecules in salt form, or after certain 
#' operations such as bond splits, the molecular graph may contained 
#' \href{http://mathworld.wolfram.com/DisconnectedGraph.html}{disconnected components}.
#' This method can be used to tested whether the molecule is complete (i.e. fully
#' connected).
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @return `TRUE` if molecule is complete, `FALSE` otherwise
#' @seealso \code{\link{get.largest.component}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @examples 
#' m <- parse.smiles("CC.CCCCCC.CCCC")[[1]]
#' is.connected(m)
is.connected <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  
  .jcall("org.openscience.cdk.graph.ConnectivityChecker",
         "Z", "isConnected", mol)
}

#' Gets the largest component in a disconnected molecular graph.
#' 
#' A molecule may be represented as a 
#' \href{http://mathworld.wolfram.com/DisconnectedGraph.html}{disconnected graph}, such as
#' when read in as a salt form. This method will return the larges connected component
#' or if there is only a single component (i.e., the molecular graph is 
#' \href{https://en.wikipedia.org/wiki/Complete_graph}{complete} or fully connected), that
#' component is returned.
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @return The largest component as an `IAtomContainer` object or else the input molecule itself
#' @seealso \code{\link{is.connected}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @examples 
#' m <- parse.smiles("CC.CCCCCC.CCCC")[[1]]
#' largest <- get.largest.component(m)
#' length(get.atoms(largest)) == 6
get.largest.component <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  
  isConnected <- .jcall("org.openscience.cdk.graph.ConnectivityChecker",
                        "Z", "isConnected", mol)
  if (isConnected) return(mol)
  molSet <- .jcall("org.openscience.cdk.graph.ConnectivityChecker",
                   "Lorg/openscience/cdk/interfaces/IAtomContainerSet;",
                   "partitionIntoMolecules", mol)
  ncomp <- .jcall(molSet, "I", "getAtomContainerCount")
  max.idx <- -1
  max.atom.count <- -1
  for (i in seq_len(ncomp)) {
    m <- .jcall(molSet, "Lorg/openscience/cdk/interfaces/IAtomContainer;",
                "getAtomContainer", as.integer(i-1))
    natom <- .jcall(m, "I", "getAtomCount")
    if (natom > max.atom.count) {
      max.idx <- i
      max.atom.count <- natom
    }
  }
  m <- .jcall(molSet, "Lorg/openscience/cdk/interfaces/IAtomContainer;",
              "getAtomContainer", as.integer(max.idx-1))
  .jcast(m, "org/openscience/cdk/interfaces/IAtomContainer")
}

#' Get the number of atoms in the molecule.
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @return An integer representing the number of atoms in the molecule
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.atom.count <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  .jcall(mol, "I", "getAtomCount")
}

#' Get the title of the molecule.
#' 
#' Some molecules may not have a title (such as when parsing in a SMILES
#' with not title).
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @return A character string with the title, `NA` is no title is specified
#' @seealso \code{\link{set.title}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.title <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  get.property(mol, "cdk:Title")
}

#' Set the title of the molecule.
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @param title The title of the molecule as a character string. This will overwrite
#' any pre-existing title. The default value is an empty string.
#' @seealso \code{\link{get.title}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
set.title <- function(mol, title = "") {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  set.property(mol, "cdk:Title", title)
}

#' Generate 2D coordinates for a molecule.
#' 
#' Some file formats such as SMILES do not support 2D (or 3D) coordinates
#' for the atoms. Other formats such as SD or MOL have support for coordinates
#' but may not include them. This method will generate reasonable 2D coordinates 
#' based purely on connectivity information, overwriting
#' any existing coordinates if present. 
#' 
#' Note that when depicting a molecule (\code{\link{view.molecule.2d}}), 2D coordinates
#' are generated, but since it does not modify the input molecule, we do not have access
#' to the generated coordinates.
#' 
#' @param mol The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`
#' @return The input molecule, with 2D coordinates added
#' @seealso \code{\link{get.point2d}}, \code{\link{view.molecule.2d}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
generate.2d.coordinates <- function(mol) {
  if (!.check.class(mol, "org/openscience/cdk/interfaces/IAtomContainer"))
    stop("molecule must be of class IAtomContainer")
  
  .jcall('org/guha/rcdk/util/Misc', 'Lorg/openscience/cdk/interfaces/IAtomContainer;',
         'getMoleculeWithCoordinates', mol)
}

test.is.connected <- function()
{
  m <- parse.smiles('CCCC')[[1]]
  connected <- is.connected(m)
  checkTrue(connected)
  m <- parse.smiles('CCCC.CCCC')[[1]]
  connected <- is.connected(m)
  checkTrue(!connected)
}

test.get.largest <- function() {
  m <- parse.smiles('CCCC')[[1]]
  l <- get.largest.component(m)
  checkEquals(length(get.atoms(l)), 4)

  m <- parse.smiles('CCCC.CCCCCC.CC')[[1]]
  l <- get.largest.component(m)
  checkEquals(length(get.atoms(l)), 6)
}

test.atom.count <- function() {
  m <- parse.smiles("CCC")[[1]]
  natom <- get.atom.count(m)
  checkEquals(natom, 3)

  convert.implicit.to.explicit(m)
  natom <- get.atom.count(m)
  checkEquals(natom, 11)
}

test.is.neutral <- function() {
  m <- parse.smiles("CCC")[[1]]
  checkTrue(is.neutral(m))
  m <- parse.smiles('[O-]CC')[[1]]
  checkTrue(!is.neutral(m))
}

test.formula <- function() {
   formula_file <- system.file("molfiles/formulatest.mol", package = "rcdk")
   m <- load.molecules(formula_file)
   f1 <- get.mol2formula(m[[1]])
   checkEquals(f1@string, "C35H64N3O21P3S")
   m <- parse.smiles("C1(C(C(C(C(C1OP(=O)(O)OCC(COC(=O)CCCCCCCNC(=O)CCCCC2SCC3C2NC(=O)N3)OC(=O)CCCCCCC)O)OP(=O)(O)O)OP(=O)(O)O)O)O")[[1]]
   do.aromaticity(m)
   set.atom.types(m)
   do.isotopes(m)
   convert.implicit.to.explicit(m)
   f2 <- get.mol2formula(m)
   checkEquals(f2@string, "C35H64N3O21P3S")
}

test.desc.cats <- function() {
  cats <- get.desc.categories()
  print(cats)
  checkEquals(6, length(cats))
}

test.desc.names <- function() {
  cats <- get.desc.categories()
  for (acat in cats) {
    dnames <- get.desc.names(acat)
    checkTrue(length(dnames) > 0)
  }
}

test.desc.calc <- function() {
  dnames <- get.desc.names("topological")
  mols <- parse.smiles("c1ccccc1CCC")
  dvals <- eval.desc(mols, dnames[1])
  checkTrue(dvals[1,1] == 1)
}

test.exact.natural.mass <- function() {
  # Skip on CRAN - isotope calculations are computationally intensive and trigger multi-threading
  if (Sys.getenv("_R_CHECK_PACKAGE_NAME_", "") != "" || Sys.getenv("_R_CHECK_TIMINGS_", "") != "") {
    return(TRUE)
  }
  smiles <- c("CCNC1=NC(NC(C)C)=NC(Cl)=N1", # normal atrazine, DTXSID9020112
              "[2H]N(CC)C1=NC(=NC(Cl)=N1)N([2H])C(C)C", #2H on implicit locations, DTXSID40892885
              "[2H]C([2H])([2H])C([2H])([2H])NC1=NC(Cl)=NC(NC(C)C)=N1", #d5, DTXSID20486781
              "CC[15NH]C1=NC(NC(C)C)=NC(Cl)=N1", #15N DTXSID40583908
              "OC1=C(Br)C(Br)=C(Br)C(Br)=C1Br", #pentabromophenol, DTXSID9022079
              "C[Se]CC[C@H](N)C(O)=O" # Selenium-L-methionine, DTXSID8046824
  )
  #atrzine
  m <- parse.smiles(smiles[1])[[1]]
  do.aromaticity(m)
  set.atom.types(m)
  do.isotopes(m)
  convert.implicit.to.explicit(m)
  # Dashboard ref mass: 215.093773, 215.69
  checkEquals(get.exact.mass(m),215.0938, tolerance=1e-6)
  checkEquals(get.natural.mass(m),215.6835, tolerance=1e-6)

  #deuterium on exchangeable locations
  m <- parse.smiles(smiles[2])[[1]]
  do.aromaticity(m)
  set.atom.types(m)
  do.isotopes(m)
  convert.implicit.to.explicit(m)
  # Dashboard ref mass: 217.106327, 217.7
  checkEquals(get.exact.mass(m),217.1063, tolerance=1e-6)
  #checkEquals(get.natural.mass(m),215.6835) #this is wrong! It should be 217.7something

  #deuterium on fixed locations
  m <- parse.smiles(smiles[3])[[1]]
  do.aromaticity(m)
  set.atom.types(m)
  do.isotopes(m)
  convert.implicit.to.explicit(m)
  # Dashboard ref mass: 220.125157, 220.72
  checkEquals(get.exact.mass(m),220.1252, tolerance=1e-6)
  #checkEquals(get.natural.mass(m),215.6835) #this is wrong! It should be 220.7something

  #15N-atrazine
  m <- parse.smiles(smiles[4])[[1]]
  do.aromaticity(m)
  set.atom.types(m)
  do.isotopes(m)
  convert.implicit.to.explicit(m)
  # Dashboard ref mass: 216.090808, 216.68
  checkEqualsNumeric(get.exact.mass(m),216.0908, tolerance=1e-6)
  #checkEquals(get.natural.mass(m),215.6835) #this is wrong! It should be 216.68something

  #pentabromophenol, DTXSID9022079 - tricky as lots of Br shifts pattern
  m <- parse.smiles(smiles[5])[[1]]
  do.aromaticity(m)
  set.atom.types(m)
  do.isotopes(m)
  convert.implicit.to.explicit(m)
  # Dashboard ref mass: 483.59443, 488.593
  checkEquals(get.exact.mass(m),483.5944, tolerance=1e-6)
  checkEquals(get.natural.mass(m),488.5894, tolerance=1e-6)

  # Selenium-L-methionine, DTXSID8046824 - tricky as Se primary isotope not lowest mass
  m <- parse.smiles(smiles[6])[[1]]
  do.aromaticity(m)
  set.atom.types(m)
  do.isotopes(m)
  convert.implicit.to.explicit(m)
  # Dashboard ref mass: 196.995501, 196.119
  checkEquals(get.exact.mass(m),196.9955, tolerance=1e-6)
  checkEquals(get.natural.mass(m),196.1059, tolerance=1e-6) #quite a discrepancy in ref value


}

test.is.aromatic <- function()
{
  m <- parse.smiles('c1ccccc1CC')[[1]]
  x <- unlist(lapply(get.atoms(m), is.aromatic))
  checkEquals(6, length(which(x)))
  set.atom.types(m)
  do.aromaticity(m)
  x <- unlist(lapply(get.atoms(m), is.aromatic))
  checkEquals(6, length(which(x)))  
}

test.get.hcount <- function() {
  m <- parse.smiles('c1ccccc1')[[1]]
  x <- unlist(lapply(get.atoms(m), get.hydrogen.count))
  checkEquals(1, unique(x))
}


test.charges <- function() {
  m <- parse.smiles("CCC")[[1]]
  a <- get.atoms(m)
  for (atom in a) {
    checkTrue(is.null(get.charge(atom)))
  }

  m <- parse.smiles("[O-]CC")[[1]]
  a <- get.atoms(m)
  checkTrue(is.null(get.charge(a[[1]])))

  checkEquals(-1, get.formal.charge(a[[1]]))
  checkEquals(0, get.formal.charge(a[[2]]))
  checkEquals(0, get.formal.charge(a[[3]]))  
}

test.frag1 <- function() {
  m <- parse.smiles("c1(ccc(cc1C)CCC(C(CCC)C2C(C2)CC)C3C=C(C=C3)CC)C")[[1]]
  do.aromaticity(m)  
  do.typing(m)
  f <- get.murcko.fragments(m, as.smiles=TRUE, min.frag.size = 6, single.framework = TRUE)
  checkEquals(length(f), 1)
  checkEquals(length(f[[1]]$rings), 1)
  checkEquals(f[[1]]$rings, "c1ccccc1")
  checkEquals(f[[1]]$frameworks, "c1ccc(cc1)CCC(CC2CC2)C3C=CC=C3")
}

test.frag2 <- function() {
  ms <- parse.smiles(c('c1(ccc(cc1C)CCC(C(CCC)C2C(C2)CC)C3C=C(C=C3)CC)C',
                       'c1ccc(cc1)c2c(oc(n2)N(CCO)CCO)c3ccccc3',
                       'COc1ccc(cc1OCc2ccccc2)C(=S)N3CCOCC3'))
  lapply(ms, do.aromaticity)  
  lapply(ms, do.typing)
  f <- get.murcko.fragments(ms, as.smiles=TRUE, min.frag.size = 6, single.framework = TRUE)
  checkEquals(length(f), 3)

  fworks <- unlist(lapply(f, function(x) length(x$frameworks)))
  checkTrue(all(fworks == 1))
}

test.frag3 <- function() {
  ms <- parse.smiles(c('c1(ccc(cc1C)CCC(C(CCC)C2C(C2)CC)C3C=C(C=C3)CC)C',
                       'c1ccc(cc1)c2c(oc(n2)N(CCO)CCO)c3ccccc3',
                       'COc1ccc(cc1OCc2ccccc2)C(=S)N3CCOCC3'))
  lapply(ms, do.aromaticity)  
  lapply(ms, do.typing)
  f <- get.murcko.fragments(ms, as.smiles=FALSE, min.frag.size = 6, single.framework = TRUE)
  checkEquals(length(f), 3)

  fworks <- unlist(lapply(f, function(x) unlist(lapply(x$frameworks, .jclass))))
  checkTrue(all(fworks == "org.openscience.cdk.silent.AtomContainer2"))
}

test.is.connected <- function()
{
  m <- parse.smiles('CCCC')[[1]]
  connected <- is.connected(m)
  checkTrue(connected)
  m <- parse.smiles('CCCC.CCCC')[[1]]  
  connected <- is.connected(m)
  checkTrue(!connected)  
}

test.get.largest <- function() {
  m <- parse.smiles('CCCC')[[1]]
  l <- get.largest.component(m)
  checkEquals(length(get.atoms(l)), 4)

  m <- parse.smiles('CCCC.CCCCCC.CC')[[1]]
  l <- get.largest.component(m)
  checkEquals(length(get.atoms(l)), 6)  
}

test.atom.count <- function() {
  m <- parse.smiles("CCC")[[1]]
  natom <- get.atom.count(m)
  checkEquals(natom, 3)

  convert.implicit.to.explicit(m)
  natom <- get.atom.count(m)
  checkEquals(natom, 11)  
}

test.is.neutral <- function() {
  m <- parse.smiles("CCC")[[1]]
  checkTrue(is.neutral(m))
  m <- parse.smiles('[O-]CC')[[1]]
  checkTrue(!is.neutral(m))
}

test.formula <- function() {
   formula_file <- system.file("molfiles/formulatest.mol", package = "rcdk")
   m <- load.molecules(formula_file)
   f1 <- get.mol2formula(m[[1]])
   checkEquals(f1@string, "C35H64N3O21P3S")
   m <- parse.smiles("C1(C(C(C(C(C1OP(=O)(O)OCC(COC(=O)CCCCCCCNC(=O)CCCCC2SCC3C2NC(=O)N3)OC(=O)CCCCCCC)O)OP(=O)(O)O)OP(=O)(O)O)O)O")[[1]]
   do.aromaticity(m)
   do.typing(m)
   do.isotopes(m)
   convert.implicit.to.explicit(m)
   f2 <- get.mol2formula(m)
   checkEquals(f2@string, "C35H64N3O21P3S")  
}

test.desc.cats <- function() {
  cats <- get.desc.categories()
  print(cats)
  checkEquals(5, length(cats))
}

test.desc.names <- function() {
  cats <- get.desc.categories()
  for (acat in cats) {
    dnames <- get.desc.names(acat)
    checkTrue(length(dnames) > 0)
  }
}

test.desc.calc <- function() {
  dnames <- get.desc.names("topological")
  mols <- parse.smiles("c1ccccc1CCC")
  dvals <- eval.desc(mols, dnames[1])
  checkTrue(dvals[1,1] == 1)
}

test.exact.natural.mass <- function() {
  smiles <- c("CCNC1=NC(NC(C)C)=NC(Cl)=N1", # normal atrazine, DTXSID9020112
              "[2H]N(CC)C1=NC(=NC(Cl)=N1)N([2H])C(C)C", #2H on implicit locations, DTXSID40892885
              "[2H]C([2H])([2H])C([2H])([2H])NC1=NC(Cl)=NC(NC(C)C)=N1", #d5, DTXSID20486781
              "CC[15NH]C1=NC(NC(C)C)=NC(Cl)=N1", #15N DTXSID40583908
              "OC1=C(Br)C(Br)=C(Br)C(Br)=C1Br", #pentabromophenol, DTXSID9022079
              "C[Se]CC[C@H](N)C(O)=O" # Selenium-L-methionine, DTXSID8046824
  ) 
  #atrzine
  m <- parse.smiles(smiles[1])[[1]]
  do.aromaticity(m)
  do.typing(m)
  do.isotopes(m)
  convert.implicit.to.explicit(m)
  # Dashboard ref mass: 215.093773, 215.69
  checkEquals(get.exact.mass(m),215.0938, tolerance=1e-6)
  checkEquals(get.natural.mass(m),215.6835, tolerance=1e-6)
  
  #deuterium on exchangeable locations
  m <- parse.smiles(smiles[2])[[1]]
  do.aromaticity(m)
  do.typing(m)
  do.isotopes(m)
  convert.implicit.to.explicit(m)
  # Dashboard ref mass: 217.106327, 217.7
  checkEquals(get.exact.mass(m),217.1063, tolerance=1e-6)
  #checkEquals(get.natural.mass(m),215.6835) #this is wrong! It should be 217.7something
  
  #deuterium on fixed locations
  m <- parse.smiles(smiles[3])[[1]]
  do.aromaticity(m)
  do.typing(m)
  do.isotopes(m)
  convert.implicit.to.explicit(m)
  # Dashboard ref mass: 220.125157, 220.72
  checkEquals(get.exact.mass(m),220.1252, tolerance=1e-6)
  #checkEquals(get.natural.mass(m),215.6835) #this is wrong! It should be 220.7something
  
  #15N-atrazine
  m <- parse.smiles(smiles[4])[[1]]
  do.aromaticity(m)
  do.typing(m)
  do.isotopes(m)
  convert.implicit.to.explicit(m)
  # Dashboard ref mass: 216.090808, 216.68
  checkEqualsNumeric(get.exact.mass(m),216.0908, tolerance=1e-6)
  #checkEquals(get.natural.mass(m),215.6835) #this is wrong! It should be 216.68something
  
  #pentabromophenol, DTXSID9022079 - tricky as lots of Br shifts pattern
  m <- parse.smiles(smiles[5])[[1]]
  do.aromaticity(m)
  do.typing(m)
  do.isotopes(m)
  convert.implicit.to.explicit(m)
  # Dashboard ref mass: 483.59443, 488.593
  checkEquals(get.exact.mass(m),483.5944, tolerance=1e-6)
  checkEquals(get.natural.mass(m),488.5894, tolerance=1e-6) 
  
  # Selenium-L-methionine, DTXSID8046824 - tricky as Se primary isotope not lowest mass
  m <- parse.smiles(smiles[6])[[1]]
  do.aromaticity(m)
  do.typing(m)
  do.isotopes(m)
  convert.implicit.to.explicit(m)
  # Dashboard ref mass: 196.995501, 196.119
  checkEquals(get.exact.mass(m),196.9955, tolerance=1e-6)
  checkEquals(get.natural.mass(m),196.1059, tolerance=1e-6) #quite a discrepancy in ref value
  
  
}

test.fp <- function() {
    mol <- parse.smiles("CCCCC")[[1]]
    fp <- get.fingerprint(mol, type='maccs')
    checkTrue(length(fp@bits) > 0)
    fp <- get.fingerprint(mol, type='kr')
    checkTrue(length(fp@bits) > 0)
    fp <- get.fingerprint(mol, type='shortestpath')
    checkTrue(length(fp@bits) > 0)
}

# Substructure test are inspired by the test for the substructure fingerprints in CDK
test.fp.substructures.binary <- function() {
    # Default patterns: functional groups
    mol <- parse.smiles("c1ccccc1CCC")[[1]]
    fp <- get.fingerprint(mol, type="substructure", fp.mode="bit")
    fp_bits <- fingerprint::fp.to.matrix(list(fp))
    
    checkEquals(length(fp), 307)
    checkEquals(fp_bits[1], 1)
    checkEquals(fp_bits[2], 1)
    checkEquals(fp_bits[274], 1)
    checkEquals(fp_bits[101], 0)
    
    # User defined patterns
    smarts <- c("c1ccccc1", "[CX4H3][#6]", "[CX2]#[CX2]")
    mol <- parse.smiles("c1ccccc1CCC")[[1]]
    fp <- get.fingerprint(mol, type="substructure", fp.mode="bit", 
                          substructure.pattern = smarts)
    fp_bits <- fingerprint::fp.to.matrix(list(fp))
    
    checkEquals(length(fp), 3)
    checkEquals(length(fp@bits), 2)
    checkEquals(fp_bits[1], 1)
    checkEquals(fp_bits[2], 1)
    checkEquals(fp_bits[3], 0)
    
    mol <- parse.smiles("C=C=C")[[1]]
    fp <- get.fingerprint(mol, type="substructure", fp.mode="bit", 
                          substructure.pattern = smarts)
    fp_bits <- fingerprint::fp.to.matrix(list(fp))
    
    checkEquals(length(fp), 3)
    checkEquals(length(fp@bits), 0)
    for (i_fp in 1:3) {
        checkEquals(fp_bits[i_fp], 0)
    }
    
    # Check for aromatic ring
    smarts <- "a:1:a:a:a:a:a1"
    mol <- parse.smiles("C1=CC=CC(=C1)CCCC2=CC=CC=C2")[[1]]
    do.typing(mol)
    do.aromaticity(mol)
    fp <- get.fingerprint(mol, type="substructure", fp.mode="bit", 
                          substructure.pattern = smarts)
    fp_bits <- fingerprint::fp.to.matrix(list(fp))
    checkEquals(length(fp), 1)
    checkEquals(fp_bits[1], 1)
}

test.fp.substructures.count <- function() {
    # Default patterns: functional groups
    mol <- parse.smiles("c1ccccc1CCC")[[1]]
    fp <- get.fingerprint(mol, type="substructure", fp.mode="count")
    
    checkEquals(length(fp), 307)
    checkTrue(fingerprint::count(fp@features[[1]]) > 0)
    checkTrue(fingerprint::count(fp@features[[2]]) > 0)
    checkTrue(fingerprint::count(fp@features[[274]]) > 0)
    checkTrue(fingerprint::count(fp@features[[101]]) == 0)
    
    # User defined patterns
    smarts <- c("c1ccccc1", "[CX4H3][#6]", "[CX2]#[CX2]")
    mol <- parse.smiles("c1ccccc1CCC")[[1]]
    fp <- get.fingerprint(mol, type="substructure", fp.mode="count", 
                          substructure.pattern = smarts)
    
    checkEquals(length(fp), 3)
    checkEquals(fingerprint::count(fp@features[[1]]), 1)
    checkEquals(fingerprint::count(fp@features[[2]]), 1)
    checkEquals(fingerprint::count(fp@features[[3]]), 0)
    
    mol <- parse.smiles("C=C=C")[[1]]
    fp <- get.fingerprint(mol, type="substructure", fp.mode="count", 
                          substructure.pattern = smarts)
    
    checkEquals(length(fp), 3)
    for (i_fp in 1:3) {
        checkEquals(fingerprint::count(fp@features[[i_fp]]), 0)
    }
    
    # Check for aromatic ring
    smarts <- "a:1:a:a:a:a:a1"
    mol <- parse.smiles("C1=CC=CC(=C1)CCCC2=CC=CC=C2")[[1]]
    do.typing(mol)
    do.aromaticity(mol)
    fp <- get.fingerprint(mol, type="substructure", fp.mode="count", 
                          substructure.pattern = smarts)
    
    checkEquals(length(fp), 1)
    checkEquals(fingerprint::count(fp@features[[1]]), 2)
}

test.is.aromatic <- function()
{
  m <- parse.smiles('c1ccccc1CC')[[1]]
  x <- unlist(lapply(get.atoms(m), is.aromatic))
  checkEquals(6, length(which(x)))
  do.typing(m)
  do.aromaticity(m)
  x <- unlist(lapply(get.atoms(m), is.aromatic))
  checkEquals(6, length(which(x)))  
}

test.get.hcount <- function() {
  m <- parse.smiles('c1ccccc1')[[1]]
  x <- unlist(lapply(get.atoms(m), get.hydrogen.count))
  checkEquals(1, unique(x))
}


test.charges <- function() {
  m <- parse.smiles("CCC")[[1]]
  a <- get.atoms(m)
  for (atom in a) {
    checkTrue(is.null(get.charge(atom)))
  }

  m <- parse.smiles("[O-]CC")[[1]]
  a <- get.atoms(m)
  checkTrue(is.null(get.charge(a[[1]])))

  checkEquals(-1, get.formal.charge(a[[1]]))
  checkEquals(0, get.formal.charge(a[[2]]))
  checkEquals(0, get.formal.charge(a[[3]]))  
}

## ---- eval=FALSE--------------------------------------------------------------
#  
#  # Bioconductor Packages. Use BiocManager::install()
#  #   Rdisop MetaboCoreUtils ChemmineR ChemmineOB enviPat
#  
#  library(plyr)
#  library(CHNOSZ)
#  library(enviPat)
#  library(MetaboCoreUtils)
#  library(rcdk)
#  library(ChemmineR)
#  library(OrgMassSpecR)
#  library(Rdisop)
#  #library(ChemmineOB)
#  
#  data(isotopes)
#  
#  # original
#  # https://github.com/CDK-R/cdkr/blob/master/rcdk/R/formula.R
#  # get.formula <- function(mf, charge=0) {
#  #
#  #   manipulator <- get("mfManipulator", envir = .rcdk.GlobalEnv)
#  #   if(!is.character(mf)) {
#  #     stop("Must supply a Formula string");
#  #   }else{
#  #     dcob <- .cdkFormula.createChemObject()
#  #     molecularformula <- .cdkFormula.createFormulaObject()
#  #     molecularFormula <- .jcall(manipulator,
#  #                                "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
#  #                                "getMolecularFormula",
#  #                                mf,
#  #                                .jcast(molecularformula,.IMolecularFormula),
#  #                                TRUE);
#  #   }
#  #
#  #   D <- new(J("java/lang/Integer"), as.integer(charge))
#  #   .jcall(molecularFormula,"V","setCharge",D);
#  #   object <- .cdkFormula.createObject(.jcast(molecularFormula,.IMolecularFormula));
#  #   return(object);
#  # }
#  
#  
#  mfManipulator    <- J("org/openscience/cdk/tools/manipulator/MolecularFormulaManipulator")
#  silentchemobject <- J("org.openscience.cdk.silent.SilentChemObjectBuilder")
#  
#  
#  #' Rewrite the formual object and directly access Java
#  #'
#  get.formula2 <- function(mf) {
#  
#    formula <- mfManipulator$getMolecularFormula(
#      "C2H3",
#      silentchemobject$getInstance())
#  
#    mfManipulator$getMass(formula)
#  
#  }
#  
#  #' Add type hints
#  #'
#  get.formula3 <- function(mf) {
#    builderinstance <- .jcall(
#        silentchemobject,
#       "Lorg/openscience/cdk/interfaces/IChemObjectBuilder;",
#       "getInstance")
#  
#    formula  <- .jcall(
#        mfManipulator,
#       "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
#       "getMolecularFormula",
#        mf,
#        builderinstance);
#  
#    mfManipulator$getMass(formula)
#  
#  }
#  
#  
#  #' Add type hints
#  #'
#  get.formula4 <- function(mf) {
#    builderinstance <- .jcall(
#        silentchemobject,
#        "Lorg/openscience/cdk/interfaces/IChemObjectBuilder;",
#        "getInstance")
#  
#    formula  <- .jcall(
#        mfManipulator,
#       "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
#       "getMolecularFormula",
#       mf,
#       builderinstance);
#  
#    .jcall(
#        mfManipulator,
#        "D",
#        "getMass",
#       formula)
#  }
#  
#  
#  
#  benchmark <- microbenchmark::microbenchmark(
#    MetaboCoreUtils = MetaboCoreUtils::calculateMass("C2H6O"),
#    rcdk = rcdk::get.formula("C2H6O", charge = 0)@mass,
#    rcdk2 = get.formula2("C2H6O"),
#    rcdk3 = get.formula3("C2H6O"),
#    rcdk4 = get.formula4("C2H6O"),
#    Rdisop = Rdisop::getMolecule("C2H6O")$exactmass,
#    ChemmineR = ChemmineR::exactMassOB(ChemmineR::smiles2sdf("CCO")),
#    OrgMassSpecR = OrgMassSpecR::MonoisotopicMass(formula = OrgMassSpecR::ListFormula("C2H6O)"), charge = 0),
#  
#    CHNOSZ = CHNOSZ::mass("C2H6O"),
#    enviPat = enviPat::isopattern(isotopes, "C2H6O", charge=FALSE, verbose=FALSE)[[1]][1,1]
#    , times=1000L)
#  
#  
#  masses <- c(
#    MetaboCoreUtils=MetaboCoreUtils::calculateMass("C2H6O"),
#    rcdk=rcdk::get.formula("C2H6O", charge = 0)@mass,
#    Rdisop=Rdisop::getMolecule("C2H6O")$exactmass,
#    #ChemmineR=ChemmineR::exactMassOB(ChemmineR::smiles2sdf("CCO")),
#    OrgMassSpecR=OrgMassSpecR::MonoisotopicMass(formula = OrgMassSpecR::ListFormula("C2H6O)"), charge = 0),
#    CHNOSZ=CHNOSZ::mass("C2H6O"),
#    enviPat=enviPat::isopattern(isotopes, "C2H6O", charge=FALSE, verbose=FALSE)[[1]][1,1]
#  )
#  
#  options(digits=10)
#  t(t(sort(masses)))
#  summary(benchmark)[order(summary(benchmark)[,"median"]) , ]
#  clipr::write_clip(as.data.frame(summary(benchmark)[order(summary(benchmark)[,"median"]) , ] ))

#' @name Atoms
#' @title Operations on Atoms
#' 
#' @description 
#' \code{\link{get.symbol}} returns the chemical symbol for an atom
#' \code{\link{get.point3d}} returns the 3D coordinates of the atom
#' \code{\link{get.point2d}} returns the 2D coordinates of the atom
#' \code{\link{get.atomic.number}} returns the atomic number of the atom
#' \code{\link{get.hydrogen.count}}  returns the number of implicit H’s on the atom. 
#'           Depending on where the molecule was read from this may be \code{NULL}
#'           or an integer greater than or equal to \code{0}
#' \code{\link{get.charge}} returns the partial charge on the atom. If charges 
#'           have not been set the return value is \code{NULL}, otherwise 
#'           the appropriate charge.
#' \code{\link{get.formal.charge}} returns the formal charge on the atom. By 
#'            default the formal charge will be \code{0} (i.e., \code{NULL}
#'            is never returned)
#' \code{\link{is.aromatic}} returns \code{TRUE} if the atom is aromatic,
#'            \code{FALSE} otherwise
#' \code{\link{is.aliphatic}} returns \code{TRUE} if the atom is part of an 
#'            aliphatic chain, \code{FALSE} otherwise
#' \code{\link{is.in.ring}} returns \code{TRUE} if the atom is in a ring, 
#'             \code{FALSE} otherwise
#' \code{\link{get.atom.index}} eturns the index of the atom in the molecule
#'             (starting from \code{0})
#' \code{\link{get.connected.atoms}} returns a list of atoms that are connected to the specified atom
#' 
#' @section Usage:
#' get.symbol(atom)
#' get.point3d(atom)
#' get.point2d(atom)
#' get.atomic.number(atom)
#' get.hydrogen.count(atom)
#' get.charge(atom)
#' get.formal.charge(atom)
#' get.connected.atoms(atom, mol)
#' get.atom.index(atom, mol)
#' is.aromatic(atom)
#' is.aliphatic(atom)
#' is.in.ring(atom) 
#' 
#' 
#' @section Arguments:
#'  atom A jobjRef representing an IAtom object
#'  mol  A jobjRef representing an IAtomContainer object
#' 
#' @section Value:
#'  In the case of \code{\link{get.point3d}} the return value is a 3-element vector 
#'  containingthe X, Y and Z co-ordinates of the atom.  If the atom does not 
#'  have 3D coordinates, it returns a vector of the form \code{c(NA,NA,NA)}.
#'  Similarly for \code{\link{get.point2d}}, in which case the return vector is of 
#'  length \code{2}.
#'  
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @docType package
NULL

## An example of getting all the coordinates for a molecule
## atoms <- get.atoms(mol)
## coords <- do.call('rbind', lapply(apply, get.point3d))

.valid.atom <- function(atom) {
  if (is.null(attr(atom, 'jclass'))) stop("Must supply an Atom or IAtom object")
  if (!.check.class(atom, "org/openscience/cdk/interfaces/IAtom") &&
      !.check.class(atom, "org/openscience/cdk/Atom"))
    stop("Must supply an Atom or IAtom object")

  if (.check.class(atom, "org/openscience/cdk/Atom"))
    atom <- .jcast(atom, "org/openscience/cdk/interfaces/IAtom")
  return(atom)
}

#' @name get.point3d
#' @title Get the 3D coordinates of the atom.
#' 
#' In case, coordinates are unavailable (e.g., molecule was read in from a 
#' SMILES file) or have not been generated yet, `NA`'s are returned for the 
#' X, Y and Z coordinates.
#' 
#' @param atom The atom to query
#' @return A 3-element numeric vector representing the X, Y and Z coordinates.
#' @seealso \code{\link{get.point2d}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @examples 
#' \dontrun{
#' atoms <- get.atoms(mol)
#' coords <- do.call('rbind', lapply(apply, get.point3d))
#' }
get.point3d <- function(atom) {
  atom <- .valid.atom(atom)
  p3d <- .jcall(atom, "Ljavax/vecmath/Point3d;", "getPoint3d")
  if (is.jnull(p3d)) return( c(NA,NA,NA) )
  else {
    c(.jfield(p3d, name='x'),
      .jfield(p3d, name='y'),
      .jfield(p3d, name='z'))
  }
}

#' @name get.point2d
#' @title Get the 2D coordinates of the atom.
#' 
#' In case, coordinates are unavailable (e.g., molecule was read in from a 
#' SMILES file) or have not been generated yet, `NA`'s are returned for the 
#' X & Y coordinates.
#' 
#' @param atom The atom to query
#' @return A 2-element numeric vector representing the X & Y coordinates.
#' @seealso \code{\link{get.point3d}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @examples 
#' \dontrun{
#' atoms <- get.atoms(mol)
#' coords <- do.call('rbind', lapply(apply, get.point2d))
#' }
get.point2d <- function(atom) {
  atom <- .valid.atom(atom)    
  p3d <- .jcall(atom, "Ljavax/vecmath/Point2d;", "getPoint2d")
  if (is.jnull(p3d)) return( c(NA,NA) )
  else {
    c(.jfield(p3d, name='x'),
      .jfield(p3d, name='y'))
  }
}

#' @name get.symbol
#' @title Get the atomic symbol of the atom.
#' 
#' Get the atomic symbol of the atom.
#' 
#' @param atom The atom to query
#' @return A character representing the atomic symbol
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.symbol <- function(atom) {
  atom <- .valid.atom(atom)  
  .jcall(atom, "S", "getSymbol")
}

#' @name get.atomic.number
#' @title Get the atomic number of the atom.
#' 
#' @param atom The atom to query
#' @return An integer representing the atomic number
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.atomic.number <- function(atom) {
  atom <- .valid.atom(atom)
  .jcall(.jcall(atom, "Ljava/lang/Integer;", "getAtomicNumber"), "I", "intValue")
}

#' @name get.charge
#' @title Get the charge on the atom.
#' 
#' This method returns the partial charge on the atom. If charges have not been set the
#' return value is \code{NULL}, otherwise the appropriate charge.
#' 
#' @param atom The atom to query
#' @return An numeric representing the partial charge. If charges have not been set,
#' `NULL` is returned
#' @seealso \code{\link{get.formal.charge}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.charge <- function(atom) {
  atom <- .valid.atom(atom)
  ch <- .jcall(atom, "Ljava/lang/Double;", "getCharge")
  if (!is.null(ch)) return(.jcall(ch, "D", "doubleValue"))
  else return(ch)
}

#' @name get.formal.charge
#' @title Get the formal charge on the atom.
#' 
#' By default the formal charge will be 0 (i.e., \code{NULL} is never returned).
#' 
#' @param atom The atom to query
#' @return An integer representing the formal charge
#' @seealso \code{\link{get.charge}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.formal.charge <- function(atom) {
  atom <- .valid.atom(atom)
  ch <- .jcall(atom, "Ljava/lang/Integer;", "getFormalCharge")
  if (!is.null(ch)) return(.jcall(ch, "I", "intValue"))
  else return(ch)  
}

#' @name get.hydrogen.count
#' @title Get the implicit hydrogen count for the atom.
#' 
#' This method returns the number of implicit H's on the atom. 
#' Depending on where the molecule was read from this may be \code{NULL} or an integer
#' greater than or equal to 0
#' 
#' @param atom The atom to query
#' @return An integer representing the hydrogen count
#' @aliases hydrogen
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.hydrogen.count <- function(atom) {
  atom <- .valid.atom(atom)  
  .jcall(.jcall(atom, "Ljava/lang/Integer;", "getImplicitHydrogenCount"), "I", "intValue")
}

#' @name is.aromatic
#' @title Tests whether an atom is aromatic.
#' 
#' This assumes that the molecule containing the atom has been 
#' appropriately configured.
#' 
#' @param atom The atom to test
#' @return `TRUE` is the atom is aromatic, `FALSE` otherwise
#' @seealso \code{\link{is.aliphatic}}, \code{\link{is.in.ring}}, \code{\link{do.aromaticity}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
is.aromatic <- function(atom) {
  atom <- .valid.atom(atom)
  flag.idx <- .jfield("org/openscience/cdk/CDKConstants", "I", "ISAROMATIC")
  .jcall(atom, "Z", "getFlag", as.integer(flag.idx))
}

#' @name is.aliphatic
#' @title Tests whether an atom is aliphatic.
#' 
#' This assumes that the molecule containing the atom has been 
#' appropriately configured.
#' 
#' @param atom The atom to test
#' @return `TRUE` is the atom is aliphatic, `FALSE` otherwise
#' @seealso \code{\link{is.in.ring}}, \code{\link{is.aromatic}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
is.aliphatic <- function(atom) {
  atom <- .valid.atom(atom)
  flag.idx <- .jfield("org/openscience/cdk/CDKConstants", "I", "ISALIPHATIC")
  .jcall(atom, "Z", "getFlag", as.integer(flag.idx))
}

#' @name is.in.ring
#' @title Tests whether an atom is in a ring.
#' 
#' This assumes that the molecule containing the atom has been 
#' appropriately configured.
#' 
#' @param atom The atom to test
#' @return `TRUE` is the atom is in a ring, `FALSE` otherwise
#' @seealso \code{\link{is.aliphatic}}, \code{\link{is.aromatic}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
is.in.ring <- function(atom) {
  atom <- .valid.atom(atom)
  flag.idx <- .jfield("org/openscience/cdk/CDKConstants", "I", "ISINRING")
  .jcall(atom, "Z", "getFlag", as.integer(flag.idx))
}

#' @name get.connected.atoms
#' @title Get atoms connected to the specified atom
#' 
#' Returns a `list`` of atoms that are connected to the specified atom.
#' @param atom The atom object
#' @param mol The `IAtomContainer` object containing the atom
#' @return A `list` containing `IAtom` objects, representing the atoms directly
#' connected to the specified atom
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.connected.atoms <- function(atom, mol) {
  if (is.null(attr(mol, 'jclass')))
    stop("object must be of class IAtomContainer")
  
  if (attr(mol, 'jclass') != "org/openscience/cdk/interfaces/IAtomContainer")
    stop("object must be of class IAtomContainer")
  
  atom <- .valid.atom(atom)
  ret <- .jcall(mol, "Ljava/util/List;", "getConnectedAtomsList", atom)
  ret <- lapply(.javalist.to.rlist(ret), .jcast, new.class='org/openscience/cdk/interfaces/IAtom')
  return(ret)
}

#' @name get.atom.index
#' @title Get the index of an atom in a molecule.
#' 
#' Acces the index of an atom in the context of an IAtomContainer. 
#' Indexing starts from 0. If the index is not known, -1 is returned.
#' 
#' @param atom The atom object
#' @param mol The `IAtomContainer` object containing the atom
#' @return An integer representing the atom index. 
#' @seealso \code{\link{get.connected.atom}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.atom.index <- function(atom, mol) {
  if (is.null(attr(mol, 'jclass')))
    stop("object must be of class IAtomContainer")
  
  if (attr(mol, 'jclass') != "org/openscience/cdk/interfaces/IAtomContainer")
    stop("object must be of class IAtomContainer")
  
  atom <- .valid.atom(atom)
  .jcall(mol, "I", "getAtomNumber", atom)
}

.packageName <- "rcdk"

#' Write molecules to disk.
#' 
#' This function writes one or more molecules to an SD file on disk,
#' which can be of the single- or multi-molecule variety. In
#' addition, if the molecule has keyed properties, they can also be
#' written out as SD tags.
#' 
#' @details In case individual SD files are desired the
#' \code{together} argument can be set ot \code{FALSE}. In this case, the
#' value of \code{filename} is used as a prefix, to which a numeric
#' identifier and the suffix of ".sdf" is appended. 
#' 
#' @param mols A `list` of `jobjRef` objects representing  `IAtomContainer` objects
#' @param filename The name of the SD file to write. Note that if
#' `together` is `FALSE` then this argument is taken as a prefix for
#' the name of the individual files
#' @param together If `TRUE` then all the molecules are written to a
#' single SD file. If `FALSE` each molecule is written to an
#' individual file
#' @param write.props If `TRUE`, keyed properties are included in the SD file output
#' @seealso \code{\link{load.molecules}}, \code{\link{parse.smiles}}, \code{\link{iload.molecules}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
write.molecules <- function(mols, filename, together=TRUE, write.props=FALSE) {
  if (together) {
    value <-.jcall('org/guha/rcdk/util/Misc', 'V', 'writeMoleculesInOneFile',
                   .jarray(mols,
                           contents.class = "org/openscience/cdk/interfaces/IAtomContainer"),
                   as.character(filename), as.integer(ifelse(write.props,1,0)))
  } else {
    value <- .jcall('org/guha/rcdk/util/Misc', 'V', 'writeMolecules',
                    .jarray(mols,
                            contents.class = "org/openscience/cdk/interfaces/IAtomContainer"),
                    as.character(filename), as.integer(ifelse(write.props,1,0)))
  }
}

#' Load molecular structures from disk or URL
#' 
#' The CDK can read a variety of molecular structure formats. This function
#' encapsulates the calls to the CDK API to load a structure given its filename
#' or a URL to a structure file.
#' 
#' @details 
#' Note that this method will load all molecules into memory. For files containing
#' tens of thousands of molecules this may lead to out of memory errors. In such 
#' situations consider using the iterating file readers.
#' 
#' Note that if molecules are read in from formats that do not have rules for
#' handling implicit hydrogens (such as MDL MOL), the molecule will not have
#' implicit or explicit hydrogens. To add explicit hydrogens, make sure that the molecule
#' has been typed (this is `TRUE` by default for this function) and then call 
#' \code{\link{convert.implicit.to.explicit}}. On the other hand for a format 
#' such as SMILES, implicit or explicit hydrogens will be present.

#' @param molfiles A `character` vector of filenames. Note that the full
#' path to the files should be provided. URL's can also be used as
#' paths. In such a case, the URL should start with "http://"
#' @param aromaticity If `TRUE` then aromaticity detection is
#' performed on all loaded molecules. If this fails for a given
#' molecule, then the molecule is set to `NA` in the return list
#' @param typing If `TRUE` then atom typing is
#' performed on all loaded molecules. The assigned types will be CDK
#' internal types. If this fails for a given molecule, then the molecule 
#' is set to `NA` in the return list
#' @param isotopes If `TRUE` then atoms are configured with isotopic masses
#' @param verbose If `TRUE`, output (such as file download progress) will
#' be bountiful
#' @return A `list` of CDK `IAtomContainer` objects, represented as `jobjRef` objects 
#' in R, which can be used in other `rcdk` functions
#' @seealso \code{\link{write.molecules}}, \code{\link{parse.smiles}}, \code{\link{iload.molecules}}
#' @importFrom utils download.file
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @examples 
#' \dontrun{
#' sdffile <- system.file("molfiles/dhfr00008.sdf", package="rcdk")
#' mols <- load.molecules(c('mol1.sdf', 'mol2.smi', sdfile))

#' }
load.molecules <- function(molfiles=NA, aromaticity = TRUE, 
                           typing = TRUE, isotopes = TRUE, 
                           verbose=FALSE) {
  if (any(is.na(molfiles))) {
    stop("Must supply a vector of file names")
  }
  if (length(molfiles) == 0) {
    stop("Must supply a vector of file names")
  }

  for (f in molfiles) {
    if (!file.exists(f) && length(grep('http://', f)) == 0 && length(grep('https://', f)) == 0)
      stop(paste(f, ": Does not exist", sep=''))
  }

  urls <- grep('http|https', molfiles)
  if (length(urls) > 0) { ## download the files and replace the URL's with the temp names
    for (idx in urls) {
      url <- molfiles[idx]
      tmpdest <- tempfile(pattern='xxx')
      status <- try(download.file(url, destfile=tmpdest,
                                  method='curl',
                                  mode='wb', quiet=!verbose),
                    silent=verbose)
      if (class(status) == 'try-error') {
        molfiles[idx] <- NA
        cat("Can't get ", url, '\n')
      } else {
        molfiles[idx] <- tmpdest
      }
    }
  }
  molfiles <- molfiles[ !is.na(molfiles) ]
  farr <- .jarray(molfiles, contents.class = 'S')
  molecules <- .jcall('org/guha/rcdk/util/Misc', '[Lorg/openscience/cdk/interfaces/IAtomContainer;',
                      'loadMolecules', farr, aromaticity, typing, isotopes,
                      check=FALSE)
  exception <- .jgetEx(clear = TRUE)
  if (!is.null(exception)) {
    stop(exception$toString())
  }

  if (is.jnull(molecules)) {
    return(NA)
  }
  if (length(molecules) == 0) {
    return(molecules)
  } else {
    nulls <- which( unlist(lapply(molecules, is.jnull)) )
    if (length(nulls) > 0) molecules[nulls] <- NA
    return(molecules)
  }
}

#' @importFrom itertools hasNext
hasNext <- function(obj, ...) { UseMethod("hasNext") } 
#' @export
hasNext.iload.molecules <- function(obj, ...) obj$hasNext()

#' Load molecules using an iterator.
#' 
#' The CDK can read a variety of molecular structure formats. Some file
#' formats support multiple molecules in a single file. If read using
#' \code{\link{load.molecules}}, all are read into memory. For very large
#' structure files, this can lead to out of memory errors. Instead it is 
#' recommended to use the iterating version of the loader so that only a
#' single molecule is read at a time.
#' 
#' Note that the iterating loader only supports SDF and SMILES file formats.
#' 
#' @param molfile A string containing the filename to load. Must be a local file
#' @param type Indicates whether the input file is SMILES or SDF. Valid values are
#' `"smi"` or `"sdf"`
#' @param skip If `TRUE`, then the reader will continue reading even when 
#' faced with an invalid molecule. If `FALSE`, the reader will stop at 
#' the fist invalid molecule
#' @param aromaticity If `TRUE` then aromaticity detection is
#' performed on all loaded molecules. If this fails for a given
#' molecule, then the molecule is set to `NA` in the return list
#' @param typing If `TRUE` then atom typing is
#' performed on all loaded molecules. The assigned types will be CDK
#' internal types. If this fails for a given molecule, then the molecule 
#' is set to `NA` in the return list
#' @param isotopes If `TRUE` then atoms are configured with isotopic masses
#' @seealso \code{\link{write.molecules}}, \code{\link{load.molecules}}, \code{\link{parse.smiles}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @examples 
#' \dontrun{
#' moliter <- iload.molecules("big.sdf", type="sdf")
#' while(hasNext(moliter)) {
#' mol <- nextElem(moliter)
#'   print(get.property(mol, "cdk:Title"))
#' }
#' }
iload.molecules<- function(molfile, type = 'smi', 
                           aromaticity = TRUE, typing = TRUE, isotopes = TRUE, 
                           skip=TRUE) {
  
  if (!file.exists(molfile) && length(grep('http://', molfile)) == 0)
    stop(paste(molfile, ": Does not exist", sep=''))
  
  fr <- .jnew("java/io/FileReader", as.character(molfile))
  dcob <- get.chem.object.builder()
  if (type == 'smi') {
    sreader <- .jnew("org/openscience/cdk/io/iterator/IteratingSMILESReader",.jcast(fr, "java/io/Reader"), dcob)
  } else if (type == 'sdf') {
    sreader <- .jnew("org/openscience/cdk/io/iterator/IteratingSDFReader",.jcast(fr, "java/io/Reader"), dcob)
    .jcall(sreader, "V", "setSkip", skip)
  }
  hasNext <- NA
  mol <- NA
  molr <- NA
  
  hasNx <- function() {
    hasNext <<- .jcall(sreader, "Z", "hasNext")
    if (!hasNext) {
      .jcall(sreader, "V", "close")      
      mol <<- NA
    }
    return(hasNext)
  }
  
  nextEl <- function() {
    mol <<- .jcall(sreader, "Ljava/lang/Object;", "next")
    mol <<- .jcast(mol, "org/openscience/cdk/interfaces/IAtomContainer")
    if (aromaticity) do.aromaticity(mol)
    if (typing) do.typing(mol)
    if (isotopes) do.isotopes(mol)
    
    hasNext <<- NA    
    return(mol)
  }
  
  obj <- list(nextElem = nextEl, hasNext = hasNx)
  class(obj) <- c("iload.molecules", "abstractiter", "iter")
  obj
}

#' Identify which atoms are stereocenters.
#'
#' This method identifies stereocenters based on connectivity.
#'
#' @param mol A \code{jObjRef} representing an \code{IAtomContainer}
#' @return A logical vector of length equal in length to the number of atoms. The i'th element is \code{TRUE} if the i'th element is identified as a stereocenter
#' @seealso \code{\link{get.element.types}}, \code{\link{get.stereo.types}}
#' @author Rajarshi Guha \email{rajarshi.guha@gmail.com}
#' @export
get.stereocenters <- function(mol) {
    if (attr(mol, "jclass") != "org/openscience/cdk/interfaces/IAtomContainer")
        stop("Must supply an IAtomContainer object")
    
    centers <- .jcall("org/openscience/cdk/stereo/Stereocenters",
                      "Lorg/openscience/cdk/stereo/Stereocenters;",
                      "of",
                      mol)
    sapply(0:(get.atom.count(mol)-1), function(i) {
        .jcall(centers, "Z", "isStereocenter", as.integer(i))
    })
}

#' Obtain the type of stereo element support for atom.
#'
#' Supported elements types are
#' \describe{
#' \item{Bicoordinate}{an central atom involved in a cumulated system (not yet supported)}
#' \item{Tricoordinate}{an atom at one end of a geometric (double-bond) stereo bond or cumulated system}
#' \item{Tetracoordinate}{a tetrahedral atom (could also be square planar in future)}
#' \item{None}{the atom is not a (supported) stereo element type}
#' }
#'
#' @param mol A \code{jObjRef} representing an \code{IAtomContainer}
#' @return A factor of length equal in length to the number of atoms, indicating the element type
#' @seealso \code{\link{get.stereocenters}}, \code{\link{get.stereo.types}}
#' @author Rajarshi Guha \email{rajarshi.guha@gmail.com}
#' @export
get.element.types <- function(mol) {
    if (attr(mol, "jclass") != "org/openscience/cdk/interfaces/IAtomContainer")
        stop("Must supply an IAtomContainer object")
    
    centers <- .jcall("org/openscience/cdk/stereo/Stereocenters",
                      "Lorg/openscience/cdk/stereo/Stereocenters;",
                      "of",
                      mol)
    stypes <- sapply(0:(get.atom.count(mol)-1), function(i) {
        .jcall(centers,
               "Lorg/openscience/cdk/stereo/Stereocenters$Type;",
               "elementType", as.integer(i))
    })
    stypes <- sapply(stypes, function(x) x$toString())
    factor(stypes, levels=c("Bicoordinate",
                            "None",
                            "Tetracoordinate",
                            "Tricoordinate"))
}

#' Obtain the stereocenter type for atom.
#'
#' Supported stereo center types are
#' \describe{
#' \item{True}{the atom has constitutionally different neighbors}
#' \item{Para}{the atom resembles a stereo centre but has constitutionally equivalent neighbors (e.g. inositol, decalin). The stereocenter depends on the configuration of one or more stereocenters.}
#' \item{Potential}{the atom can supported stereo chemistry but has not be shown ot be a true or para center}
#' \item{Non}{the atom is not a stereocenter (e.g. methane)}
#' }
#'
#' @param mol A \code{jObjRef} representing an \code{IAtomContainer}
#' @return A factor of length equal in length to the number of atoms indicating the stereocenter type.
#' @seealso \code{\link{get.stereocenters}}, \code{\link{get.element.types}}
#' @author Rajarshi Guha \email{rajarshi.guha@gmail.com}
#' @export
get.stereo.types <- function(mol) {
    if (attr(mol, "jclass") != "org/openscience/cdk/interfaces/IAtomContainer")
        stop("Must supply an IAtomContainer object")
    
    centers <- .jcall("org/openscience/cdk/stereo/Stereocenters",
                      "Lorg/openscience/cdk/stereo/Stereocenters;",
                      "of",
                      mol)
    stypes <- sapply(0:(get.atom.count(mol)-1), function(i) {
        .jcall(centers,
               "Lorg/openscience/cdk/stereo/Stereocenters$Stereocenter;",
               "stereocenterType", as.integer(i))
    })
    stypes <- sapply(stypes, function(x) x$toString())
    factor(stypes, levels=c("True","Para","Potential","Non"))
}

##get.bond.order <- function(bond) {
##  stop("Doesn't work at this point")
##  if (is.null(attr(bond, 'jclass')) ||
##      attr(bond, "jclass") != "org/openscience/cdk/interfaces/IBond") {
##    stop("Must supply an IBond object")
##  }
##
##  bo <- .jcall(bond, "Lorg/openscience/cdk/interfaces/IBond.Order;", "getBondOrder")
##  bo
##}
##

#' Get the atom connected to an atom in a bond.
#' 
#' This function returns the atom that is connected to a
#' specified in a specified bond. Note that this function assumes
#' 2-atom bonds, mainly because the CDK does not currently
#' support other types of bonds
#' 
#' @param bond A \code{jObjRef} representing an `IBond` object
#' @param atom A \code{jObjRef} representing an `IAtom` object
#' @return A \code{jObjRef} representing an `IAtom`` object
#' @seealso \code{\link{get.atoms}}
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.connected.atom <- function(bond, atom) {
  if (is.null(attr(bond,"jclass")) || is.null(attr(atom,"jclass")))
    stop("Must supply an IBond object or Bond")
  
  if (!.check.class(bond, "org/openscience/cdk/interfaces/IBond") &&
      !.check.class(bond, "org/openscience/cdk/Bond"))
    stop("Must supply an IBond or Bond object")

  if (.check.class(bond, "org/openscience/cdk/Bond"))
    bond <- .jcast(bond, "org/openscience/cdk/interfaces/IBond")

  if (!.check.class(atom, "org/openscience/cdk/interfaces/IAtom") &&
      !.check.class(atom, "org/openscience/cdk/Atom"))
    stop("Must supply an IAtom or Atom object")
  
  if (.check.class(atom, "org/openscience/cdk/Atom"))
    atom <- .jcast(atom, "org/openscience/cdk/interfaces/IAtom")
  
  .jcall(bond, "Lorg/openscience/cdk/interfaces/IAtom;", "getConnectedAtom", atom);
}

#' @keywords internal
.get.desc.values <- function(dval, nexpected) {
  if (!inherits(dval, "jobjRef")) {
    if (is.null(dval) || is.na(dval)) return(NA)
  }

  if (!is.null(.jcall(dval, "Ljava/lang/Exception;", "getException"))) {
    return(rep(NA, nexpected))
  }
  
  nval <- numeric()
  if (!inherits(dval,'jobjRef') && is.na(dval)) {
    return(NA)
  }
  
  result <- .jcall(dval, "Lorg/openscience/cdk/qsar/result/IDescriptorResult;", "getValue")
  methods <- .jmethods(result)

  if ("public double org.openscience.cdk.qsar.result.DoubleArrayResult.get(int)" %in% methods) {
    result <- .jcast(result, "org/openscience/cdk/qsar/result/DoubleArrayResult")
    len <- .jcall(result, "I", "length")
    for (i in 1:len) nval[i] <- .jcall(result, "D", "get", as.integer(i-1))
  } else if ("public int org.openscience.cdk.qsar.result.IntegerArrayResult.get(int)" %in% methods) {
    result <- .jcast(result, "org/openscience/cdk/qsar/result/IntegerArrayResult")    
    len <- .jcall(result, "I", "length")
    for (i in 1:len) nval[i] <- .jcall(result, "I", "get", as.integer(i-1))    
  }  else if ("public int org.openscience.cdk.qsar.result.IntegerResult.intValue()" %in% methods) {
    result <- .jcast(result, "org/openscience/cdk/qsar/result/IntegerResult")    
    nval <- .jcall(result, "I", "intValue")
  } else if ("public double org.openscience.cdk.qsar.result.DoubleResult.doubleValue()" %in% methods) {
    result <- .jcast(result, "org/openscience/cdk/qsar/result/DoubleResult")    
    nval <- .jcall(result, "D", "doubleValue")    
  }

  return(nval)
}


#' @keywords internal
.get.desc.engine <- function(type = 'molecular') {
  if (!(type %in% c('molecular', 'atomic', 'bond'))) {
    stop('type must bond, molecular or atomic')
  }
  if (type == 'molecular') {
    interface <- J("org.openscience.cdk.qsar.IMolecularDescriptor")
  } else if (type == 'atomic') {
    interface <- J("org.openscience.cdk.qsar.IAtomicDescriptor")    
  } else if (type == 'bond') {
    interface <- J("org.openscience.cdk.qsar.IBondDescriptor")        
  }
  dklass <- interface@jobj
  dcob <- get.chem.object.builder()
  dengine <- .jnew('org/openscience/cdk/qsar/DescriptorEngine', dklass, dcob)
  attr(dengine, 'descType') <- type
  pkg <- c('org.openscience.cdk.qsar.descriptors.atomic',
           'org.openscience.cdk.qsar.descriptors.bond',
           'org.openscience.cdk.qsar.descriptors.molecular')[ type ]
  attr(dengine, 'descPkg') <- pkg
  dengine
}

.get.desc.all.classnames <- function(type = 'molecular') {
  dengine <- .get.desc.engine(type)
  type <- attr(dengine, "descType")
  pkg <- attr(dengine, "descPkg")
  cn <- .jcall(dengine, 'Ljava/util/List;', 'getDescriptorClassNames')
  size <- .jcall(cn, "I", "size")
  cnames <- list()
  for (i in 1:size)
    cnames[[i]] <- .jsimplify(.jcast(.jcall(cn, "Ljava/lang/Object;", "get", as.integer(i-1)), "java/lang/String"))
                                        #cnames <- gsub(paste(pkg, '.', sep='',collapse=''), '',  unlist(cnames))
  unique(unlist(cnames)  )
}

#' Get descriptor class names
#' 
#' @param type A string indicating which class of descriptors to return. Specifying
#' `"all"` will return class names for all molecular descriptors. Options include
#' * topological
#' * geometrical
#' * hybrid
#' * constitutional
#' * protein
#' * electronic
#' @seealso \link{get.atomic.desc.names}
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @export
get.desc.names <- function(type = "all") {
  if (type == 'all') return(.get.desc.all.classnames())
  if (!(type %in% c('topological', 'geometrical', 'hybrid',
                    'constitutional', 'protein', 'electronic'))) {
    stop("Invalid descriptor category specified")
  }
  ret <- .jcall("org/guha/rcdk/descriptors/DescriptorUtilities", "[Ljava/lang/String;",
                "getDescriptorNamesByCategory", type)
  if ("org.openscience.cdk.qsar.descriptors.molecular.IPMolecularLearningDescriptor" %in% ret) {
    pos <- which(ret == "org.openscience.cdk.qsar.descriptors.molecular.IPMolecularLearningDescriptor")
    return(ret[-pos])
  } else {
    return(ret)
  }
}

#' List available descriptor categories
#' 
#' @return A character vector listing available descriptor categories. This can be 
#' used in \link{get.desc.names}
#' @seealso \link{get.desc.names}
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @export
get.desc.categories <- function() {
  cats <- .jcall("org/guha/rcdk/descriptors/DescriptorUtilities", "[Ljava/lang/String;",
                 "getDescriptorCategories");
  gsub("Descriptor", "", cats)
}

#' Compute descriptor values for a set of molecules
#' 
#' @param molecules A `list` of molecule objects
#' @param which.desc A character vector listing descriptor class names
#' @param verbose If `TRUE`, verbose output
#' @return A `data.frame` with molecules in the rows and descriptors in the columns. If
#' a descriptor value cannot be computed for a molecule, `NA` is returned.
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @export
eval.desc <- function(molecules, which.desc, verbose = FALSE) {
  if (class(molecules) != 'list') {
    jclassAttr <- attr(molecules, "jclass")
    if (jclassAttr != "org/openscience/cdk/interfaces/IAtomContainer") {
      stop("Must provide a list of molecule objects or a single molecule object")
    }
    molecules <- list(molecules)
  } else {
    jclassAttr <- lapply(molecules, attr, "jclass")
    if (any(jclassAttr != "org/openscience/cdk/interfaces/IAtomContainer")) {
      stop("molecule must be an IAtomContainer")
    }
  }

  dcob <- get.chem.object.builder()
  
  if (length(which.desc) == 1) {
    desc <- .jnew(which.desc)
    .jcall(desc, "V", "initialise", dcob)
    
    dnames <- .jcall(desc, "[Ljava/lang/String;", "getDescriptorNames")
    dnames <- gsub('-', '.', dnames)
    
    descvals <- lapply(molecules, function(a,b) {
      val <- tryCatch({.jcall(b, "Lorg/openscience/cdk/qsar/DescriptorValue;", "calculate", a)},
                      warning = function(e) return(NA),
                      error = function(e) return(NA))
    }, b=desc)


    vals <- lapply(descvals, .get.desc.values, nexpected = length(dnames))
    vals <- data.frame(do.call('rbind', vals))
    names(vals) <- dnames 
    return(vals)
  } else {
    counter <- 1
    dl <- list()
    dnames <- c()
    for (desc in which.desc) {
      if (verbose) { cat("Processing ", gsub('org.openscience.cdk.qsar.descriptors.molecular.', '', desc)
                         , "\n") }
      desc <- .jnew(desc)
      .jcall(desc, "V", "initialise", dcob)
      
      dnames <- .jcall(desc, "[Ljava/lang/String;", "getDescriptorNames")
      dnames <- gsub('-', '.', dnames)

      descvals <- lapply(molecules, function(a, check) {
        val <- tryCatch({.jcall(desc, "Lorg/openscience/cdk/qsar/DescriptorValue;", "calculate", a, check=check)})
      }, check=FALSE)

      vals <- lapply(descvals, .get.desc.values, nexpected = length(dnames))
      vals <- data.frame(do.call('rbind', vals))

      
      if (length(vals) == 1 && any(is.na(vals))) {

        vals <- as.data.frame(matrix(NA, nrow=1, ncol=length(dnames)))
      }
      
      names(vals) <- dnames
      ## idx <- which(is.na(names(vals)))
      ## if (length(idx) > 0) vals <- vals[,-idx]
      
      dl[[counter]] <- vals
      counter <- counter + 1
    }
    do.call('cbind', dl)
  }
}

#' Get class names for atomic descriptors
#' 
#' @param type A string indicating which class of descriptors to return. Specifying
#' `"all"` will return class names for all molecular descriptors. Options include
#' * topological
#' * geometrical
#' * hybrid
#' * constitutional
#' * protein
#' * electronic
#' @return A character vector containing class names for atomic descriptors
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @export
get.atomic.desc.names <- function(type = "all") {
  if (type == 'all') return(.get.desc.all.classnames('atomic'))
  return(.jcall("org/guha/rcdk/descriptors/DescriptorUtilities", "[Ljava/lang/String;",
                "getDescriptorNamesByCategory", type))
}

#' Compute descriptors for each atom in a molecule
#' 
#' @param molecule A molecule object
#' @param which.desc A character vector of atomic descriptor class names
#' @param verbose Optional. Default \code{FALSE}. Toggle verbosity.
#' @return A `data.frame` with atoms in the rows and descriptors in the columns
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @seealso \link{get.atomic.desc.names}
eval.atomic.desc <- function(molecule, which.desc, verbose = FALSE) {
  if (attr(molecule, "jclass") != "org/openscience/cdk/interfaces/IAtomContainer") {
    stop("Must supply an IAtomContainer object")
  }

  if (length(which.desc) > 1) {
    counter <- 1
    dl <- list()
    for (desc in which.desc) {
      if (verbose) { cat("Processing ", gsub('org.openscience.cdk.qsar.descriptors.atomic.', '', desc)
                         , "\n") }
      desc <- .jnew(desc)
      atoms = get.atoms(molecule)
      descvals <- lapply(atoms, function(a) {
        dval <- .jcall(desc, "Lorg/openscience/cdk/qsar/DescriptorValue;", "calculate", a, molecule, check=FALSE)
        if (!is.null(e<-.jgetEx())) {
          print("Java exception was raised")
          .jclear()
          dval <- NA
        }
        return(dval)
      })
      vals <- lapply(descvals, .get.desc.values)
      vals <- data.frame(do.call('rbind', vals))

      if (inherits(descvals[[1]], "jobjRef")) {
        names(vals) <- .jcall(descvals[[1]], "[Ljava/lang/String;", "getNames")
      } else {
        names(vals) <- gsub('org.openscience.cdk.qsar.descriptors.atomic.', '', desc)
      }
      dl[[counter]] <- vals
      counter <- counter + 1
    }
    do.call('cbind', dl)
  }
}

#' Compute TPSA for a molecule
#' @param molecule A molecule object
#' @return A double value representing the TPSA value
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @export
get.tpsa <- function(molecule) {
  if (attr(molecule, "jclass") != "org/openscience/cdk/interfaces/IAtomContainer") {
    stop("Must supply an IAtomContainer object")
  }

  desc <- .jnew("org.openscience.cdk.qsar.descriptors.molecular.TPSADescriptor")
  descval <- .jcall(desc, "Lorg/openscience/cdk/qsar/DescriptorValue;", "calculate", molecule)
  value <- .get.desc.values(descval, 1)
  return(value)
}

#' Compute ALogP for a molecule
#' @param molecule A molecule object
#' @return A double value representing the ALogP value
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.alogp <- function(molecule) {
  if (attr(molecule, "jclass") != "org/openscience/cdk/interfaces/IAtomContainer") {
    stop("Must supply an IAtomContainer object")
  }

  desc <- .jnew("org.openscience.cdk.qsar.descriptors.molecular.ALOGPDescriptor")
  descval <- .jcall(desc, "Lorg/openscience/cdk/qsar/DescriptorValue;", "calculate", molecule)
  value <- .get.desc.values(descval, 3)
  return(value[1])
}

#' Compute XLogP for a molecule
#' @param molecule A molecule object
#' @return A double value representing the XLogP value
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
#' @export
get.xlogp <- function(molecule) {
  if (attr(molecule, "jclass") != "org/openscience/cdk/interfaces/IAtomContainer") {
    stop("Must supply an IAtomContainer object")
  }

  desc <- .jnew("org.openscience.cdk.qsar.descriptors.molecular.XLogPDescriptor")
  descval <- .jcall(desc, "Lorg/openscience/cdk/qsar/DescriptorValue;", "calculate", molecule)
  value <- .get.desc.values(descval, 3)
  return(value)
}

#' Compute volume of a molecule
#' 
#' This method does not require 3D coordinates. As a result its an 
#' approximation
#' @param molecule A molecule object
#' @return A double value representing the volume
#' @export
#' @author Rajarshi Guha (\email{rajarshi.guha@@gmail.com})
get.volume <- function(molecule) {
  if (attr(molecule, "jclass") != "org/openscience/cdk/interfaces/IAtomContainer") {
    stop("Must supply an IAtomContainer object")
  }
  return(J("org.openscience.cdk.geometry.volume.VABCVolume", "calculate", molecule))
}

---
title: "Using the CDK from R"
author: "Rajarshi Guha"
date: "`r Sys.Date()`"
description: >
  Learn how to get started with the basics of pkgdown.
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using the CDK from R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


---
title: "Using the CDK from R"
author: "Rajarshi Guha"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Using the CDK from R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Using the CDK from R

## Introduction

Given that much of cheminformatics involves mathematical and statistical
modeling of chemical information, R is a natural platform for such work.
There are many cheminformatics applications that will generate useful information
such as descriptors, fingerprints and so on. While one can always
run these applications to generate data that is then imported into R, it can
be convenient to be able to manipulate chemical structures and generate
chemical information with the R environment.

The [CDK](https://cdk.github.io) is a Java library for cheminformatics that supports a wide variety
of cheminformatics functionality ranging from reading molecular file
formats, performing ring perception and aromaticity detection to fingerprint
generation and molecular descriptors. The CDK website provides links to useful documentation as well as complete [Javadocs](https://cdk.github.io/cdk/2.3/docs/api/index.html?overview-summary.html)

## Getting started

The goal of the `rcdk` package is to allow an R user to access the cheminformatics
functionality of the [CDK](https://cdk.github.io) from within R. While one can use the
[rJava](https://CRAN.R-project.org/package=rJava) package to make direct calls to specific methods in the [CDK](https://cdk.github.io), from R,
such usage does not usually follow common R idioms. Thus `rcdk` aims to 
 allow users to use the [CDK](https://cdk.github.io) classes and methods in an R-like fashion.

The library is loaded as follows
```{r}
library(rcdk)
```
We can also check the version of the [CDK](https://cdk.github.io) that is being used in the package
```{r}
cdk.version()
```
The package also provides an example data set, called `bpdata` which contains
277 molecules, in SMILES format and their associated boiling points (BP)
in Kelvin. The `data.frame` has two columns, viz., the SMILES and the BP.
Molecules names are used as row names:
```{r echo=FALSE}
data(bpdata)
str(bpdata)
```
## Input and Output

Chemical structures come in a variety of formats and the CDK supports
many of them. Many such formats are disk based and these files can be
parsed and loaded by specifying their full paths

```{r, eval=FALSE}
mols <- load.molecules( c('data1.sdf', '/some/path/data2.sdf') )
```

Note that the above function will load any file format that is supported by
the CDK, so there’s no need to specify formats. In addition one can specify a
URL (which should start with `http://`) to specify remote files as well. The
result of this function is a list of molecule objects. The molecule objects
are of class `jobjRef` (provided by the [rJava](https://CRAN.R-project.org/package=rJava) package). As a result,they
are pretty opaque to the user and are really meant to be processed using
methods from the `rcdk` or [rJava](https://CRAN.R-project.org/package=rJava) packages.

However, since it loads all the molecules from the specified file into a list,
large files can lead to out of memory errors. In such a situation it is preferable
to iterate over the file, one structure at a time. Currently this behavior
is supported for SDF and SMILES files. An example of such a usage for a large
SD file would be
```{r eval=FALSE}
iter <- iload.molecules('verybig.sdf', type='sdf')
while(hasNext(iter)) {
 mol <- nextElem(iter)
 print(get.property(mol, "cdk:Title"))
}
```

### Parsing SMILES

Another common way to obtain molecule objects is by parsing SMILES
strings. The simplest way to do this is
```{r eval=FALSE}
smile <- 'c1ccccc1CC(=O)C(N)CC1CCCCOC1'
mol <- parse.smiles(smile)[[1]]
```
Usage is more efficient when multiple SMILE are supplied, since then a single
SMILES parser object is used to parse all the supplied SMILES.

If you plan on parsing a large number of SMILES, you may run into memory
issues, due to the large size of `IAtomContainer` objects. In such a case, it
can be useful to call the Java and R garbage collectors explicitly at the
appropriate time. In addition it can be useful to explicitly allocate a large
amount of memory for the JVM. For example,
```{r eval=FALSE}
options("java.parameters"=c("-Xmx4000m"))
library(rcdk)
for (smile in smiles) {
    m <- parse.smiles(smile)
    ## perform operations on this molecule
    
    jcall("java/lang/System","V","gc")
    gc()
}
```
Given a list of molecule objects, it is possible to serialize them to a file in
some specified format. Currently, the only output formats are SMILES or
SDF. To write molecules to a disk file in SDF format.
```{r eval=FALSE}
write.molecules(mols, filename='mymols.sdf')
```
By default, if `mols` is a list of multiple molecules, all of them will be written
to a single SDF file. If this is not desired, you can write each on to individual
files (which are prefixed by the value of filename):

```{r eval=FALSE}
 write.molecules(mols, filename='mymols.sdf', together=FALSE)
```
 
### Generating SMILES
 
 Finally, we can generate a SMILES representation of a molecule using
 
```{r}
smiles <- c('CCC', 'c1ccccc1', 'CCCC(C)(C)CC(=O)NC')
mols <- parse.smiles(smiles)
get.smiles(mols[[1]])
unlist(lapply(mols, get.smiles))
```

 
 The [CDK](https://cdk.github.io) supports a number of _flavors_ when generating SMILES. For example, you can generate a SMILES with or without chirality information or generate SMILES in [Kekule form](https://www.chemguide.co.uk/basicorg/bonding/benzene1.html). The `smiles.flavors` generates an object that represents the various flavors desired for SMILES output. See the [SmiFlavor javadocs](https://cdk.github.io/cdk/2.3/docs/api/index.html?org/openscience/cdk/smiles/SmiFlavor.html) for the full list of possible flavors. Example usage is
 
```{r}
smiles <- c('CCC', 'c1ccccc1', 'CCc1ccccc1CC(C)(C)CC(=O)NC')
mols <- parse.smiles(smiles)
get.smiles(mols[[3]], smiles.flavors(c('UseAromaticSymbols')))
get.smiles(mols[[3]], smiles.flavors(c('Generic','CxSmiles')))
```

Using the [CxSmiles](https://docs.chemaxon.com/display/docs/formats_chemaxon-extended-smiles-and-smarts-cxsmiles-and-cxsmarts.md) flavors allows the user to encode a variety of information in the SMILES string, such as 2D or 3D coordinates.

```{r}
m <- parse.smiles('CCC')[[1]]
m <- generate.2d.coordinates(m)
m
get.smiles(m, smiles.flavors(c('CxSmiles')))
get.smiles(m, smiles.flavors(c('CxCoordinates')))
```

## Visualization

The `rcdk` package supports 2D rendering of chemical structures. This can be
used to view the structure of individual molecules or multiple molecules in
a tabular format. It is also possible to view a molecular-data table, where
one of the columns is the 2D image and the remainder can contain data
associated with the molecules.

Due to Java event handling issues on OS X, depictions are handled using an external helper, which means that 
depiction generation can be slower on OS X compared to other platforms. 

Molecule visualization is performed using the `view.molecule.2d` function. This handles individual molecules as well as a list of molecules. In the latter case, the depictions are arranged in a grid (with 4 columns by default).

```{r eval=FALSE}
smiles <- c('CCC', 'CCN', 'CCN(C)(C)',
            'c1ccccc1Cc1ccccc1',
            'C1CCC1CC(CN(C)(C))CC(=O)CC')
mols <- parse.smiles(smiles)
view.molecule.2d(mols[[1]])
view.molecule.2d(mols)
```

The CDK depiction routines allow for extensive customization. These customizations can be accessed by creating a depictor object using `get.depictor`, which allows you to specify the size of the depiction, the depiction style (black and white, color on white, etc.), atom annotations (e.g., atom index), whether functional group abbreviations should be used or not and so on.

```{r eval=FALSE}
depictor <- get.depictor(style='cob', abbr='reagents', width=300, height=300)
view.molecule.2d(mols[[5]], depictor=depictor)
```
Once you have a depictor object, you can set individual properties using the `$` notation. This can be useful if you plan to generate a lot of depictions so that a new depictor is not recreated for each new structure.
```{r eval=FALSE}
depictor <- get.depictor(style='cob', abbr='reagents', width=300, height=300)
view.molecule.2d(mols[[5]], depictor=depictor)
#depictor$setStyle('cow')
#view.molecule.2d(mols[[5]], depictor=depictor)
```

The method also allows you to highlight substructures using [SMARTS](https://en.wikipedia.org/wiki/Smiles_arbitrary_target_specification). This is useful in highlight common substructures in a set of molecules
```{r eval=FALSE}
depictor <- get.depictor(style='cob', abbr='reagents', sma='N(C)(C)')
view.molecule.2d(mols, depictor=depictor)
```

In many cases, it is useful to view a “molecular spreadsheet”, which is a table
of molecular structures along with information (numeric or textual) related to the molecules being viewed. The data is arranged in a spreadsheet like manner, with one of the columns
being molecules and the remainder being textual or numeric information.

This can be achieved using the `view.table` method which takes a list of
molecule objects and a `data.frame` containing the associated data. As expected,
the number of rows in the `data.frame` should equal the length of
the molecule list. Note that currently, there is not explicit binding between the rows of the `data.frame` and the elements of the list containing the molecules. Thus the user should take care that the ordering of the `data.frame` matches that of the list.
```{r eval=FALSE}
smiles <- c('CCC', 'CCN', 'CCN(C)(C)','c1ccccc1Cc1ccccc1')
mols <- parse.smiles(smiles)
dframe <- data.frame(x = runif(4),
  toxicity = factor(c('Toxic', 'Toxic', 'Nontoxic', 'Nontoxic')),
  solubility = c('yes', 'yes', 'no', 'yes'))
view.table(mols, dframe)
```

While the `view.molecule.2d` function is useful to visualize structures, the depictions can't be included in other visualizations such as plots. For such use cases, the `view.image.2d` function produces a raster image that can be included in plots. This function handles one molecule at a time.
```{r}
img <- view.image.2d(parse.smiles("B([C@H](CC(C)C)NC(=O)[C@H](CC1=CC=CC=C1)NC(=O)C2=NC=CN=C2)(O)O")[[1]])
plot(1:10, 1:10, pch=19)
rasterImage(img, 1,6, 5,10)
```

Finally, the `copy.image.to.clipboard` function allows you to copy a depiction to the system clipboard, from where it can be pasted into other applications. This can be more convenient than saving a raster image.

## Manipulating Molecules

In general, given a `jobjRef` for a molecule object one can access all the
class and methods of the [CDK](https://cdk.github.io) library via rJava. However this can be
cumbersome. The `rcdk` package exposes methods and
classes that manipulate molecules.

### Adding Information to Molecules

In many scenarios it’s useful to associate information with molecules. Within
R, you could always create a `data.frame` and store the molecule objects
along with relevant information in it. However, when serializing the molecules,
you want to be able to store the associated information with the structure itself (though keep in mind that only certain chemical file formats support metadata along with the structure).

Using the [CDK](https://cdk.github.io) it’s possible to directly add information to a molecule object
using properties. Note that adding such properties uses a key-value
paradigm, where the key should be of class character. The value can be
of class `integer`, `double`, `character` or `jobjRef`. Obviously, after setting
a property, you can get a property by its key.

```{r}
mol <- parse.smiles('c1ccccc1')[[1]]
set.property(mol, "title", "Molecule 1")
set.property(mol, "hvyAtomCount", 6)
get.property(mol, "title")
```
It is also possible to get all available properties at once in the from of a list.
The property names are used as the list names.
```{r}
get.properties(mol)
```
After adding such properties to the molecule, you can write it out to an SD
file, so that the property values become SD tags.
```{r eval=FALSE}
write.molecules(mol, 'tagged.sdf', write.props=TRUE)
```

### Atoms and Bonds

Probably the most important thing to do is to get the atoms and bonds of
a molecule. The code below gets the atoms and bonds as lists of `jobjRef`
objects, which can be manipulated using rJava or via other methods of this
package.
```{r}
mol <- parse.smiles('c1ccccc1C(Cl)(Br)c1ccccc1')[[1]]
atoms <- get.atoms(mol)
bonds <- get.bonds(mol)
cat('No. of atoms =', length(atoms), '\n')
cat('No. of bonds =', length(bonds), '\n')
```
Given an atom the `rcdk` package does not offer a lot of methods
to operate on it. One must access the [CDK](https://cdk.github.io) directly.
In the future more
manipulators will be added. Right now, you can get the symbol for each
atom

```{r}
unlist(lapply(atoms, get.symbol))
```
It’s also possible to get the 3D (or 2D coordinates) for an atom.
```{r}
coords <- get.point3d(atoms[[1]])
```
Given this, it’s quite easy to get the 3D coordinate matrix for a molecule
```{r}
coords <- do.call('rbind', lapply(atoms, get.point3d))
```
Once you have the coordinate matrix, a quick way to check whether the
molecule is flat is to do
```{r}
if ( any(apply(coords, 2, function(x) length(unique(x))) == 1) ) {
    print("molecule is flat")
}
```
This is quite a simplistic check that just looks at whether the X, Y or
Z coordinates are constant. To be more rigorous one could evaluate the
moments of inertia about the axes.

### Substructure matching

The [CDK](https://cdk.github.io) library supports substructure searches using [SMARTS](https://en.wikipedia.org/wiki/Smiles_arbitrary_target_specification) (or SMILES)
patterns. The implementation allows one to check whether a target molecule
contains a substructure or not as well as to retrieve the atoms and bonds
of the target molecule that match the query substructure. At this point,
the `rcdk` only support the former operation - given a query pattern, does it
occur or not in a list of target molecules. The `matches` method of this package
returns a logical vector where the $i$'th element is `TRUE` if the $i$'th target molecules contains the query substructure. An example of its
usage would be to identify molecules that contain a carbon atom that has
exactly two bonded neighbors.
```{r}
mols <- parse.smiles(c('CC(C)(C)C','c1ccc(Cl)cc1C(=O)O', 'CCC(N)(N)CC'))
query <- '[#6D2]'
matches(query, mols)
```

## Molecular Descriptors

A key requirement for the predictive modeling of molecular properties and activities are [molecular descriptors](https://en.wikipedia.org/wiki/Molecular_descriptor) - numerical characterizations of the molecular structure.  The [CDK](https://cdk.github.io/) implements a variety of molecular descriptors, categorized into topological, constitutional, geometric, electronic and hybrid. It is possible to evaluate all available descriptors at one go, or evaluate individual descriptors.

First, we can take a look at the available descriptor categories.
```{r}
dc <- get.desc.categories()
dc
```
Given the categories we can get the names of the descriptors for a single
category. Of course, you can always provide the category name directly.
```{r}
dn <- get.desc.names(dc[4])
dn
```
Each descriptor name is actually a fully qualified Java class name for the
corresponding descriptor. These names can be supplied to `eval.desc` to
evaluate a single or multiple descriptors for one or more molecules.
```{r eval=FALSE}
aDesc <- eval.desc(mol, dn[4])
allDescs <- eval.desc(mol, dn)
```

The return value of eval.desc is a `data.frame` with the descriptors in the
columns and the molecules in the rows. For the above example we get a single
row. But given a list of molecules, we can easily get a descriptor matrix.

For example, let's build a linear regression model to predict boiling points
for the BP dataset. First we need a set of descriptors and so we evaluate
all available descriptors. Also note that since a descriptor might belong to
more than one category, we should obtain a unique set of descriptor names
```{r}
descNames <- unique(unlist(sapply(get.desc.categories(), get.desc.names)))
```
For the current discussion we focus on a few, manually selected descriptors
that we know will be related to boiling point.
```{r}
data(bpdata)
mols <- parse.smiles(bpdata[,1])
descNames <- c(
 'org.openscience.cdk.qsar.descriptors.molecular.KierHallSmartsDescriptor',
 'org.openscience.cdk.qsar.descriptors.molecular.APolDescriptor',
 'org.openscience.cdk.qsar.descriptors.molecular.HBondDonorCountDescriptor')
descs <- eval.desc(mols, descNames)
class(descs)
dim(descs)
```
When a descriptor value cannot be computed, it's value is set to `NA`. This may happen if a descriptor requires 3D coordinates, but only 2D coordinates are available. In this case, we have manually selected descriptors such that there will be no undefined values.

Given the ubiquity of certain descriptors, some of them are directly available
via their own functions. Specifically, one can calculate [TPSA](https://pubmed.ncbi.nlm.nih.gov/19149561/) (topological
polar surface area), [AlogP](https://pubs.acs.org/doi/abs/10.1021/jp980230o) and [XlogP](http://www.compchemcons.com/xlogp/xlogp.html) without having to go through
`eval.desc`. (Note that AlogP and XlogP assume that hydrogens are explicitly specified in the molecule. This may not be true if the molecules were obtained from SMILES)
```{r}
mol <- parse.smiles('CC(=O)CC(=O)NCN')[[1]]
convert.implicit.to.explicit(mol)
get.tpsa(mol)
get.xlogp(mol)
get.alogp(mol)
```

Now that we have a descriptor matrix, we easily build a linear
regression model. First, remove `NA`’s, correlated and constant columns. The
code is shown below, but since it involves a stochastic element, we will not
run it for this example. If we were to perform feature selection, then this
type of reduction would have to be performed.

```{r}
descs <- descs[, !apply(descs, 2, function(x) any(is.na(x)) )]
descs <- descs[, !apply( descs, 2, function(x) length(unique(x)) == 1 )]
r2 <- which(cor(descs)^2 > .6, arr.ind=TRUE)
r2 <- r2[ r2[,1] > r2[,2] , ]
descs <- descs[, -unique(r2[,2])]
```

Note that the above correlation reduction step is pretty crude and there
are better ways to do it. Given the reduced descriptor matrix, we can
perform feature selection (say using [leaps](https://CRAN.R-project.org/package=leaps), [caret](https://CRAN.R-project.org/package=caret) or a [GA](https://CRAN.R-project.org/package=GenAlgo) to identify a suitable
subset of descriptors. Given that we selected the descriptors by hand, we can skip this section, and directly build the model and generate a plot of predicted versus observed BP. (Note that this is a toy example and is not an example of good QSAR practice!)
```{r}
model <- lm(BP ~ khs.sCH3 + khs.sF + apol + nHBDon, data.frame(bpdata, descs))
summary(model)
plot(bpdata$BP, predict(model, descs),
     xlab="Observed BP", ylab="Predicted BP",
     pch=19, xlim=c(100, 700), ylim=c(100, 700))
abline(0,1, col='red')
```
 


## Fingerprints

Fingerprints are a common representation used for a variety of purposes such as similarity searching and predictive modeling. The [CDK](https://cdk.github.io/) provides a variety of fingerprints ranging from path-based hashed fingerprints to circular (specifically, an implementation fo the [ECFP](https://pubs.acs.org/doi/abs/10.1021/ci100050t) fingerprints) and signature fingerprints (based on the [signature](https://pubs.acs.org/doi/abs/10.1021/ci020345w) molecular descriptor). Some of the fingerprints are represented as binary strings and other by integer vectors. The `rcdk` employs the [fingerprint](https://CRAN.R-project.org/package=fingerprint) package to support operations on the resultant fingerprints.

In this section, we present an example of using fingerprints to generate a hierarchical clustering of a set of molecules from the included boiling point dataset. We first parse the SMILES for the molecules in the dataset and then compute the fingerprints, specifying the `circular` type.
```{r}
data(bpdata)
mols <- parse.smiles(bpdata[,1])
fps <- lapply(mols, get.fingerprint, type='circular')
```

With the fingerprints, we can then compute a pairwise similarity matrix using the [Tanimoto](https://en.wikipedia.org/wiki/Chemical_similarity) metric. Since R's `hclust` method requires a distance matrix, we convert the similarity matrix to a distance matrix
```{r}
fp.sim <- fingerprint::fp.sim.matrix(fps, method='tanimoto')
fp.dist <- 1 - fp.sim
```
Finally, we can perform the clustering. In this case we use the `hclust` method though any of R's clustering methods could be used.
```{r}
cls <- hclust(as.dist(fp.dist))
plot(cls, main='A Clustering of the BP dataset', labels=FALSE)
```

Another common task for fingerprints is similarity searching. That is, 
given a collection of _target_ molecules, find those molecules that are similar
to a _query_ molecule. This is achieved by evaluating a similarity metric
between the query and each of the target molecules. Those target molecules
exceeding a user defined cutoff will be returned. With the help of the [fingerprint](https://CRAN.R-project.org/package=fingerprint) package this is easily accomplished.

For example, we can identify all the molecules in the BP dataset that have
a Tanimoto similarity of 0.3 or more with acetalehyde, and then create a tabular
summary. Note that this could also be accomplished with
molecular descriptors, in which case you’d probably evaluate the Euclidean
distance between descriptor vectors.

```{r}
query.mol <- parse.smiles('CC(=O)')[[1]]
target.mols <- parse.smiles(bpdata[,1])
query.fp <- get.fingerprint(query.mol, type='circular')
target.fps <- lapply(target.mols, get.fingerprint, type='circular')
sims <- data.frame(sim=do.call(rbind, lapply(target.fps,
     fingerprint::distance,
     fp2=query.fp, method='tanimoto')))
subset(sims, sim >= 0.3)
```

---
title: "Handling Molecular Formulae"
author: "Rajarshi Guha"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true	
vignette: >
  %\VignetteIndexEntry{Handling Molecular Formulae}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Handling Molecular Formulae

## Introduction

The molecular formula is the simplest way to characterize a molecular compound.
It specifies the actual number of atoms of each element contained
in the molecule. A molecular formula is represented by the chemical symbol
of each constituent element. If a molecule contains more than one atom for
a particular element, the quantity is shown as subscript after the chemical
symbol. Otherwise, the number of neutrons (atomic mass) that an atom is
composed can differ. This different type of atoms are known as isotopes. The
number of nuclei is represented as superscripted prefix previous to the chemical
element. Generally it is not added when the isotope that characterizes
the element is the most occurrence in nature, e.g., $C_4H_{11}O^2D$.

## Parsing a Molecule To a Molecular Formula

Front a molecule, defined as conjunct of atoms helding together by chemical
bonds, we can simplify it taking only the information about the atoms. The `rcdk`
package provides a parser to translate molecules to molecular formlulas, the
`get.mol2formula` function.

```{r}
library(rcdk)
sp <- get.smiles.parser()
molecule <- parse.smiles('N')[[1]]
convert.implicit.to.explicit(molecule)
formula <- get.mol2formula(molecule,charge=0)
```

Note that the above formula object is a `CDKFormula-class`. This class
contains some attributes that defines a molecular formula. For example,
the mass, the charge, the isotopes, the character representation of the
molecular formula and the [IMolecularFormula](https://cdk.github.io/cdk/2.3/docs/api/org/openscience/cdk/interfaces/IMolecularFormula.html) `jobjRef` object.

The molecular mass, charge and string representation for this formula are given by
```{r}
formula@mass
formula@charge
formula@string
```
The isotopes for this formula. It is formed from  three attributes. `isoto` (the
symbol expression of the isotope), `number` (number of atoms for this isotope)
and `mass` (exact mass of this isotope).
```{r}
formula@isotopes
```
Depending of the circumstances, you may want to change the charge of the
molecular formula.
```{r}
formula <- set.charge.formula(formula, charge=1)
```

## Initializing a Formula from the Symbol Expression

Other way to create a `cdkFormula` is from the symbol expression. Thus,
setting the characters of the elemental formula, the function `get.formula`
parses it to an object of `cdkFormula-class`.
```{r}
formula <- get.formula('NH4', charge = 1)
formula
```

## Generating Molecular Formula

Mass spectrometry is an essential and reliable technique to determine the
molecular mass of compounds. Conversely, one can use the measured mass
to identify the compound via its elemental formula. One of the limitations
of the method is the precision and accuracy of the instrumentation. As
a result, rather than specify exact masses, we specify tolerances or ranges
of possible mass, resulting in multiple candidate formulae for a given mass
window. The `generate.formula` function returns a list of formulae which have
a given mass (within an error window)
```{r warn=FALSE}
mfSet <- generate.formula(18.03383, window=1,
    elements=list(c("C",0,50),c("H",0,50),c("N",0,50)),
    validation=FALSE)
mfSet
```
It is important to know if an elemental formula is valid. The method `isvalid.formula`
provides this function. Two constraints can be applied, the
[nitrogen rule](https://en.wikipedia.org/wiki/Nitrogen_rule) and the  [RDBE rule](https://fiehnlab.ucdavis.edu/projects/seven-golden-rules/ring-double-bonds) (Ring Double Bond Equivalent).
```{r}
formula <- get.formula('NH4', charge = 0)
isvalid.formula(formula,rule=c("nitrogen","RDBE"))
```
We can observe that the ammonium is only valid if it is defined with charge
of +1.
```{r}
formula <- get.formula('NH4', charge = 1)
isvalid.formula(formula,rule=c("nitrogen","RDBE"))
```
The `generate.formula` method can perform these validation tests during formula generation by setting `validation=TRUE`. However, this can significantly slow down the process of genersating formulae, especially with larger mass windows. When the default of `FALSE` is used, nonsensical formulae may be generated, and it's up to the user to filter them out.

In contrast, the `generate.formula.iter` method employs the Round Robin formula generation algorithm, which is significantly faster than the default method. In contrast to `generate/formula`, this method returns an iterator which allows you to process large formula sets without excessive memory usage.
```{r warn=FALSE}
mit <- generate.formula.iter(100, charge=0, window=0.1,
                             elements=list(c("C",0,50), c("H",0,50), c("N",0,50)))
hit <- itertools::ihasNext(mit)
while (itertools::hasNext(hit)) 
    print(iterators::nextElem(hit))
```
Note that by default, this function will not check for validity of the generated formulae.

## Calculating Isotope Pattern


Due to the measurement errors in medium resolution spectrometry, a given
error window can result in a massive number of candidate formulae. The
isotope pattern of ions obtained experimentally can be compared with the
theoretical ones. The best match is reflected as the most probable elemental
formula. `rcdk` provides the function `get.isotope.pattern` which predicts
the theoretical isotope pattern given a formula.

```{r}
formula <- get.formula('CHCl3', charge = 0)
isotopes <- get.isotopes.pattern(formula,minAbund=0.1)
isotopes
```

In this example we generate a formula for a possible compound with a charge
($z \approx 0$) containing the standard elements C, H, and Cl. The isotope
pattern can be visually inspected, as shown below.
```{r}
plot(isotopes, type="h", xlab="m/z", ylab="Intensity")
```

---
title: "Performance Notes"
author: "Zaachary Charlop-Powers"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true	
vignette: >
  %\VignetteIndexEntry{Performance Notes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## rCDK Performance

In September 2022, of this year, [Stepehn Neumann](https://gist.github.com/sneumann)
[created a benchmark](https://gist.github.com/sneumann/959a6d205ea4ac73eaf1393da0ec0673) for moecular weight calculation that he [announced on twitter](https://twitter.com/sneumannoffice/status/1570070283083710465?s=20&t=RqJR3Bbh-DEcbCf2tWUEBQ) showing that rCDK had dismal performance relative to other tools in the R ecosystem. Something seemed a bit off so I looked into the code. 

What I discovered is that the mass spec calculations were mediated by R classes instead of accessing the underlying Java code directly and if you write a function that does
that you get a speedup, and if you avoid reflection by creating static calls then you
get a really fast function.

This is a good example of how to think about using Java from within R where the
use of the helper rJava functions, `J` and `$` allow you to prototype code and benefit
from reflection so you can code sort of like you would in R. Then, if you need
performance, you can tighten down the code a bit by making the calls static. You can
see that progression in the code below which is accompanied by the outputs from those benchmarks.



```sh
will give (2/3) runtime in µs:
   21 OrgMassSpecR 
  163 MetaboCoreUtils
  197 enviPat 
  545 Rdisop 
  645 CHNOSZ 
 4863 ChemmineR 
22510 rcdk
```



https://gist.github.com/sneumann/959a6d205ea4ac73eaf1393da0ec0673
## Benchmark

```{r, eval=FALSE}

# Bioconductor Packages. Use BiocManager::install()
#   Rdisop MetaboCoreUtils ChemmineR ChemmineOB enviPat

library(plyr)
library(CHNOSZ)
library(enviPat)
library(MetaboCoreUtils)
library(rcdk)
library(ChemmineR)
library(OrgMassSpecR)
library(Rdisop)
#library(ChemmineOB)

data(isotopes)

# original
# https://github.com/CDK-R/cdkr/blob/master/rcdk/R/formula.R
# get.formula <- function(mf, charge=0) {
#   
#   manipulator <- get("mfManipulator", envir = .rcdk.GlobalEnv)
#   if(!is.character(mf)) {
#     stop("Must supply a Formula string");
#   }else{
#     dcob <- .cdkFormula.createChemObject()
#     molecularformula <- .cdkFormula.createFormulaObject()
#     molecularFormula <- .jcall(manipulator,
#                                "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
#                                "getMolecularFormula",
#                                mf,
#                                .jcast(molecularformula,.IMolecularFormula),
#                                TRUE);
#   }
#   
#   D <- new(J("java/lang/Integer"), as.integer(charge))
#   .jcall(molecularFormula,"V","setCharge",D);
#   object <- .cdkFormula.createObject(.jcast(molecularFormula,.IMolecularFormula));
#   return(object);
# }


mfManipulator    <- J("org/openscience/cdk/tools/manipulator/MolecularFormulaManipulator")
silentchemobject <- J("org.openscience.cdk.silent.SilentChemObjectBuilder")


#' Rewrite the formual object and directly access Java
#'
get.formula2 <- function(mf) {
  
  formula <- mfManipulator$getMolecularFormula(
    "C2H3", 
    silentchemobject$getInstance())
  
  mfManipulator$getMass(formula)
  
}

#' Add type hints
#'
get.formula3 <- function(mf) {
  builderinstance <- .jcall(
      silentchemobject,
     "Lorg/openscience/cdk/interfaces/IChemObjectBuilder;",
     "getInstance")
                       
  formula  <- .jcall(
      mfManipulator,
     "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
     "getMolecularFormula",
      mf,
      builderinstance);

  mfManipulator$getMass(formula)
  
}


#' Add type hints
#'
get.formula4 <- function(mf) {
  builderinstance <- .jcall(
      silentchemobject,
      "Lorg/openscience/cdk/interfaces/IChemObjectBuilder;",
      "getInstance")

  formula  <- .jcall(
      mfManipulator,
     "Lorg/openscience/cdk/interfaces/IMolecularFormula;",
     "getMolecularFormula",
     mf,
     builderinstance);

  .jcall(
      mfManipulator,
      "D",
      "getMass",
     formula)
}



benchmark <- microbenchmark::microbenchmark(
  MetaboCoreUtils = MetaboCoreUtils::calculateMass("C2H6O"),
  rcdk = rcdk::get.formula("C2H6O", charge = 0)@mass,
  rcdk2 = get.formula2("C2H6O"),
  rcdk3 = get.formula3("C2H6O"),
  rcdk4 = get.formula4("C2H6O"),
  Rdisop = Rdisop::getMolecule("C2H6O")$exactmass,
  ChemmineR = ChemmineR::exactMassOB(ChemmineR::smiles2sdf("CCO")),
  OrgMassSpecR = OrgMassSpecR::MonoisotopicMass(formula = OrgMassSpecR::ListFormula("C2H6O)"), charge = 0),
  
  CHNOSZ = CHNOSZ::mass("C2H6O"),
  enviPat = enviPat::isopattern(isotopes, "C2H6O", charge=FALSE, verbose=FALSE)[[1]][1,1]
  , times=1000L)


masses <- c(
  MetaboCoreUtils=MetaboCoreUtils::calculateMass("C2H6O"),
  rcdk=rcdk::get.formula("C2H6O", charge = 0)@mass,
  Rdisop=Rdisop::getMolecule("C2H6O")$exactmass,
  #ChemmineR=ChemmineR::exactMassOB(ChemmineR::smiles2sdf("CCO")),
  OrgMassSpecR=OrgMassSpecR::MonoisotopicMass(formula = OrgMassSpecR::ListFormula("C2H6O)"), charge = 0),
  CHNOSZ=CHNOSZ::mass("C2H6O"),
  enviPat=enviPat::isopattern(isotopes, "C2H6O", charge=FALSE, verbose=FALSE)[[1]][1,1]
)

options(digits=10)
t(t(sort(masses)))
summary(benchmark)[order(summary(benchmark)[,"median"]) , ]
clipr::write_clip(as.data.frame(summary(benchmark)[order(summary(benchmark)[,"median"]) , ] ))
```

## Results

```shell
              expr       min         lq         mean     median         uq
1  MetaboCoreUtils    69.479   122.8465   154.049427   139.6495   156.2700
10         enviPat    83.250   143.0935   170.429197   160.5360   179.6570
5            rcdk4   175.889   228.8605   324.182735   271.2955   327.7135
8     OrgMassSpecR   249.287   333.3135   392.479869   357.6665   401.5585
6           Rdisop   382.417   459.8790   538.068697   490.1505   557.9975
9           CHNOSZ   355.145   510.2910   588.186951   555.9165   632.2060
4            rcdk3   781.987  1004.7160  1294.507318  1133.3415  1339.4695
3            rcdk2  2078.465  2392.4950  2920.601088  2612.8025  2931.5465
7        ChemmineR  3227.320  3790.0455  4808.783873  4044.1410  4465.1000
2             rcdk 14823.815 16456.7715 19088.569430 17485.0800 19468.7195
```

---
title: "Using the CDK from R"
author: "Rajarshi Guha"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
vignette: >
  %\VignetteIndexEntry{Using the CDK from R}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Using the CDK from R

## Introduction

Given that much of cheminformatics involves mathematical and statistical
modeling of chemical information, R is a natural platform for such work.
There are many cheminformatics applications that will generate useful information
such as descriptors, fingerprints and so on. While one can always
run these applications to generate data that is then imported into R, it can
be convenient to be able to manipulate chemical structures and generate
chemical information with the R environment.

The [CDK](https://cdk.github.io) is a Java library for cheminformatics that supports a wide variety
of cheminformatics functionality ranging from reading molecular file
formats, performing ring perception and armaticity detection to fingerprint
generation and molecular descriptors. The CDK website provides links to useful documentation as well as complete [Javadocs](https://cdk.github.io/cdk/2.3/docs/api/index.html?overview-summary.html)

## Getting started

The goal of the `rcdk` package is to allow an R user to access the cheminformatics
functionality of the [CDK](https://cdk.github.io) from within R. While one can use the
[rJava](https://CRAN.R-project.org/package=rJava) package to make direct calls to specific methods in the [CDK](https://cdk.github.io), from R,
such usage does not usually follow common R idioms. Thus `rcdk` aims to 
 allow users to use the [CDK](https://cdk.github.io) classes and methods in an R-like fashion.

The library is loaded as follows
```{r}
library(rcdk)
```
We can also check the version of the [CDK](https://cdk.github.io) that is being used in the package
```{r}
cdk.version()
```
The package also provides an example data set, called `bpdata` which contains
277 molecules, in SMILES format and their associated boiling points (BP)
in Kelvin. The `data.frame` has two columns, viz., the SMILES and the BP.
Molecules names are used as row names:
```{r echo=FALSE}
data(bpdata)
str(bpdata)
```
## Input and Output

Chemical structures come in a variety of formats and the CDK supports
many of them. Many such formats are disk based and these files can be
parsed and loaded by specifying their full paths

```{r, eval=FALSE}
mols <- load.molecules( c('data1.sdf', '/some/path/data2.sdf') )
```

Note that the above function will load any file format that is supported by
the CDK, so there’s no need to specify formats. In addition one can specify a
URL (which should start with `http://`) to specify remote files as well. The
result of this function is a list of molecule objects. The molecule objects
are of class `jobjRef` (provided by the [rJava](https://CRAN.R-project.org/package=rJava) package). As a result,they
are pretty opaque to the user and are really meant to be processed using
methods from the `rcdk` or [rJava](https://CRAN.R-project.org/package=rJava) packages.

However, since it loads all the molecules from the specified file into a list,
large files can lead to out of memory errors. In such a situtation it is preferable
to iterate over the file, one structure at a time. Currently this behavior
is supported for SDF and SMILES files. An example of such a usage for a large
SD file would be
```{r eval=FALSE}
iter <- iload.molecules('verybig.sdf', type='sdf')
while(hasNext(iter)) {
 mol <- nextElem(iter)
 print(get.property(mol, "cdk:Title"))
}
```

### Parsing SMILES

Another common way to obtain molecule objects is by parsing SMILES
strings. The simplest way to do this is
```{r eval=FALSE}
smile <- 'c1ccccc1CC(=O)C(N)CC1CCCCOC1'
mol <- parse.smiles(smile)[[1]]
```
Usage is more efficient when multiple SMILE are supplied, since then a single
SMILES parser object is used to parse all the supplied SMILES.

If you plan on parsing a large number of SMILES, you may run into memory
issues, due to the large size of `IAtomContainer` objects. In such a case, it
can be useful to call the Java and R garbage collectors explicitly at the
appropriate time. In addition it can be useful to explicitly allocate a large
amount of memory for the JVM. For example,
```{r eval=FALSE}
options("java.parameters"=c("-Xmx4000m"))
library(rcdk)
for (smile in smiles) {
    m <- parse.smiles(smile)
    ## perform operations on this molecule
    
    jcall("java/lang/System","V","gc")
    gc()
}
```
Given a list of molecule objects, it is possible to serialize them to a file in
some specified format. Currently, the only output formats are SMILES or
SDF. To write molecules to a disk file in SDF format.
```{r eval=FALSE}
write.molecules(mols, filename='mymols.sdf')
```
By default, if mols is a list of multiple molecules, all of them will be written
to a single SDF file. If this is not desired, you can write each on to individual
files (which are prefixed by the value of filename):

```{r eval=FALSE}
 write.molecules(mols, filename='mymols.sdf', together=FALSE)
```
 
### Generating SMILES
 
 Finally, we can generate a SMILES representation of a molecule using
 
```{r}
smiles <- c('CCC', 'c1ccccc1', 'CCCC(C)(C)CC(=O)NC')
mols <- parse.smiles(smiles)
get.smiles(mols[[1]])
unlist(lapply(mols, get.smiles))
```

 
 The [CDK](https://cdk.github.io) supports a number of _flavors_ when generating SMILES. For example, you can generate a SMILES with or without chirality information or generate SMILES in [Kekule form](https://www.chemguide.co.uk/basicorg/bonding/benzene1.html). The `smiles.flavors` generates an object that represents the various flavors desired for SMILES output. See the [SmiFlavor javadocs](https://cdk.github.io/cdk/2.3/docs/api/index.html?org/openscience/cdk/smiles/SmiFlavor.html) for the full list of possible flavors. Example usage is
 
 ```{r}
smiles <- c('CCC', 'c1ccccc1', 'CCc1ccccc1CC(C)(C)CC(=O)NC')
mols <- parse.smiles(smiles)
get.smiles(mols[[3]], smiles.flavors(c('UseAromaticSymbols')))
get.smiles(mols[[3]], smiles.flavors(c('Generic','CxSmiles')))
```

Using the [CxSmiles](http://www.chemeddl.org/tools/marvin/help/formats/cxsmiles-doc.html) flavors allows the user to encode a variety of information in the SMILES string, such as 2D or 3D coordinates.

```{r}
m <- parse.smiles('CCC')[[1]]
m <- generate.2d.coordinates(m)
m
get.smiles(m, smiles.flavors(c('CxSmiles')))
get.smiles(m, smiles.flavors(c('CxCoordinates')))
```

## Visualization

The `rcdk` package supports 2D rendering of chemical structures. This can be
used to view the structure of individual molecules or multiple molecules in
a tabular format. It is also possible to view a molecular-data table, where
one of the columns is the 2D image and the remainder can contain data
associated with the molecules.

Due to Java event handling issues on OS X, depictions are handled using an external helper, which means that 
depiction generation can be slower on OS X compared to other platforms. 

Molecule visualization is performed using the `view.molecule.2d` function. This handles individual molecules as well as a list of molecules. In the latter case, the depictions are arranged in a grid (with 4 columns by default).

```{r eval=FALSE}
smiles <- c('CCC', 'CCN', 'CCN(C)(C)',
            'c1ccccc1Cc1ccccc1',
            'C1CCC1CC(CN(C)(C))CC(=O)CC')
mols <- parse.smiles(smiles)
view.molecule.2d(mols[[1]])
view.molecule.2d(mols)
```

The CDK depiction routines allow for extensive customization. These customizations can be accessed by creating a depictor object using `get.depictor`, which allows you to specify the size of the depiction, the depiction style (black and white, color on white, etc.), atom annotations (e.g., atom index), whether functional group abbreviations should be used or not and so on.

```{r eval=FALSE}
depictor <- get.depictor(style='cob', abbr='reagents', width=300, height=300)
view.molecule.2d(mols[[5]], depictor=depictor)
```
Once you have a depictor object, you can set individual properties using the `$` notation. This can be useful if you plan to generate a lot of depictions so that a new depictor is not recreated for each new structure.
```{r eval=FALSE}
depictor <- get.depictor(style='cob', abbr='reagents', width=300, height=300)
view.molecule.2d(mols[[5]], depictor=depictor)
depictor$setStyle('cow')
view.molecule.2d(mols[[5]], depictor=depictor)
```

The method also allows you to highlight substructures using [SMARTS](https://en.wikipedia.org/wiki/Smiles_arbitrary_target_specification). This is useful in highlight commen substructures in a set of molecules
```{r eval=FALSE}
depictor <- get.depictor(style='cob', abbr='reagents', sma='N(C)(C)')
view.molecule.2d(mols, depictor=depictor)
```

In many cases, it is useful to view a “molecular spreadsheet”, which is a table
of molecular structures along with information (numeric or textual) related to the molecules being viewed. The data is arranged in a spreadsheet like manner, with one of the columns
being molecules and the remainder being textual or numeric information.

This can be achieved using the `view.table` method which takes a list of
molecule objects and a `data.frame` containing the associated data. As expected,
the number of rows in the `data.frame` should equal the length of
the molecule list. Note that currently, there is not explicit binding between the rows of the `data.frame` and the elements of the list containing the molecules. Thus the user should take care that the ordering of the `data.frame` matches that of the list.
```{r eval=FALSE}
smiles <- c('CCC', 'CCN', 'CCN(C)(C)','c1ccccc1Cc1ccccc1')
mols <- parse.smiles(smiles)
dframe <- data.frame(x = runif(4),
  toxicity = factor(c('Toxic', 'Toxic', 'Nontoxic', 'Nontoxic')),
  solubility = c('yes', 'yes', 'no', 'yes'))
view.table(mols, dframe)
```

While the `view.molecule.2d` function is useful to visualize structures, the depictions can't be included in other visualizations such as plots. For such use cases, the `view.image.2d` function produces a raster image that can be included in plots. This function handles one molecule at a time.
```{r}
img <- view.image.2d(parse.smiles("B([C@H](CC(C)C)NC(=O)[C@H](CC1=CC=CC=C1)NC(=O)C2=NC=CN=C2)(O)O")[[1]])
plot(1:10, 1:10, pch=19)
rasterImage(img, 1,6, 5,10)
```

Finally, the `copy.image.to.clipboard` function allows you to copy a depiction to the system clipboard, from where it can be pasted into other applications. This can be more convenient than saving a raster image.

## Manipulating Molecules

In general, given a `jobjRef` for a molecule object one can access all the
class and methods of the [CDK](https://cdk.github.io) library via rJava. However this can be
cumbersome. The `rcdk` package exposes methods and
classes that manipulate molecules.

### Adding Information to Molecules

In many scenarios it’s useful to associate information with molecules. Within
R, you could always create a `data.frame` and store the molecule objects
along with relevant information in it. However, when serializing the molecules,
you want to be able to store the associated information with the structure itself (though keep in mind that only certain chemical file formats support metadata along with the structure).

Using the [CDK](https://cdk.github.io) it’s possible to directly add information to a molecule object
using properties. Note that adding such properties uses a key-value
paradigm, where the key should be of class character. The value can be
of class `integer`, `double`, `character` or `jobjRef`. Obviously, after setting
a property, you can get a property by its key.

```{r}
mol <- parse.smiles('c1ccccc1')[[1]]
set.property(mol, "title", "Molecule 1")
set.property(mol, "hvyAtomCount", 6)
get.property(mol, "title")
```
It is also possible to get all available properties at once in the from of a list.
The property names are used as the list names.
```{r}
get.properties(mol)
```
After adding such properties to the molecule, you can write it out to an SD
file, so that the property values become SD tags.
```{r eval=FALSE}
write.molecules(mol, 'tagged.sdf', write.props=TRUE)
```

### Atoms and Bonds

Probably the most important thing to do is to get the atoms and bonds of
a molecule. The code below gets the atoms and bonds as lists of `jobjRef`
objects, which can be manipulated using rJava or via other methods of this
package.
```{r}
mol <- parse.smiles('c1ccccc1C(Cl)(Br)c1ccccc1')[[1]]
atoms <- get.atoms(mol)
bonds <- get.bonds(mol)
cat('No. of atoms =', length(atoms), '\n')
cat('No. of bonds =', length(bonds), '\n')
```
Given an atom the `rcdk` package does not offer a lot of methods
to operate on it. One must access the [CDK](https://cdk.github.io) directly.
In the future more
manipulators will be added. Right now, you can get the symbol for each
atom

```{r}
unlist(lapply(atoms, get.symbol))
```
It’s also possible to get the 3D (or 2D coordinates) for an atom.
```{r}
coords <- get.point3d(atoms[[1]])
```
Given this, it’s quite easy to get the 3D coordinate matrix for a molecule
```{r}
coords <- do.call('rbind', lapply(atoms, get.point3d))
```
Once you have the coordinate matrix, a quick way to check whether the
molecule is flat is to do
```{r}
if ( any(apply(coords, 2, function(x) length(unique(x))) == 1) ) {
    print("molecule is flat")
}
```
This is quite a simplistic check that just looks at whether the X, Y or
Z coordinates are constant. To be more rigorous one could evaluate the
moments of inertia about the axes.

### Substructure matching

The [CDK](https://cdk.github.io) library supports substructure searches using [SMARTS](https://en.wikipedia.org/wiki/Smiles_arbitrary_target_specification) (or SMILES)
patterns. The implementation allows one to check whether a target molecule
contains a substructure or not as well as to retrieve the atoms and bonds
of the target molecule that match the query substructure. At this point,
the `rcdk` only support the former operation - given a query pattern, does it
occur or not in a list of target molecules. The `matches` method of this package
returns a logical vector where the $i$'th element is `TRUE` if the $i$'th target molecules contains the query substructure. An example of its
usage would be to identify molecules that contain a carbon atom that has
exactly two bonded neighbors.
```{r}
mols <- parse.smiles(c('CC(C)(C)C','c1ccc(Cl)cc1C(=O)O', 'CCC(N)(N)CC'))
query <- '[#6D2]'
matches(query, mols)
```

## Molecular Descriptors

A key requirement for the predictive modeling of molecular properties and activities are [molecular descriptors](https://en.wikipedia.org/wiki/Molecular_descriptor) - numerical charaterizations of the molecular structure.  The [CDK](https://cdk.github.io/) implements a variety of molecular descriptors, categorized into topological, constitutional, geometric, electronic and hybrid. It is possible to evaluate all available descriptors at one go, or evaluate individual descriptors.

First, we can take a look at the available descriptor categories.
```{r}
dc <- get.desc.categories()
dc
```
Given the categories we can get the names of the descriptors for a single
category. Of course, you can always provide the category name directly.
```{r}
dn <- get.desc.names(dc[4])
dn
```
Each descriptor name is actually a fully qualified Java class name for the
corresponding descriptor. These names can be supplied to `eval.desc` to
evaluate a single or multiple descriptors for one or more molecules.
```{r eval=FALSE}
aDesc <- eval.desc(mol, dn[4])
allDescs <- eval.desc(mol, dn)
```

The return value of eval.desc is a `data.frame` with the descriptors in the
columns and the molecules in the rows. For the above example we get a single
row. But given a list of molecules, we can easily get a descriptor matrix.

For example, let's build a linear regression model to predict boiling points
for the BP dataset. First we need a set of descriptors and so we evaluate
all available descriptors. Also note that since a descriptor might belong to
more than one category, we should obtain a unique set of descriptor names
```{r}
descNames <- unique(unlist(sapply(get.desc.categories(), get.desc.names)))
```
For the current discussion we focus on a few, manually selected descriptors
that we know will be related to boiling point.
```{r}
data(bpdata)
mols <- parse.smiles(bpdata[,1])
descNames <- c(
 'org.openscience.cdk.qsar.descriptors.molecular.KierHallSmartsDescriptor',
 'org.openscience.cdk.qsar.descriptors.molecular.APolDescriptor',
 'org.openscience.cdk.qsar.descriptors.molecular.HBondDonorCountDescriptor')
descs <- eval.desc(mols, descNames)
class(descs)
dim(descs)
```
When a descriptor value cannot be computed, it's value is set to `NA`. This may happen if a descriptor requires 3D coordinates, but only 2D coordinates are available. In this case, we have manually selected descriptors such that there will be no undefined values.

Given the ubiquity of certain descriptors, some of them are directly available
via their own functions. Specifically, one can calculate [TPSA](https://www.ncbi.nlm.nih.gov/pubmed/19149561) (topological
polar surface area), [AlogP](http://pubs.acs.org/doi/abs/10.1021/jp980230o) and [XlogP](http://www.compchemcons.com/xlogp/xlogp.html) without having to go through
`eval.desc`. (Note that AlogP and XlogP assume that hydrogens are explicitly specified in the molecule. This may not be true if the molecules were obtained from SMILES)
```{r}
mol <- parse.smiles('CC(=O)CC(=O)NCN')[[1]]
convert.implicit.to.explicit(mol)
get.tpsa(mol)
get.xlogp(mol)
get.alogp(mol)
```

Now that we have a descriptor matrix, we easily build a linear
regression model. First, remove `NA`’s, correlated and constant columns. The
code is shown below, but since it involves a stochastic element, we will not
run it for this example. If we were to perform feature selection, then this
type of reduction would have to be performed.

```{r}
descs <- descs[, !apply(descs, 2, function(x) any(is.na(x)) )]
descs <- descs[, !apply( descs, 2, function(x) length(unique(x)) == 1 )]
r2 <- which(cor(descs)^2 > .6, arr.ind=TRUE)
r2 <- r2[ r2[,1] > r2[,2] , ]
descs <- descs[, -unique(r2[,2])]
```

Note that the above correlation reduction step is pretty crude and there
are better ways to do it. Given the reduced descriptor matrix, we can
perform feature selection (say using [leaps](https://CRAN.R-project.org/package=leaps), [caret](https://CRAN.R-project.org/package=caret) or a [GA](https://CRAN.R-project.org/package=GenAlgo) to identify a suitable
subset of descriptors. Given that we selected the descriptors by hand, we can skip this section, and directly build the model and generate a plot of predicte versus observed BP. (Note that this is a toy example and is not an example of good QSAR practice!)
```{r}
model <- lm(BP ~ khs.sCH3 + khs.sF + apol + nHBDon, data.frame(bpdata, descs))
summary(model)
plot(bpdata$BP, predict(model, descs),
     xlab="Observed BP", ylab="Predicted BP",
     pch=19, xlim=c(100, 700), ylim=c(100, 700))
abline(0,1, col='red')
```
 


## Fingerprints

Fingerprints are a common representation used for a variety of purposes such as similarity searching and predictive modeling. The [CDK](https://cdk.github.io/) provides a variety of fingerprints ranging from path-based hashed fingerprints to circular (specifically, an implementation fo the [ECFP](http://pubs.acs.org/doi/abs/10.1021/ci100050t) fingerprints) and signature fingerprints (based on the [signature](http://pubs.acs.org/doi/abs/10.1021/ci020345w) molecular descriptor). Some of the fingerprints are represented as binary strings and other by integer vectors. The `rcdk` employs the [fingerprint](https://CRAN.R-project.org/package=fingerprint) package to support operations on the resultant fingerprints.

In this section, we present an example of using fingerprints to generate a hierarchical clustering of a set of molecules from the included boiling point dataset. We first parse the SMILES for the molecules in the dataset and then compute the fingerprints, specifying the `circular` type.
```{r}
data(bpdata)
mols <- parse.smiles(bpdata[,1])
fps <- lapply(mols, get.fingerprint, type='circular')
```

With the fingerprints, we can then compute a pairwise similarity matrix using the [Tanimoto](https://en.wikipedia.org/wiki/Chemical_similarity) metric. Since R's `hclust` method requires a distance matrix, we convert the similarity matrix to a distance matrix
```{r}
fp.sim <- fingerprint::fp.sim.matrix(fps, method='tanimoto')
fp.dist <- 1 - fp.sim
```
Finally, we can perform the clustering. In this case we use the `hclust` method though any of R's clustering methods could be used.
```{r}
cls <- hclust(as.dist(fp.dist))
plot(cls, main='A Clustering of the BP dataset', labels=FALSE)
```

Another common task for fingerprints is similarity searching. That is, 
given a collection of _target_ molecules, find those molecules that are similar
to a _query_ molecule. This is achieved by evaluating a similarity metric
between the query and each of the target molecules. Those target molecules
exceeding a user defined cutoff will be returned. With the help of the [fingerprint](https://CRAN.R-project.org/package=fingerprint) package this is easily accomplished.

For example, we can identify all the molecules in the BP dataset that have
a Tanimoto similarity of 0.3 or more with acetalehyde, and then create a tabular
summary. Note that this could also be accomplished with
molecular descriptors, in which case you’d probably evaluate the Euclidean
distance between descriptor vectors.

```{r}
query.mol <- parse.smiles('CC(=O)')[[1]]
target.mols <- parse.smiles(bpdata[,1])
query.fp <- get.fingerprint(query.mol, type='circular')
target.fps <- lapply(target.mols, get.fingerprint, type='circular')
sims <- data.frame(sim=do.call(rbind, lapply(target.fps,
     fingerprint::distance,
     fp2=query.fp, method='tanimoto')))
subset(sims, sim >= 0.3)
```

---
title: "Handling Molecular Formulae"
author: "Rajarshi Guha"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true	
vignette: >
  %\VignetteIndexEntry{Handling Molecular Formulae}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Handling Molecular Formulae

## Introduction

The molecular formula is the simplest way to characterize a molecular compound.
It specifies the actual number of atoms of each element contained
in the molecule. A molecular formula is represented by the chemical symbol
of each constituent element. If a molecule contains more than one atom for
a particular element, the quantity is shown as subscript after the chemical
symbol. Otherwise, the number of neutrons (atomic mass) that an atom is
composed can differ. This different type of atoms are known as isotopes. The
number of nuclei is represented as superscripted prefix previous to the chemical
element. Generally it is not added when the isotope that characterizes
the element is the most occurrence in nature, e.g., $C_4H_{11}O^2D$.

## Parsing a Molecule To a Molecular Formula

Front a molecule, defined as conjunct of atoms helding together by chemical
bonds, we can simplify it taking only the information about the atoms. The `rcdk`
package provides a parser to translate molecules to molecular formlulas, the
`get.mol2formula` function.

```{r}
library(rcdk)
sp <- get.smiles.parser()
molecule <- parse.smiles('N')[[1]]
convert.implicit.to.explicit(molecule)
formula <- get.mol2formula(molecule,charge=0)
```

Note that the above formula object is a `CDKFormula-class`. This class
contains some attributes that defines a molecular formula. For example,
the mass, the charge, the isotopes, the character representation of the
molecular formula and the [IMolecularFormula](https://cdk.github.io/cdk/2.3/docs/api/org/openscience/cdk/interfaces/IMolecularFormula.html) `jobjRef` object.

The molecular mass, charge and string representation for this formula are given by
```{r}
formula@mass
formula@charge
formula@string
```
The isotopes for this formula. It is formed from  three attributes. `isoto` (the
symbol expression of the isotope), `number` (number of atoms for this isotope)
and `mass` (exact mass of this isotope).
```{r}
formula@isotopes
```
Depending of the circumstances, you may want to change the charge of the
molecular formula.
```{r}
formula <- set.charge.formula(formula, charge=1)
```

## Initializing a Formula from the Symbol Expression

Other way to create a `cdkFormula` is from the symbol expression. Thus,
setting the characters of the elemental formula, the function `get.formula`
parses it to an object of `cdkFormula-class`.
```{r}
formula <- get.formula('NH4', charge = 1)
formula
```

## Generating Molecular Formula

Mass spectrometry is an essential and reliable technique to determine the
molecular mass of compounds. Conversely, one can use the measured mass
to identify the compound via its elemental formula. One of the limitations
of the method is the precision and accuracy of the instrumentation. As
a result, rather than specify exact masses, we specify tolerances or ranges
of possible mass, resulting in multiple candidate formulae for a given mass
window. The `generate.formula` function returns a list of formulae which have
a given mass (within an error window)
```{r warn=FALSE}
mfSet <- generate.formula(18.03383, window=1,
    elements=list(c("C",0,50),c("H",0,50),c("N",0,50)),
    validation=FALSE)
mfSet
```
It is important to know if an elemental formula is valid. The method `isvalid.formula`
provides this function. Two constraints can be applied, the
[nitrogen rule](https://en.wikipedia.org/wiki/Nitrogen_rule) and the  [RDBE rule](http://fiehnlab.ucdavis.edu/projects/seven-golden-rules/ring-double-bonds) (Ring Double Bond Equivalent).
```{r}
formula <- get.formula('NH4', charge = 0)
isvalid.formula(formula,rule=c("nitrogen","RDBE"))
```
We can observe that the ammonium is only valid if it is defined with charge
of +1.
```{r}
formula <- get.formula('NH4', charge = 1)
isvalid.formula(formula,rule=c("nitrogen","RDBE"))
```
The `generate.formula` method can perform these validation tests during formula generation by setting `validation=TRUE`. However, this can significantly slow down the process of genersating formulae, especially with larger mass windows. When the default of `FALSE` is used, nonsensical formulae may be generated, and it's up to the user to filter them out.

In contrast, the `generate.formula.iter` method employs the Round Robin formula generation algorithm, which is significantly faster than the default method. In contrast to `generate/formula`, this method returns an iterator which allows you to process large formula sets without excessive memory usage.
```{r warn=FALSE}
mit <- generate.formula.iter(100, charge=0, window=0.1,
                             elements=list(c("C",0,50), c("H",0,50), c("N",0,50)))
hit <- itertools::ihasNext(mit)
while (itertools::hasNext(hit)) 
    print(iterators::nextElem(hit))
```
Note that by default, this function will not check for validity of the generated formulae.

## Calculating Isotope Pattern


Due to the measurement errors in medium resolution spectrometry, a given
error window can result in a massive number of candidate formulae. The
isotope pattern of ions obtained experimentally can be compared with the
theoretical ones. The best match is reflected as the most probable elemental
formula. `rcdk` provides the function `get.isotope.pattern` which predicts
the theoretical isotope pattern given a formula.

```{r}
formula <- get.formula('CHCl3', charge = 0)
isotopes <- get.isotopes.pattern(formula,minAbund=0.1)
isotopes
```

In this example we generate a formula for a possible compound with a charge
($z \approx 0$) containing the standard elements C, H, and Cl. The isotope
pattern can be visually inspected, as shown below.
```{r}
plot(isotopes, type="h", xlab="m/z", ylab="Intensity")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visual.R
\name{view.image.2d}
\alias{view.image.2d}
\title{view.image.2d}
\usage{
view.image.2d(molecule, depictor = NULL)
}
\arguments{
\item{molecule}{The molecule to display Should be a `jobjRef` representing an `IAtomContainer`}

\item{depictor}{Default \code{NULL}}
}
\description{
view.image.2d
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\name{get.point2d}
\alias{get.point2d}
\title{get.point2d}
\usage{
get.point2d(atom)
}
\arguments{
\item{atom}{The atom to query}
}
\value{
A 2-element numeric vector representing the X & Y coordinates.
}
\description{
Get the 2D coordinates of the atom.
}
\details{
In case, coordinates are unavailable (e.g., molecule was read in from a 
SMILES file) or have not been generated yet, `NA`'s are returned for the 
X & Y coordinates.
}
\examples{
\dontrun{
atoms <- get.atoms(mol)
coords <- do.call('rbind', lapply(apply, get.point2d))
}
}
\seealso{
\code{\link{get.point3d}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io.R
\name{iload.molecules}
\alias{iload.molecules}
\title{Load molecules using an iterator.}
\usage{
iload.molecules(
  molfile,
  type = "smi",
  aromaticity = TRUE,
  typing = TRUE,
  isotopes = TRUE,
  skip = TRUE
)
}
\arguments{
\item{molfile}{A string containing the filename to load. Must be a local file}

\item{type}{Indicates whether the input file is SMILES or SDF. Valid values are
`"smi"` or `"sdf"`}

\item{aromaticity}{If `TRUE` then aromaticity detection is
performed on all loaded molecules. If this fails for a given
molecule, then the molecule is set to `NA` in the return list}

\item{typing}{If `TRUE` then atom typing is
performed on all loaded molecules. The assigned types will be CDK
internal types. If this fails for a given molecule, then the molecule 
is set to `NA` in the return list}

\item{isotopes}{If `TRUE` then atoms are configured with isotopic masses}

\item{skip}{If `TRUE`, then the reader will continue reading even when 
faced with an invalid molecule. If `FALSE`, the reader will stop at 
the fist invalid molecule}
}
\description{
The CDK can read a variety of molecular structure formats. Some file
formats support multiple molecules in a single file. If read using
\code{\link{load.molecules}}, all are read into memory. For very large
structure files, this can lead to out of memory errors. Instead it is 
recommended to use the iterating version of the loader so that only a
single molecule is read at a time.
}
\details{
Note that the iterating loader only supports SDF and SMILES file formats.
}
\examples{
\dontrun{
moliter <- iload.molecules("big.sdf", type="sdf")
while(hasNext(moliter)) {
mol <- nextElem(moliter)
  print(get.property(mol, "cdk:Title"))
}
}
}
\seealso{
\code{\link{write.molecules}}, \code{\link{load.molecules}}, \code{\link{parse.smiles}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{get.largest.component}
\alias{get.largest.component}
\title{Gets the largest component in a disconnected molecular graph.}
\usage{
get.largest.component(mol)
}
\arguments{
\item{mol}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}
}
\value{
The largest component as an `IAtomContainer` object or else the input molecule itself
}
\description{
A molecule may be represented as a 
\href{http://mathworld.wolfram.com/DisconnectedGraph.html}{disconnected graph}, such as
when read in as a salt form. This method will return the larges connected component
or if there is only a single component (i.e., the molecular graph is 
\href{https://en.wikipedia.org/wiki/Complete_graph}{complete} or fully connected), that
component is returned.
}
\examples{
m <- parse.smiles("CC.CCCCCC.CCCC")[[1]]
largest <- get.largest.component(m)
length(get.atoms(largest)) == 6
}
\seealso{
\code{\link{is.connected}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/props.R
\name{remove.property}
\alias{remove.property}
\title{Remove a property associated with a molecule.}
\usage{
remove.property(molecule, key)
}
\arguments{
\item{molecule}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}

\item{key}{The property key as a character string}
}
\description{
In this context a property is a value associated with a key and stored
with the molecule. This methd will remove the property defined by the key.
If there is such key, a warning is raised.
}
\examples{
mol <- parse.smiles("CC1CC(C=O)CCC1")[[1]]
set.property(mol, 'prop1', 23.45)
set.property(mol, 'prop2', 'inactive')
get.properties(mol)
remove.property(mol, 'prop2')
get.properties(mol)
}
\seealso{
\code{\link{set.property}}, \code{\link{get.property}}, \code{\link{get.properties}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\name{get.atomic.number}
\alias{get.atomic.number}
\title{Get the atomic number of the atom.}
\usage{
get.atomic.number(atom)
}
\arguments{
\item{atom}{The atom to query}
}
\value{
An integer representing the atomic number
}
\description{
Get the atomic number of the atom.
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{convert.implicit.to.explicit}
\alias{convert.implicit.to.explicit}
\title{Convert implicit hydrogens to explicit.}
\usage{
convert.implicit.to.explicit(mol)
}
\arguments{
\item{mol}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}
}
\description{
In some cases, a molecule may not have any hydrogens (such as when read
in from an MDL MOL file that did not have hydrogens or SMILES with no
explicit hydrogens). In such cases, this method
will add implicit hydrogens and then convert them to explicit ones. The 
newly added H's will not have any 2D or 3D coordinates associated with them.
Ensure that the molecule has been typed beforehand.
}
\seealso{
\code{\link{get.hydrogen.count}}, \code{\link{remove.hydrogens}}, \code{\link{do.typing}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\name{get.atom.index}
\alias{get.atom.index}
\title{Get the index of an atom in a molecule.

Acces the index of an atom in the context of an IAtomContainer. 
Indexing starts from 0. If the index is not known, -1 is returned.}
\usage{
get.atom.index(atom, mol)
}
\arguments{
\item{atom}{The atom object}

\item{mol}{The `IAtomContainer` object containing the atom}
}
\value{
An integer representing the atom index.
}
\description{
Get the index of an atom in a molecule.

Acces the index of an atom in the context of an IAtomContainer. 
Indexing starts from 0. If the index is not known, -1 is returned.
}
\seealso{
\code{\link{get.connected.atom}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\name{get.hydrogen.count}
\alias{get.hydrogen.count}
\alias{hydrogen}
\title{get.hydrogen.count}
\usage{
get.hydrogen.count(atom)
}
\arguments{
\item{atom}{The atom to query}
}
\value{
An integer representing the hydrogen count
}
\description{
Get the implicit hydrogen count for the atom.
}
\details{
This method returns the number of implicit H's on the atom. 
Depending on where the molecule was read from this may be \code{NULL} or an integer
greater than or equal to 0
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\docType{class}
\name{cdkFormula-class}
\alias{cdkFormula-class}
\alias{show,cdkFormula-method}
\title{Class cdkFormula, ac class for handling molecular formula}
\description{
This class handles molecular formulae. It provides extra 
 information such as the IMolecularFormula Java object, elements contained 
 and number of them.
}
\section{Objects from the Class}{

  Objects can be created using new constructor and filled with a specific 
  mass and window accuracy
}

\references{
A parallel effort to expand the Chemistry Development 
  Kit: \url{http://cdk.sourceforge.net}
}
\seealso{
\code{\link{get.formula}}
\code{\link{set.charge.formula}}
\code{\link{get.isotopes.pattern}}
\code{\link{isvalid.formula}}
}
\author{
Miguel Rojas-Cherto (\email{miguelrojasch@yahoo.es})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bonds.R
\name{get.bond.order}
\alias{get.bond.order}
\title{Get an object representing bond order}
\usage{
get.bond.order(order = "single")
}
\arguments{
\item{order}{A character vector that can be one of single, double, triple,
quadruple, quintuple, sextuple or unset. Case is ignored}
}
\value{
A \code{jObjRef} representing an `Order` enum object
}
\description{
This function returns a Java enum representing a
bond order. This can be used to modify the order
of pre-existing bonds
}
\examples{
\dontrun{
m <- parse.smiles('CCN')[[1]]
b <- get.bonds(m)[[1]]
b$setOrder(get.bond.order("double"))
}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{get.total.hydrogen.count}
\alias{get.total.hydrogen.count}
\title{Get total number of implicit hydrogens in the molecule.}
\usage{
get.total.hydrogen.count(mol)
}
\arguments{
\item{mol}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}
}
\value{
An integer representing the total number of implicit hydrogens
}
\description{
Counts the number of hydrogens on the provided molecule. As this method 
will sum all implicit hydrogens on each atom it is important to ensure 
the molecule has already been configured (and thus each atom has an 
implicit hydrogen count).
}
\seealso{
\code{\link{get.hydrogen.count}}, \code{\link{remove.hydrogens}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io.R
\name{write.molecules}
\alias{write.molecules}
\title{Write molecules to disk.}
\usage{
write.molecules(mols, filename, together = TRUE, write.props = FALSE)
}
\arguments{
\item{mols}{A `list` of `jobjRef` objects representing  `IAtomContainer` objects}

\item{filename}{The name of the SD file to write. Note that if
`together` is `FALSE` then this argument is taken as a prefix for
the name of the individual files}

\item{together}{If `TRUE` then all the molecules are written to a
single SD file. If `FALSE` each molecule is written to an
individual file}

\item{write.props}{If `TRUE`, keyed properties are included in the SD file output}
}
\description{
This function writes one or more molecules to an SD file on disk,
which can be of the single- or multi-molecule variety. In
addition, if the molecule has keyed properties, they can also be
written out as SD tags.
}
\details{
In case individual SD files are desired the
\code{together} argument can be set ot \code{FALSE}. In this case, the
value of \code{filename} is used as a prefix, to which a numeric
identifier and the suffix of ".sdf" is appended.
}
\seealso{
\code{\link{load.molecules}}, \code{\link{parse.smiles}}, \code{\link{iload.molecules}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{get.atom.count}
\alias{get.atom.count}
\title{Get the number of atoms in the molecule.}
\usage{
get.atom.count(mol)
}
\arguments{
\item{mol}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}
}
\value{
An integer representing the number of atoms in the molecule
}
\description{
Get the number of atoms in the molecule.
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frags.R
\name{get.exhaustive.fragments}
\alias{get.exhaustive.fragments}
\title{Generate Bemis-Murcko Fragments}
\usage{
get.exhaustive.fragments(mols, min.frag.size = 6, as.smiles = TRUE)
}
\arguments{
\item{mols}{A list of `jobjRef` objects of Java class `IAtomContainer`}

\item{min.frag.size}{The smallest fragment to consider (in terms of heavy atoms)}

\item{as.smiles}{If `TRUE` return the fragments as SMILES strings. If not, then fragments
are returned as `jobjRef` objects}
}
\value{
returns a list of length equal to the number of input molecules. Each
element is a character vector of SMILES strings or a list of `jobjRef` objects.
}
\description{
Fragment the input molecule using the Bemis-Murcko scheme
}
\details{
A variety of methods for fragmenting molecules are available ranging from
exhaustive, rings to more specific methods such as Murcko frameworks. Fragmenting a
collection of molecules can be a useful for a variety of analyses. In addition
fragment based analysis can be a useful and faster alternative to traditional 
clustering of the whole collection, especially when it is large.

Note that exhaustive fragmentation of large molecules (with many single bonds) can become
time consuming.
}
\examples{
mol <- parse.smiles('c1ccc(cc1)CN(c2cc(ccc2[N+](=O)[O-])c3c(nc(nc3CC)N)N)C')[[1]]
mf1 <- get.murcko.fragments(mol, as.smiles=TRUE, single.framework=TRUE)
mf1 <- get.murcko.fragments(mol, as.smiles=TRUE, single.framework=FALSE)
}
\seealso{
[get.exhuastive.fragments()]
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smiles.R
\name{get.smiles.parser}
\alias{get.smiles.parser}
\title{Get a SMILES parser object.}
\usage{
get.smiles.parser()
}
\value{
A `jobjRef` object corresponding to the CDK 
\href{http://cdk.github.io/cdk/2.2/docs/api/org/openscience/cdk/smiles/SmilesParser.html}{SmilesParser} class
}
\description{
This function returns a reference to a SMILES parser
object. If you are parsing multiple SMILES strings using multiple
calls to \code{\link{parse.smiles}}, it is
preferable to create your own parser and supply it to
\code{\link{parse.smiles}} rather than forcing that function
to instantiate a new parser for each call
}
\seealso{
\code{\link{get.smiles}}, \code{\link{parse.smiles}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{get.bonds}
\alias{get.bonds}
\title{Get the bonds in a molecule.}
\usage{
get.bonds(mol)
}
\arguments{
\item{mol}{A `jobjRef` representing the molecule (`IAtomContainer`) object.}
}
\value{
A list of `jobjRef` representing the bonds (`IBond`) objects in the molecule
}
\description{
Get the bonds in a molecule.
}
\seealso{
\code{\link{get.atoms}}, \code{\link{get.connected.atoms}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smsd.R
\name{get.mcs}
\alias{get.mcs}
\title{get.mcs}
\usage{
get.mcs(mol1, mol2, as.molecule = TRUE)
}
\arguments{
\item{mol1}{Required. First molecule to compare. Should be a `jobjRef` representing an `IAtomContainer`}

\item{mol2}{Required. Second molecule to compare. Should be a `jobjRef` representing an `IAtomContainer`}

\item{as.molecule}{Optional. Default \code{TRUE}.}
}
\description{
get.mcs
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matrix.R
\name{get.adjacency.matrix}
\alias{get.adjacency.matrix}
\title{Get adjacency matrix for a molecule.}
\usage{
get.adjacency.matrix(mol)
}
\arguments{
\item{mol}{A \code{jobjRef} object with Java class \code{IAtomContainer}}
}
\value{
A \eqn{N \times N} numeric matrix
}
\description{
The adjacency matrix for a molecule with \eqn{N} non-hydrogen atoms is an
\eqn{N \times N} matrix where the element [\eqn{i},\eqn{j}] is set to 1
if atoms \eqn{i} and \eqn{j} are connected by a bond, otherwise set to 0.
}
\examples{
m <- parse.smiles("CC=C")[[1]]
get.adjacency.matrix(m)
}
\seealso{
\code{\link{get.connection.matrix}}
}
\author{
Rajarshi Guha \email{rajarshi.guha@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{do.typing}
\alias{do.typing}
\title{do.typing}
\usage{
do.typing(mol)
}
\arguments{
\item{mol}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}
}
\description{
configure atom typings.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\name{set.atom.types}
\alias{set.atom.types}
\title{set.atom.types}
\usage{
set.atom.types(mol)
}
\arguments{
\item{mol}{The molecule whose atoms should be typed}
}
\value{
Nothing is returned, the molecule is modified in place
}
\description{
Set the CDK atom types for all atoms in the molecule.
}
\details{
Calling this method will overwrite any pre-existing type information. 
Currently there is no way to choose other atom typing schemes
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stereo.R
\name{get.element.types}
\alias{get.element.types}
\title{Obtain the type of stereo element support for atom.}
\usage{
get.element.types(mol)
}
\arguments{
\item{mol}{A \code{jObjRef} representing an \code{IAtomContainer}}
}
\value{
A factor of length equal in length to the number of atoms, indicating the element type
}
\description{
Supported elements types are
\describe{
\item{Bicoordinate}{an central atom involved in a cumulated system (not yet supported)}
\item{Tricoordinate}{an atom at one end of a geometric (double-bond) stereo bond or cumulated system}
\item{Tetracoordinate}{a tetrahedral atom (could also be square planar in future)}
\item{None}{the atom is not a (supported) stereo element type}
}
}
\seealso{
\code{\link{get.stereocenters}}, \code{\link{get.stereo.types}}
}
\author{
Rajarshi Guha \email{rajarshi.guha@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc.R
\name{get.alogp}
\alias{get.alogp}
\title{Compute ALogP for a molecule}
\usage{
get.alogp(molecule)
}
\arguments{
\item{molecule}{A molecule object}
}
\value{
A double value representing the ALogP value
}
\description{
Compute ALogP for a molecule
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\name{get.isotope.pattern.generator}
\alias{get.isotope.pattern.generator}
\title{Construct an isotope pattern generator.}
\usage{
get.isotope.pattern.generator(minAbundance = NULL)
}
\arguments{
\item{minAbundance}{The minimum abundance}
}
\value{
A \code{jobjRef} corresponding to an instance of \code{IsotopePatternGenerator}
}
\description{
Constructs an instance of the CDK \code{IsotopePatternGenerator}, with an optional
minimum abundance specified. This object can be used to generate all combinatorial
chemical isotopes given a structure.
}
\references{
\url{http://cdk.github.io/cdk/1.5/docs/api/org/openscience/cdk/formula/IsotopePatternGenerator.html}
}
\author{
Miguel Rojas Cherto
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\name{isvalid.formula}
\alias{isvalid.formula}
\title{isvalid.formula}
\usage{
isvalid.formula(formula, rule = c("nitrogen", "RDBE"))
}
\arguments{
\item{formula}{Required. A CDK Formula}

\item{rule}{Optional. Default \code{rule=c("nitrogen","RDBE")}}
}
\description{
Validate a cdkFormula.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{is.neutral}
\alias{is.neutral}
\alias{charge}
\title{Tests whether the molecule is neutral.}
\usage{
is.neutral(mol)
}
\arguments{
\item{mol}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}
}
\value{
`TRUE` if molecule is neutral, `FALSE` otherwise
}
\description{
The test checks whether all atoms in the molecule have a formal charge of 0.
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{get.total.charge}
\alias{get.total.charge}
\title{get.total.charge}
\usage{
get.total.charge(mol)
}
\arguments{
\item{mol}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}
}
\description{
get.total.charge
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smiles.R
\name{smiles.flavors}
\alias{smiles.flavors}
\title{Generate flag for customizing SMILES generation.}
\usage{
smiles.flavors(flavors = c("Generic"))
}
\arguments{
\item{flavors}{A character vector of flavors. The default is \code{Generic}
(output non-canonical SMILES without stereochemistry, atomic masses). Possible
values are
\itemize{
\item Absolute
\item AtomAtomMap
\item AtomicMass
\item AtomicMassStrict
\item Canonical
\item Cx2dCoordinates
\item Cx3dCoordinates
\item CxAtomLabel
\item CxAtomValue
\item CxCoordinates
\item CxFragmentGroup
\item CxMulticenter
\item CxPolymer
\item CxRadical
\item CxSmiles
\item CxSmilesWithCoords
\item Default
\item Generic
\item InChILabelling
\item Isomeric
\item Stereo
\item StereoCisTrans
\item StereoExTetrahedral
\item StereoTetrahedral
\item Unique
\item UniversalSmiles
\item UseAromaticSymbols
}}
}
\value{
A numeric representing the bitwise `OR`` of the specified flavors
}
\description{
The CDK supports a variety of customizations for SMILES generation including
the use of lower case symbols for aromatic compounds to the use of the ChemAxon
\href{http://www.chemeddl.org/tools/marvin/help/formats/cxsmiles-doc.html}{CxSmiles}
format. Each 'flavor' is represented by an integer and multiple
customizations are bitwise OR'ed. This method accepts the names of one or
more customizations and returns the bitwise OR of them.
See \href{https://cdk.github.io/cdk/2.3/docs/api/index.html?org/openscience/cdk/smiles/SmiFlavor.html}{CDK documentation}
for the list of flavors and what they mean.
}
\examples{
m <- parse.smiles('C1C=CCC1N(C)c1ccccc1')[[1]]
get.smiles(m)
get.smiles(m, smiles.flavors(c('Generic','UseAromaticSymbols')))

m <- parse.smiles("OS(=O)(=O)c1ccc(cc1)C(CC)CC |Sg:n:13:m:ht,Sg:n:11:n:ht|")[[1]]
get.smiles(m,flavor = smiles.flavors(c("CxSmiles")))
get.smiles(m,flavor = smiles.flavors(c("CxSmiles","UseAromaticSymbols")))

}
\references{
\href{https://cdk.github.io/cdk/2.3/docs/api/index.html?org/openscience/cdk/smiles/SmiFlavor.html}{CDK documentation}
}
\seealso{
\code{\link{get.smiles}}
}
\author{
Rajarshi Guha \email{rajarshi.guha@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\name{is.in.ring}
\alias{is.in.ring}
\title{is.in.ring}
\usage{
is.in.ring(atom)
}
\arguments{
\item{atom}{The atom to test}
}
\value{
`TRUE` is the atom is in a ring, `FALSE` otherwise
}
\description{
Tests whether an atom is in a ring.
}
\details{
This assumes that the molecule containing the atom has been 
appropriately configured.
}
\seealso{
\code{\link{is.aliphatic}}, \code{\link{is.aromatic}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visual.R
\name{view.molecule.2d}
\alias{view.molecule.2d}
\title{view.molecule.2d}
\usage{
view.molecule.2d(
  molecule,
  ncol = 4,
  width = 200,
  height = 200,
  depictor = NULL
)
}
\arguments{
\item{molecule}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}

\item{ncol}{Default \code{4}}

\item{width}{Default \code{200}}

\item{height}{Default \code{200}}

\item{depictor}{Default \code{NULL}}
}
\description{
Create a 2D depiction of a molecule. If there are more than
one molecules supplied, return a grid woth \code{ncol} columns,.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visual.R
\name{get.depictor}
\alias{get.depictor}
\title{get.depictor}
\usage{
get.depictor(
  width = 200,
  height = 200,
  zoom = 1.3,
  style = "cow",
  annotate = "off",
  abbr = "on",
  suppressh = TRUE,
  showTitle = FALSE,
  smaLimit = 100,
  sma = NULL
)
}
\arguments{
\item{width}{Default. \code{200}}

\item{height}{Default. \code{200}}

\item{zoom}{Default. \code{1.3}}

\item{style}{Default. \code{cow}}

\item{annotate}{Default. \code{off}}

\item{abbr}{Default. \code{on}}

\item{suppressh}{Default. \code{TRUE}}

\item{showTitle}{Default. \code{FALSE}}

\item{smaLimit}{Default. \code{100}}

\item{sma}{Default. \code{NULL}}
}
\description{
return an RcdkDepictor.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visual.R
\name{view.table}
\alias{view.table}
\title{view.table}
\usage{
view.table(molecules, dat, depictor = NULL)
}
\arguments{
\item{molecules}{A list of molecule objects (`jobjRef` representing an `IAtomContainer`)}

\item{dat}{The \code{data.frame} associated with the molecules, one per row}

\item{depictor}{Default \code{NULL}}
}
\description{
Create a tabular view of a set of molecules (in 2D) and associated data columns
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc.R
\name{eval.atomic.desc}
\alias{eval.atomic.desc}
\title{Compute descriptors for each atom in a molecule}
\usage{
eval.atomic.desc(molecule, which.desc, verbose = FALSE)
}
\arguments{
\item{molecule}{A molecule object}

\item{which.desc}{A character vector of atomic descriptor class names}

\item{verbose}{Optional. Default \code{FALSE}. Toggle verbosity.}
}
\value{
A `data.frame` with atoms in the rows and descriptors in the columns
}
\description{
Compute descriptors for each atom in a molecule
}
\seealso{
\link{get.atomic.desc.names}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\name{get.isotope.pattern.similarity}
\alias{get.isotope.pattern.similarity}
\title{Construct an isotope pattern similarity calculator.}
\usage{
get.isotope.pattern.similarity(tol = NULL)
}
\arguments{
\item{tol}{The tolerance}
}
\value{
A \code{jobjRef} corresponding to an instance of \code{IsotopePatternSimilarity}
}
\description{
A method that returns an instance of the CDK \code{IsotopePatternSimilarity}
class which can be used to compute similarity scores between pairs of
isotope abundance patterns.
}
\references{
\url{http://cdk.github.io/cdk/1.5/docs/api/org/openscience/cdk/formula/IsotopePatternSimilarity.html}
}
\seealso{
\code{\link{compare.isotope.pattern}}
}
\author{
Miguel Rojas Cherto
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\name{set.charge.formula}
\alias{set.charge.formula}
\title{set.charge.formula}
\usage{
set.charge.formula(formula, charge = -1)
}
\arguments{
\item{formula}{Required. Molecular formula}

\item{charge}{Optional. Default \code{-1}}
}
\description{
Set the charge to a cdkFormula function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smiles.R
\name{get.smiles}
\alias{get.smiles}
\title{Generate a SMILES representation of a molecule.}
\usage{
get.smiles(molecule, flavor = smiles.flavors(c("Generic")), smigen = NULL)
}
\arguments{
\item{molecule}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}

\item{flavor}{The type of SMILES to generate. See \code{\link{smiles.flavors}}. Default is `Generic`
SMILES}

\item{smigen}{A pre-existing SMILES generator object. By default, a new one is created from the specified flavor}
}
\value{
A character string containing the generated SMILES
}
\description{
The function will generate a SMILES representation of an
`IAtomContainer` object. The default parameters of the CDK SMILES
generator are used. This can mean that for large ring systems the
method may fail. See CDK \href{http://cdk.github.io/cdk/2.2/docs/api/org/openscience/cdk/smiles/SmilesGenerator.html}{Javadocs}
for more information
}
\examples{
m <- parse.smiles('C1C=CCC1N(C)c1ccccc1')[[1]]
get.smiles(m)
get.smiles(m, smiles.flavors(c('Generic','UseAromaticSymbols')))
}
\references{
\href{http://cdk.github.io/cdk/2.2/docs/api/org/openscience/cdk/smiles/SmilesGenerator.html}{SmilesGenerator}
}
\seealso{
\code{\link{parse.smiles}}, \code{\link{smiles.flavors}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated_functions.R
\docType{package}
\name{rcdk-deprecated}
\alias{rcdk-deprecated}
\alias{deprecated_rcdk_function}
\alias{do.typing}
\title{Deprecated functions in the rcdk package.}
\usage{
deprecated_rcdk_function(x, value, ...)
}
\arguments{
\item{x}{For assignment operators, the object that will undergo a replacement
(object inside parenthesis).}

\item{value}{For assignment operators, the value to replace with 
(the right side of the assignment).}

\item{...}{For functions other than assignment operators, 
parameters to be passed to the modern version of the function (see table).}
}
\description{
These functions are provided for compatibility with older version of
the phyloseq package.  They may eventually be completely
removed.
}
\details{
\tabular{rl}{
  \code{do.typing} \tab now a synonym for \code{\link{set.atom.types}}\cr
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\name{get.formula}
\alias{get.formula}
\title{get.formula}
\usage{
get.formula(mf, charge = 0)
}
\arguments{
\item{mf}{Required. Molecular formula}

\item{charge}{Optional. Default \code{0}}
}
\description{
obtain molecular formula from formula string
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{do.aromaticity}
\alias{do.aromaticity}
\title{do.aromaticity}
\usage{
do.aromaticity(mol)
}
\arguments{
\item{mol}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}
}
\description{
detect aromaticity of an input compound
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{get.atoms}
\alias{get.atoms}
\title{Get the atoms from a molecule or bond.}
\usage{
get.atoms(object)
}
\arguments{
\item{object}{A `jobjRef` representing either a molecule (`IAtomContainer`) or 
bond (`IBond`) object.}
}
\value{
A list of `jobjRef` representing the `IAtom` objects in the molecule or bond
}
\description{
Get the atoms from a molecule or bond.
}
\seealso{
\code{\link{get.bonds}}, \code{\link{get.connected.atoms}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\name{compare.isotope.pattern}
\alias{compare.isotope.pattern}
\title{Compare isotope patterns.}
\usage{
compare.isotope.pattern(iso1, iso2, ips = NULL)
}
\arguments{
\item{iso1}{The first isotope pattern, which should be a \code{jobjRef} corresponding to the \code{IsotopePattern} class}

\item{iso2}{The second isotope pattern, which should be a \code{jobjRef} corresponding to the \code{IsotopePattern} class}

\item{ips}{An instance of the \code{IsotopePatternSimilarity} class. if \code{NULL} one will be constructed automatically}
}
\value{
A numeric value between 0 and 1 indicating the similarity between the two patterns
}
\description{
Computes a similarity score between two different isotope abundance patterns.
}
\references{
\url{http://cdk.github.io/cdk/2.3/docs/api/org/openscience/cdk/formula/IsotopePatternSimilarity.html}
}
\seealso{
\code{\link{get.isotope.pattern.similarity}}
}
\author{
Miguel Rojas Cherto
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/props.R
\name{set.property}
\alias{set.property}
\title{Set a property value of the molecule.}
\usage{
set.property(molecule, key, value)
}
\arguments{
\item{molecule}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}

\item{key}{The property key as a character string}

\item{value}{The value of the property. This can be a character, numeric or 
`jobjRef` R object}
}
\description{
This function sets the value of a keyed property on the molecule. 
Properties enable us to associate arbitrary pieces of data with a 
molecule. Such data can be text, numeric or a Java object 
(represented as a `jobjRef`).
}
\examples{
mol <- parse.smiles("CC1CC(C=O)CCC1")[[1]]
set.property(mol, 'prop1', 23.45)
set.property(mol, 'prop2', 'inactive')
get.property(mol, 'prop1')

}
\seealso{
\code{\link{get.property}}, \code{\link{get.properties}}, \code{\link{remove.property}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/io.R
\name{load.molecules}
\alias{load.molecules}
\title{Load molecular structures from disk or URL}
\usage{
load.molecules(
  molfiles = NA,
  aromaticity = TRUE,
  typing = TRUE,
  isotopes = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{molfiles}{A `character` vector of filenames. Note that the full
path to the files should be provided. URL's can also be used as
paths. In such a case, the URL should start with "http://"}

\item{aromaticity}{If `TRUE` then aromaticity detection is
performed on all loaded molecules. If this fails for a given
molecule, then the molecule is set to `NA` in the return list}

\item{typing}{If `TRUE` then atom typing is
performed on all loaded molecules. The assigned types will be CDK
internal types. If this fails for a given molecule, then the molecule 
is set to `NA` in the return list}

\item{isotopes}{If `TRUE` then atoms are configured with isotopic masses}

\item{verbose}{If `TRUE`, output (such as file download progress) will
be bountiful}
}
\value{
A `list` of CDK `IAtomContainer` objects, represented as `jobjRef` objects 
in R, which can be used in other `rcdk` functions
}
\description{
The CDK can read a variety of molecular structure formats. This function
encapsulates the calls to the CDK API to load a structure given its filename
or a URL to a structure file.
}
\details{
Note that this method will load all molecules into memory. For files containing
tens of thousands of molecules this may lead to out of memory errors. In such 
situations consider using the iterating file readers.

Note that if molecules are read in from formats that do not have rules for
handling implicit hydrogens (such as MDL MOL), the molecule will not have
implicit or explicit hydrogens. To add explicit hydrogens, make sure that the molecule
has been typed (this is `TRUE` by default for this function) and then call 
\code{\link{convert.implicit.to.explicit}}. On the other hand for a format 
such as SMILES, implicit or explicit hydrogens will be present.
}
\examples{
\dontrun{
sdffile <- system.file("molfiles/dhfr00008.sdf", package="rcdk")
mols <- load.molecules(c('mol1.sdf', 'mol2.smi', sdfile))
}
}
\seealso{
\code{\link{write.molecules}}, \code{\link{parse.smiles}}, \code{\link{iload.molecules}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\name{get.isotopes.pattern}
\alias{get.isotopes.pattern}
\title{get.isotopes.pattern}
\usage{
get.isotopes.pattern(formula, minAbund = 0.1)
}
\arguments{
\item{formula}{Required. A CDK molecule formula}

\item{minAbund}{Optional. Default \code{0.1}}
}
\description{
Generate the isotope pattern given a formula class
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\docType{package}
\name{Atoms}
\alias{Atoms}
\title{Operations on Atoms}
\description{
\code{\link{get.symbol}} returns the chemical symbol for an atom
\code{\link{get.point3d}} returns the 3D coordinates of the atom
\code{\link{get.point2d}} returns the 2D coordinates of the atom
\code{\link{get.atomic.number}} returns the atomic number of the atom
\code{\link{get.hydrogen.count}}  returns the number of implicit H’s on the atom. 
          Depending on where the molecule was read from this may be \code{NULL}
          or an integer greater than or equal to \code{0}
\code{\link{get.charge}} returns the partial charge on the atom. If charges 
          have not been set the return value is \code{NULL}, otherwise 
          the appropriate charge.
\code{\link{get.formal.charge}} returns the formal charge on the atom. By 
           default the formal charge will be \code{0} (i.e., \code{NULL}
           is never returned)
\code{\link{is.aromatic}} returns \code{TRUE} if the atom is aromatic,
           \code{FALSE} otherwise
\code{\link{is.aliphatic}} returns \code{TRUE} if the atom is part of an 
           aliphatic chain, \code{FALSE} otherwise
\code{\link{is.in.ring}} returns \code{TRUE} if the atom is in a ring, 
            \code{FALSE} otherwise
\code{\link{get.atom.index}} eturns the index of the atom in the molecule
            (starting from \code{0})
\code{\link{get.connected.atoms}} returns a list of atoms that are connected to the specified atom
}
\section{Usage}{

get.symbol(atom)
get.point3d(atom)
get.point2d(atom)
get.atomic.number(atom)
get.hydrogen.count(atom)
get.charge(atom)
get.formal.charge(atom)
get.connected.atoms(atom, mol)
get.atom.index(atom, mol)
is.aromatic(atom)
is.aliphatic(atom)
is.in.ring(atom)
}

\section{Arguments}{

 atom A jobjRef representing an IAtom object
 mol  A jobjRef representing an IAtomContainer object
}

\section{Value}{

 In the case of \code{\link{get.point3d}} the return value is a 3-element vector 
 containingthe X, Y and Z co-ordinates of the atom.  If the atom does not 
 have 3D coordinates, it returns a vector of the form \code{c(NA,NA,NA)}.
 Similarly for \code{\link{get.point2d}}, in which case the return vector is of 
 length \code{2}.
}

\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stereo.R
\name{get.stereo.types}
\alias{get.stereo.types}
\title{Obtain the stereocenter type for atom.}
\usage{
get.stereo.types(mol)
}
\arguments{
\item{mol}{A \code{jObjRef} representing an \code{IAtomContainer}}
}
\value{
A factor of length equal in length to the number of atoms indicating the stereocenter type.
}
\description{
Supported stereo center types are
\describe{
\item{True}{the atom has constitutionally different neighbors}
\item{Para}{the atom resembles a stereo centre but has constitutionally equivalent neighbors (e.g. inositol, decalin). The stereocenter depends on the configuration of one or more stereocenters.}
\item{Potential}{the atom can supported stereo chemistry but has not be shown ot be a true or para center}
\item{Non}{the atom is not a stereocenter (e.g. methane)}
}
}
\seealso{
\code{\link{get.stereocenters}}, \code{\link{get.element.types}}
}
\author{
Rajarshi Guha \email{rajarshi.guha@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\name{get.symbol}
\alias{get.symbol}
\title{Get the atomic symbol of the atom.

Get the atomic symbol of the atom.}
\usage{
get.symbol(atom)
}
\arguments{
\item{atom}{The atom to query}
}
\value{
A character representing the atomic symbol
}
\description{
Get the atomic symbol of the atom.

Get the atomic symbol of the atom.
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\name{get.point3d}
\alias{get.point3d}
\title{Get the 3D coordinates of the atom.

In case, coordinates are unavailable (e.g., molecule was read in from a 
SMILES file) or have not been generated yet, `NA`'s are returned for the 
X, Y and Z coordinates.}
\usage{
get.point3d(atom)
}
\arguments{
\item{atom}{The atom to query}
}
\value{
A 3-element numeric vector representing the X, Y and Z coordinates.
}
\description{
Get the 3D coordinates of the atom.

In case, coordinates are unavailable (e.g., molecule was read in from a 
SMILES file) or have not been generated yet, `NA`'s are returned for the 
X, Y and Z coordinates.
}
\examples{
\dontrun{
atoms <- get.atoms(mol)
coords <- do.call('rbind', lapply(apply, get.point3d))
}
}
\seealso{
\code{\link{get.point2d}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/props.R
\name{get.properties}
\alias{get.properties}
\title{Get all properties associated with a molecule.}
\usage{
get.properties(molecule)
}
\arguments{
\item{molecule}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}
}
\value{
A named `list` with the property values. Element names are the keys 
for each property. If no properties have been defined, an empty list.
}
\description{
In this context a property is a value associated with a key and stored
with the molecule. This methd returns a list of all the properties of 
a molecule. The names of the list are set to the property names.
}
\examples{
mol <- parse.smiles("CC1CC(C=O)CCC1")[[1]]
set.property(mol, 'prop1', 23.45)
set.property(mol, 'prop2', 'inactive')
get.properties(mol)
}
\seealso{
\code{\link{set.property}}, \code{\link{get.property}}, \code{\link{remove.property}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\name{get.connected.atoms}
\alias{get.connected.atoms}
\title{Get atoms connected to the specified atom

Returns a `list`` of atoms that are connected to the specified atom.}
\usage{
get.connected.atoms(atom, mol)
}
\arguments{
\item{atom}{The atom object}

\item{mol}{The `IAtomContainer` object containing the atom}
}
\value{
A `list` containing `IAtom` objects, representing the atoms directly
connected to the specified atom
}
\description{
Get atoms connected to the specified atom

Returns a `list`` of atoms that are connected to the specified atom.
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bonds.R
\name{get.connected.atom}
\alias{get.connected.atom}
\title{Get the atom connected to an atom in a bond.}
\usage{
get.connected.atom(bond, atom)
}
\arguments{
\item{bond}{A \code{jObjRef} representing an `IBond` object}

\item{atom}{A \code{jObjRef} representing an `IAtom` object}
}
\value{
A \code{jObjRef} representing an `IAtom`` object
}
\description{
This function returns the atom that is connected to a
specified in a specified bond. Note that this function assumes
2-atom bonds, mainly because the CDK does not currently
support other types of bonds
}
\seealso{
\code{\link{get.atoms}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc.R
\name{get.desc.categories}
\alias{get.desc.categories}
\title{List available descriptor categories}
\usage{
get.desc.categories()
}
\value{
A character vector listing available descriptor categories. This can be 
used in \link{get.desc.names}
}
\description{
List available descriptor categories
}
\seealso{
\link{get.desc.names}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{get.title}
\alias{get.title}
\title{Get the title of the molecule.}
\usage{
get.title(mol)
}
\arguments{
\item{mol}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}
}
\value{
A character string with the title, `NA` is no title is specified
}
\description{
Some molecules may not have a title (such as when parsing in a SMILES
with not title).
}
\seealso{
\code{\link{set.title}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc.R
\name{eval.desc}
\alias{eval.desc}
\title{Compute descriptor values for a set of molecules}
\usage{
eval.desc(molecules, which.desc, verbose = FALSE)
}
\arguments{
\item{molecules}{A `list` of molecule objects}

\item{which.desc}{A character vector listing descriptor class names}

\item{verbose}{If `TRUE`, verbose output}
}
\value{
A `data.frame` with molecules in the rows and descriptors in the columns. If
a descriptor value cannot be computed for a molecule, `NA` is returned.
}
\description{
Compute descriptor values for a set of molecules
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{set.title}
\alias{set.title}
\title{Set the title of the molecule.}
\usage{
set.title(mol, title = "")
}
\arguments{
\item{mol}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}

\item{title}{The title of the molecule as a character string. This will overwrite
any pre-existing title. The default value is an empty string.}
}
\description{
Set the title of the molecule.
}
\seealso{
\code{\link{get.title}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{get.chem.object.builder}
\alias{get.chem.object.builder}
\title{Get the default chemical object builder.}
\usage{
get.chem.object.builder()
}
\value{
An instance of \href{http://cdk.github.io/cdk/2.3/docs/api/org/openscience/cdk/silent/SilentChemObjectBuilder.html}{SilentChemObjectBuilder}
}
\description{
The CDK employs a builder design pattern to construct
instances of new chemical objects (e.g., atoms, bonds, parsers
and so on). Many methods require an instance of a builder 
object to function. While most functions in this package handle
this internally, it is useful to be able to get an instance of
a builder object when directly working with the CDK API via
`rJava`.
}
\details{
This method returns an instance of the \href{http://cdk.github.io/cdk/2.2/docs/api/org/openscience/cdk/silent/SilentChemObjectBuilder.html}{SilentChemObjectBuilder}. 
Note that this is a static object that is created at package load time, 
and the same instance is returned whenever this function is called.
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{remove.hydrogens}
\alias{remove.hydrogens}
\title{Remove explicit hydrogens.}
\usage{
remove.hydrogens(mol)
}
\arguments{
\item{mol}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}
}
\value{
A copy of the original molecule, with explicit hydrogens removed
}
\description{
Create an copy of the original structure with explicit hydrogens removed. 
Stereochemistry is updated but up and down bonds in a depiction may need 
to be recalculated. This can also be useful for descriptor calculations.
}
\seealso{
\code{\link{get.hydrogen.count}}, \code{\link{get.total.hydrogen.count}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{cdk.version}
\alias{cdk.version}
\title{Get the current CDK version used in the package.}
\usage{
cdk.version()
}
\value{
Returns a character containing the version of the CDK used in this package
}
\description{
Get the current CDK version used in the package.
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{bpdata}
\alias{bpdata}
\title{Boiling Point Data}
\format{A data frame with 277 rows and 2 columns.:
\describe{
  \item{SMILES}{Structure in SMILES format}
  \item{BP}{Boiling point in Kelvin}
  
} The names of the molecules are used as the row names.}
\usage{
bpdata
}
\description{
A dataset containing the structures and associated boiling 
points for 277 molecules, primarily alkanes and substituted alkanes.
}
\references{
Goll, E.S. and Jurs, P.C.; "Prediction of the Normal Boiling Points of
 Organic Compounds From Molecular Structures with a Computational
 Neural Network Model", \emph{J. Chem. Inf. Comput. Sci.}, 1999, 
 \emph{39}, 974-983.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{is.connected}
\alias{is.connected}
\title{Tests whether the molecule is fully connected.}
\usage{
is.connected(mol)
}
\arguments{
\item{mol}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}
}
\value{
`TRUE` if molecule is complete, `FALSE` otherwise
}
\description{
A single molecule will be represented as a 
\href{https://en.wikipedia.org/wiki/Complete_graph}{complete} graph. 
In some cases, such as for molecules in salt form, or after certain 
operations such as bond splits, the molecular graph may contained 
\href{http://mathworld.wolfram.com/DisconnectedGraph.html}{disconnected components}.
This method can be used to tested whether the molecule is complete (i.e. fully
connected).
}
\examples{
m <- parse.smiles("CC.CCCCCC.CCCC")[[1]]
is.connected(m)
}
\seealso{
\code{\link{get.largest.component}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\name{generate.formula}
\alias{generate.formula}
\title{generate.formula}
\usage{
generate.formula(
  mass,
  window = 0.01,
  elements = list(c("C", 0, 50), c("H", 0, 50), c("N", 0, 50), c("O", 0, 50), c("S", 0,
    50)),
  validation = FALSE,
  charge = 0
)
}
\arguments{
\item{mass}{Required. Mass.}

\item{window}{Optional. Default \code{0.01}}

\item{elements}{Optional. Default \code{
list(c('C', 0,50), c('H', 0,50), 
c('N', 0,50), c('O', 0,50), c('S', 0,50))}}

\item{validation}{Optional. Default \code{FALSE}}

\item{charge}{Optional. Default \code{FALSE}}
}
\description{
generate.formula
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\name{is.aromatic}
\alias{is.aromatic}
\title{Tests whether an atom is aromatic.

This assumes that the molecule containing the atom has been 
appropriately configured.}
\usage{
is.aromatic(atom)
}
\arguments{
\item{atom}{The atom to test}
}
\value{
`TRUE` is the atom is aromatic, `FALSE` otherwise
}
\description{
Tests whether an atom is aromatic.

This assumes that the molecule containing the atom has been 
appropriately configured.
}
\seealso{
\code{\link{is.aliphatic}}, \code{\link{is.in.ring}}, \code{\link{do.aromaticity}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smiles.R
\name{parse.smiles}
\alias{parse.smiles}
\title{Parse SMILES strings into molecule objects.}
\usage{
parse.smiles(smiles, kekulise = TRUE, omit.nulls = FALSE, smiles.parser = NULL)
}
\arguments{
\item{smiles}{A single SMILES string or a vector of SMILES strings}

\item{kekulise}{If set to `FALSE` disables electron checking and
allows for parsing of incorrect SMILES. If a SMILES does not parse by default, try
setting this to `FALSE` - though the resultant molecule may not have consistent
bonding. As an example, `c4ccc2c(cc1=Nc3ncccc3(Cn12))c4` will not be parsed by default
because it is missing a nitrogen. With this argument set to `FALSE` it will parse
successfully, but this is a hack to handle an incorrect SMILES}

\item{omit.nulls}{If set to `TRUE`, omits SMILES which were parsed as `NULL`}

\item{smiles.parser}{A SMILES parser object obtained from \code{\link{get.smiles.parser}}}
}
\value{
A `list` of `jobjRef`s to their corresponding CDK `IAtomContainer` 
objects. If a SMILES string could not be parsed and `omit.nulls=TRUE` it 
is omited from the output list.
}
\description{
This function parses a vector of SMILES strings to generate a list of
`IAtomContainer` objects. Note that the resultant molecule will
not have any 2D or 3D coordinates.
Note that the molecules obtained from this method will not have any
aromaticity perception (unless aromatic symbols are encountered, in which 
case the relevant atoms are automatically set to aromatic), atom typing or 
isotopic configuration done on them. This is in contrast to the 
\code{\link{load.molecules}} method. Thus, you should
perform these steps manually on the molecules.
}
\seealso{
\code{\link{get.smiles}}, \code{\link{parse.smiles}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formula.R
\name{get.mol2formula}
\alias{get.mol2formula}
\title{get.mol2formula}
\usage{
get.mol2formula(molecule, charge = 0)
}
\arguments{
\item{molecule}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}

\item{charge}{Optional. Default \code{0}}
}
\description{
get.mol2formula
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\name{get.formal.charge}
\alias{get.formal.charge}
\title{Get the formal charge on the atom.

By default the formal charge will be 0 (i.e., \code{NULL} is never returned).}
\usage{
get.formal.charge(atom)
}
\arguments{
\item{atom}{The atom to query}
}
\value{
An integer representing the formal charge
}
\description{
Get the formal charge on the atom.

By default the formal charge will be 0 (i.e., \code{NULL} is never returned).
}
\seealso{
\code{\link{get.charge}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visual.R
\name{copy.image.to.clipboard}
\alias{copy.image.to.clipboard}
\title{copy.image.to.clipboard}
\usage{
copy.image.to.clipboard(molecule, depictor = NULL)
}
\arguments{
\item{molecule}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}

\item{depictor}{Optional. Default \code{NULL}. Depictor from \code{get.depictor}}
}
\description{
generate an image and make it available to the system
clipboard.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/molecule.R
\name{Molecule}
\alias{Molecule}
\title{Operations on molecules}
\description{
Various functions to perform operations on molecules.

\code{\link{get.exact.mass}} returns the exact mass of a molecule
\code{\link{get.natural.mass}} returns the natural exact mass of a molecule
\code{\link{convert.implicit.to.explicit}} converts implicit hydrogens to 
    explicit hydrogens. This function does not return any value but rather 
    modifies the molecule object passed to it 
\code{\link{is.neutral}} returns \code{TRUE} if all atoms in the molecule have
    a formal charge of \code{0}, otherwise \code{FALSE}
}
\details{
In some cases, a molecule may not have any hydrogens (such as when read in 
from an MDL MOLfile that did not have hydrogens). In such cases,
\code{\link{convert.implicit.to.explicit}} will add implicit hydrogens and 
then convert them to explicit ones.  In addition, for such cases, make sure 
that the molecule has been typed beforehand.
}
\section{Usage}{

get.exact.mass(mol)
get.natural.mass(mol)
convert.implicit.to.explicit(mol)
is.neutral(mol)
}

\section{Arguments}{

 mol  A jobjRef representing an IAtomContainer or IMolecule object
}

\section{Value}{

 \code{\link{get.exact.mass}} returns a numeric 
 \code{\link{get.natural.mass}} returns a numeric
 \code{\link{convert.implicit.to.explicit}} has no return value
 \code{\link{is.neutral}} returns a boolean.
}

\seealso{
\code{\link{get.atoms}}, \code{\link{do.typing}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rcdk.R
\name{generate.2d.coordinates}
\alias{generate.2d.coordinates}
\title{Generate 2D coordinates for a molecule.}
\usage{
generate.2d.coordinates(mol)
}
\arguments{
\item{mol}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}
}
\value{
The input molecule, with 2D coordinates added
}
\description{
Some file formats such as SMILES do not support 2D (or 3D) coordinates
for the atoms. Other formats such as SD or MOL have support for coordinates
but may not include them. This method will generate reasonable 2D coordinates 
based purely on connectivity information, overwriting
any existing coordinates if present.
}
\details{
Note that when depicting a molecule (\code{\link{view.molecule.2d}}), 2D coordinates
are generated, but since it does not modify the input molecule, we do not have access
to the generated coordinates.
}
\seealso{
\code{\link{get.point2d}}, \code{\link{view.molecule.2d}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching.R
\name{matches}
\alias{matches}
\title{matches}
\usage{
matches(query, target, return.matches = FALSE)
}
\arguments{
\item{query}{Required. A SMARTSQuery}

\item{target}{Required. The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}

\item{return.matches}{Optional. Default \code{FALSE}}
}
\description{
matches
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/atoms.R
\name{get.charge}
\alias{get.charge}
\title{Get the charge on the atom.

This method returns the partial charge on the atom. If charges have not been set the
return value is \code{NULL}, otherwise the appropriate charge.}
\usage{
get.charge(atom)
}
\arguments{
\item{atom}{The atom to query}
}
\value{
An numeric representing the partial charge. If charges have not been set,
`NULL` is returned
}
\description{
Get the charge on the atom.

This method returns the partial charge on the atom. If charges have not been set the
return value is \code{NULL}, otherwise the appropriate charge.
}
\seealso{
\code{\link{get.formal.charge}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/props.R
\name{get.property}
\alias{get.property}
\title{Get a property value of the molecule.}
\usage{
get.property(molecule, key)
}
\arguments{
\item{molecule}{The molecule to query. Should be a `jobjRef` representing an `IAtomContainer`}

\item{key}{The property key as a character string}
}
\value{
The value of the property. If there is no property with the specified key, `NA` is returned
}
\description{
This function retrieves the value of a keyed property that has
previously been set on the molecule. Properties enable us to 
associate arbitrary pieces of data with a molecule. Such data
can be text, numeric or a Java object (represented as a `jobjRef`).
}
\examples{
mol <- parse.smiles("CC1CC(C=O)CCC1")[[1]]
set.property(mol, 'prop1', 23.45)
set.property(mol, 'prop2', 'inactive')
get.property(mol, 'prop1')
}
\seealso{
\code{\link{set.property}}, \code{\link{get.properties}}
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stereo.R
\name{get.stereocenters}
\alias{get.stereocenters}
\title{Identify which atoms are stereocenters.}
\usage{
get.stereocenters(mol)
}
\arguments{
\item{mol}{A \code{jObjRef} representing an \code{IAtomContainer}}
}
\value{
A logical vector of length equal in length to the number of atoms. The i'th element is \code{TRUE} if the i'th element is identified as a stereocenter
}
\description{
This method identifies stereocenters based on connectivity.
}
\seealso{
\code{\link{get.element.types}}, \code{\link{get.stereo.types}}
}
\author{
Rajarshi Guha \email{rajarshi.guha@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fingerprint.R
\name{get.fingerprint}
\alias{get.fingerprint}
\title{Generate molecular fingerprints}
\usage{
get.fingerprint(
  molecule,
  type = "standard",
  fp.mode = "bit",
  depth = 6,
  size = 1024,
  substructure.pattern = character(),
  circular.type = "ECFP6",
  verbose = FALSE
)
}
\arguments{
\item{molecule}{A \code{jobjRef} object to an \code{IAtomContaine}}

\item{type}{The type of fingerprint. Possible values are:
  \itemize{
\item standard - Considers paths of a given length. The default is
but can be changed. These are hashed fingerprints, with a
default length of 1024
\item extended - Similar to the standard type, but takes rings and
atomic properties into account into account
\item graph - Similar to the standard type by simply considers connectivity
\item hybridization - Similar to the standard type, but only consider hybridization state
\item maccs - The popular 166 bit MACCS keys described by MDL
\item estate - 79 bit fingerprints corresponding to the E-State atom types described by Hall and Kier
\item pubchem - 881 bit fingerprints defined by PubChem
\item kr - 4860 bit fingerprint defined by Klekota and Roth
\item shortestpath - A fingerprint based on the shortest paths between pairs of atoms and takes into account ring systems, charges etc.
\item signature - A feature,count type of fingerprint, similar in nature to circular fingerprints, but based on the signature 
descriptor
\item circular - An implementation of the ECFP6 (default) fingerprint. Other circular types can be chosen by modifying the \code{circular.type} parameter.
\item substructure - Fingerprint based on list of SMARTS pattern. By default a set of functional groups is tested.
}}

\item{fp.mode}{The style of fingerprint. Specifying "`bit`" will return a binary fingerprint,
"`raw`" returns the the original representation (usually sequence of integers) and 
"`count`" returns the fingerprint as a sequence of counts.}

\item{depth}{The search depth. This argument is ignored for the
`pubchem`, `maccs`, `kr` and `estate` fingerprints}

\item{size}{The final length of the fingerprint. 
This argument is ignored for the `pubchem`, `maccs`, `kr`, `signature`, `circular` and 
`estate` fingerprints}

\item{substructure.pattern}{List of characters containing the SMARTS pattern to match. If the an empty list is provided (default) than the functional groups substructures (default in CDK) are used.}

\item{circular.type}{Name of the circular fingerprint type that should be computed given as string. Possible values are: 'ECFP0', 'ECFP2', 'ECFP4', 'ECFP6' (default), 'FCFP0', 'FCFP2', 'FCFP4' and 'FCFP6'.}

\item{verbose}{Verbose output if \code{TRUE}}
}
\value{
an S4 object of class \code{\link{fingerprint-class}} or \code{\link{featvec-class}}, 
which can be manipulated with the fingerprint package.
}
\description{
`get.fingerprint` returns a `fingerprint` object representing molecular fingerprint of
the input molecule.
}
\examples{
## get some molecules
sp <- get.smiles.parser()
smiles <- c('CCC', 'CCN', 'CCN(C)(C)', 'c1ccccc1Cc1ccccc1','C1CCC1CC(CN(C)(C))CC(=O)CC')
mols <- parse.smiles(smiles)

## get a single fingerprint using the standard
## (hashed, path based) fingerprinter
fp <- get.fingerprint(mols[[1]])

## get MACCS keys for all the molecules
fps <- lapply(mols, get.fingerprint, type='maccs')

## get Signature fingerprint
## feature, count fingerprinter
fps <- lapply(mols, get.fingerprint, type='signature', fp.mode='raw')
## get Substructure fingerprint for functional group fragments
fps <- lapply(mols, get.fingerprint, type='substructure')

## get Substructure count fingerprint for user defined fragments
mol1 <- parse.smiles("c1ccccc1CCC")[[1]]
smarts <- c("c1ccccc1", "[CX4H3][#6]", "[CX2]#[CX2]")
fps <- get.fingerprint(mol1, type='substructure', fp.mode='count',
    substructure.pattern=smarts)

## get ECFP0 count fingerprints 
mol2 <- parse.smiles("C1=CC=CC(=C1)CCCC2=CC=CC=C2")[[1]]
fps <- get.fingerprint(mol2, type='circular', fp.mode='count', circular.type='ECFP0')
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc.R
\name{get.atomic.desc.names}
\alias{get.atomic.desc.names}
\title{Get class names for atomic descriptors}
\usage{
get.atomic.desc.names(type = "all")
}
\arguments{
\item{type}{A string indicating which class of descriptors to return. Specifying
`"all"` will return class names for all molecular descriptors. Options include
* topological
* geometrical
* hybrid
* constitutional
* protein
* electronic}
}
\value{
A character vector containing class names for atomic descriptors
}
\description{
Get class names for atomic descriptors
}
\author{
Rajarshi Guha (\email{rajarshi.guha@gmail.com})
}
